function e04uc_demo(use_grads, pause_time, rolling)
% Finds the minimum of Rosenbrock's Function:
%   F = 100*(x2-x1^2)^2 + (1-x1)^2
% or peaks function:
%   F = f(x,y) = 3*(1-x)^2*exp(-(x^2)-(y+1)^2)-10*(x/5-x^3-y^5)*exp(-x^2-y^2)-1/3*exp(-(x+1)^2-y^2)
% using NAG routine e04uc.

% Set the OpenGL renderer to use software rendering, rather than hardware
% opengl software

global constraints a bl bu nclin
global fun_change peaks run repeat_problem ret
global x spoint buttons
global xmin xmax ymin ymax
global usederivs mypause stlimit

if nargin < 3
  
  % rolling = 1 means use the full screen for plots (good for
  % rolling demo) and don't wait for key press before starting
  rolling = 0;
  
  if nargin < 2
    
    if nargin < 1
      use_grads = 1;
    end
    
    if use_grads == 0
      pause_time = 0.01;
    else
      pause_time = 1;
    end
    
  end
  
end

scrsz = get(0, 'ScreenSize');

if rolling
  figure('Name', 'Minimum of Rosenbrock''s or peaks Function using NAG Toolbox for MATLAB', ...
         'Position', [0 -35 scrsz(3)*1.0 scrsz(4)*1.0], ...
         'NumberTitle', 'off');
else
  figure('Name', 'Minimum of Rosenbrock''s or peaks Function using NAG Toolbox for MATLAB', ...
         'Position', [scrsz(1)+100 scrsz(2)+20 scrsz(3)*0.8 scrsz(4)*0.8], ...
         'NumberTitle', 'off');
end

% Run demo for the first time
first = 1;
% Initially start from Rosenbrock's function
peaks = 0;
% If function was changed, then start from the beginning
fun_change=1;
while fun_change
  fun_change=0;
  
  % Set appropriate axes (depending on a function chosen)
  if peaks==0
    objfun = 'e04uc_rosenbrock_objfun';
    xmin = -3.5;
    xmax = 4;
    ymin = -6;
    ymax = 6;
  else
    objfun = 'e04uc_peaks_objfun';
    xmin = -3;
    xmax = 3.5;
    ymin = -4;
    ymax = 3;
  end

  % Create data for contour plot of Rosenbrock's or peaks function
  px = xmin : 0.05 : xmax;
  py = ymin : 0.05 : ymax;
  pf = zeros(length(py),length(px));

  % User structure to pass to objective function.
  user = struct('plot', 0, ...      % plot progress? 1 = yes
                'xPrev', 0, ...      % x coordinate of the previous plotted point
                'yPrev', 0, ...      % y coordinate of the previous plotted point
                'myPause', 0, ...    % pause time in seconds after each point is plotted.
                'nCalls', 0, ...     % number of calls to this evaluation function
                'useGradients', 0);  %use gradients? 1 = yes

  for i = 1:length(py)
    for j = 1:length(px)
      xx = [px(j); py(i)];
      ggrad=[0; 0];
      nstate = nag_int(0);
      user.useGradients = 0;
      if peaks == 1
        [mode, pf(i,j)] = e04uc_peaks_objfun(0, 2, xx, ggrad, nstate, user);
      else
        [mode, pf(i,j)] = e04uc_rosenbrock_objfun(0, 2, xx, ggrad, nstate, user);
      end
    end
  end

  % Heights of contours required are in vector v.
  if peaks==1
    v = -6: 8;
  else
    v = [5 30 100 220 400 700 1200 2000 3000 4500 6500 9000];
  end
  [C,h] = contour(px, py, pf, v, 'Linewidth', 1);
  set(h, 'ShowText', 'on', 'TextStep', get(h,'LevelStep')*2);
  colormap('prism');
  hold on;
  hdl = findobj(gca,'Type','patch');
  for i = 1:length(hdl)
    numpoints = length(get(hdl(i),'XData'));
    set(hdl(i),'ZData',zeros(numpoints,1));
  end
  set(h,'HandleVisibility','off');
  
  % Plot the true minimum with a green circle and make it 'invisible' for
  % deletion by cla
  if peaks==1
    go = plot(0.23, -1.63, 'go', 'Linewidth', 2, 'MarkerSize', 16);
  else
    go = plot(1, 1, 'go', 'Linewidth', 2, 'MarkerSize', 16);
  end
  set(go,'HandleVisibility','off');

  if use_grads == 1
    if peaks==1
      title('NAG routine e04uc: minimizing peaks Function (gradients provided)', 'FontSize', 14);
    else
      title('NAG routine e04uc: minimizing Rosenbrock''s Function (gradients provided)', 'FontSize', 14);
    end
  else
    if peaks==1
      title('NAG routine e04uc: minimizing peaks Function (gradients not provided)', 'FontSize', 14);
    else
      title('NAG routine e04uc: minimizing Rosenbrock''s Function (gradients not provided)', 'FontSize', 14);
    end
  end

  % Add a legend
  if peaks==0
    frx = [-3.4 -1 -1 -3.4];
    fry = [-3.62 -3.62 -5.82 -5.82];
    coorx = [-3.25 -3.15 -1.95 -1.75 -1.65];
    coory = [-3.9 -4.4 -4.8 -5.2 -5.6];
  else
    frx = [-2.9 -0.3 -0.3 -2.9];
    fry = [-2.2 -2.2 -3.9 -3.9];
    coorx = [-2.8 -2.7 -1.55 -1.35 -1.25];
    coory = [-2.69 -3 -3.25 -3.5 -3.75];
  end
  lframe = fill3(frx, fry, ones(4,1), 'w');

  %text('Interpreter', 'latex',...
  %   'String', '$f(x,y) = 100(y-x^2)^2 + (1-x)^2$',...
  %    'Position', [-2.75, -0.45], 'FontSize', 12, 'Color', [1 0 0]);
  if peaks==1
    fun1 = text(-2.85, -2.39, 1, 'f(x,y) = 3*(1-x)^2*exp(-(x^2)-(y+1)^2)-10*(x/5-x^3-y^5)', 'FontSize', 12);
    fun2 = text(-2.5, -2.69, 1, '* exp(-x^2-y^2)-1/3 * exp(-(x+1)^2-y^2)', 'FontSize', 12);
    litem1 = [fun1 fun2];
  else
    litem1 = text(coorx(1), coory(1), 1, 'f(x,y) = 100(y-x^2)^2 + (1-x)^2', 'FontSize', 12);
  end
  litem2 = plot3(coorx(1), coory(2), 1.2, 'bo', 'Linewidth', 2, 'MarkerSize', 10);
  litem3 = text(coorx(2), coory(2), 1, 'Starting point','FontSize', 10);
  litem4 = plot3(coorx(1), coory(3), 1.2, 'r*', 'Linewidth', 2, 'MarkerSize', 8);
  litem5 = text(coorx(2), coory(3), 1, 'Current evaluation point', 'FontSize', 10);
  litem6 = plot3(coorx(1), coory(4), 1.2, 'b*', 'Linewidth', 2, 'MarkerSize', 8);
  litem7 = text(coorx(2), coory(4), 1, 'Previous evaluation points', 'FontSize', 10);
  litem8 = plot3(coorx(1), coory(5), 1.2, 'go', 'Linewidth', 2, 'MarkerSize', 10);
  litem9 = text(coorx(2), coory(5), 1, 'Target minimum', 'FontSize', 10);
  % Draw a green line from previous to current point
  litem10 = line([coorx(3),coorx(4)], [coory(2),coory(2)], [1,1], 'Color', [0 1 0], 'Linewidth', 2);
  litem11 = text(coorx(4), coory(2), 1, 'Search path  ', 'FontSize', 10);
  uistack(lframe,'down',2); 

  legend = [lframe litem1 litem2 litem3 litem4 litem5 litem6 litem7 litem8 litem9 litem10 litem11];
  % Make legend items 'invisible', so they would not be deleted by cla
  set(legend,'HandleVisibility','off');

  % There are no nonlinear constraints
  cjac = [];

  % There are no linear constraints at that point
  repeat_problem = 0;

  run = 1;
  % Reset the demo or Repeat problem from here
  while run
    % Delete all 'visible' graphics objects from axes
    cla;
 
    % If demo is run for the first time or Reset button was pressed,
    % linear constraints, starting point, use of derivatives and 
    % speed are set to initial values
    if repeat_problem==0
      % Set linear constraints and bounds to their initial values
      % N.B. We multiply the bounds by 1-eps so that MATLAB actually plots
      % points along the boundaries - for some reason it doesn't always
      % otherwise.
      a = [];
      bl = (1-eps)*[xmin; ymin];
      bu = (1-eps)*[xmax; ymax];
      % Delete all previous linear constraints
      constraints = [];
      nclin = 0;
      % Initial guess at solution
      x = [-2.75; 1.3];
      % Plot the starting point with a blue circle and make it 'invisible' for
      % deletion by cla
      spoint = plot(x(1), x(2), 'bo', 'Linewidth', 2, 'MarkerSize', 16);
      set(spoint,'HandleVisibility','off');
      % Set use of derivatives and speed to initial values
      usederivs = use_grads;
      mypause = pause_time;
      stlimit = 'Step Limit = 0.25';
    end
  
    if rolling ~= 1
      % If demo is run for the first time, create buttons and popups
      if first == 1
        ret = 0;
        % Create buttons and wait for them to be pressed before continuing
        hbutton = uicontrol('Position', [10 20 140 40], ...
                     'String', 'Start', 'Callback', @button_handler);
        func = uicontrol('Style', 'popup', 'String', 'Rosenbrock|peaks', ...
                     'Position', [10 200 110 30], 'TooltipString', ...
                     'Chooses the function', ...
                     'Value', 1, 'Callback', @choosefunc);
        hspeed = uicontrol('Style', 'popup', 'String', 'Slow|Medium|Fast', ...
                     'Position', [10 170 110 30], 'TooltipString', ...
                     'Chooses the speed of animation', ...
                     'Value', 1, 'Callback', @animspeed);
        ud = uicontrol('Style', 'popup', 'String', 'Use derivatives|No derivatives', ...
                     'Position', [10 140 110 30], 'TooltipString', ...
                     'Decides whether e04uc should use analytical derivatives', ...
                     'Value', 1, 'Callback', @usederivatives);
        usederivatives(ud, 0);
        steplim = uicontrol('Style', 'slider', 'Min', 0.01, 'Max', 5, ...
                     'Position', [10 93 110 20], 'TooltipString', ...
                     'Chooses the step limit', 'Value', 0.25, ...
                     'Callback', @steplimit);
        edittxt = uicontrol('Style','edit', 'String','0.25',...
                     'Position',[80 120 50 20], 'FontSize', 9, ...
                     'Callback', @editstep);        
        uicontrol('Style','text', 'String','Step limit:',...
                     'Position',[10 120 60 20], 'BackgroundColor', ...
                     [0.8,0.8,0.8], 'HorizontalAlignment', 'left', ...
                     'FontSize', 9);
        uicontrol('Style','text', 'String','0.01', 'Position', ...
                     [5 75 40 15], 'BackgroundColor', [0.8,0.8,0.8]);
        uicontrol('Style','text', 'String','5', 'Position', ...
                     [95 75 40 15], 'BackgroundColor', [0.8,0.8,0.8]);
        pbutton = uicontrol('Position', [160 20 140 40], ...
                     'String', 'Set starting point', 'Callback', @button_handler);
        lconbutton = uicontrol('Position', [310 20 140 40], ...
                     'String', 'Add linear constraint', 'Callback', @button_handler);  
        delbutton = uicontrol('Position', [460 20 140 40], ...
                     'String', 'Delete last constraint', 'Callback', @button_handler);  
        cbutton = uicontrol('Position', [610 20 140 40], ...
                     'String', 'Close', 'Callback', @button_handler);     
        buttons = [hbutton ud hspeed func pbutton lconbutton delbutton cbutton steplim edittxt];
      else
        % If "Reset" button was pressed, title, use of derivatives and 
        % speed popups are set to their initial values
        % If "Repeat problem" button was pressed, their values stay unchanged 
        if repeat_problem==0
          if peaks==1
            title('NAG routine e04uc: minimizing peaks Function (gradients provided)', 'FontSize', 14); 
          else
            title('NAG routine e04uc: minimizing Rosenbrock''s Function (gradients provided)', 'FontSize', 14); 
          end
          set(hspeed, 'Value', 1);      
          set(ud, 'Value', 1);
          set(steplim, 'Value', 0.25);
          set(edittxt, 'String', '0.25');
        end
        % If it is not the first time the demo is run, reset all buttons and popups
        set(hbutton, 'String', 'Start');
        set(buttons, 'Enable', 'On');
        set(pbutton, 'String', 'Set starting point');
      end
    
      % If there are no linear constraints, disable "Delete last
      % constraint" button
      if nclin==0
          set(delbutton, 'Enable', 'Off');
      end
    
      uiwait(gcf); % Wait until "Start" or "Close" button is pressed.  
    
      % Terminate if "Close" button was pressed
      if ret == 1
        return;
      end  
     
      % Disable other buttons once animation is underway
      set(buttons, 'Enable', 'Off');
      set(delbutton, 'Enable', 'Off');
      set(hbutton, 'Enable', 'On');
    else
      % Create a pause button
      hbutton = uicontrol('Position', [10 20 140 40], ...
                        'String', 'Pause', 'Callback', @button_handler);
    end

    % If a different function was chosen, clear the graph and start again
    % with another function
    if fun_change == 1
      run = 0;
      first = 0;
      set(h,'HandleVisibility','on');
      set(legend,'HandleVisibility','on');
      set(go,'HandleVisibility','on');
      set(spoint,'HandleVisibility','on');
    else
      % Initialise other e04uc arguments depending on number of linear constraints
      istate = zeros(2+nclin, 1, nag_int_name);
      clamda = zeros(2+nclin,1);
      r = zeros(2);
  
      user.useGradients = usederivs;
      
      if peaks==0
        frx = [1.2 3.9 3.9 1.2];
        fry = [-4.65 -4.65 -5.85 -5.85];
        coorx = 1.3;
        coory = [-4.85 -5.25 -5.65];
      else
        frx = [1.2 3.45 3.45 1.2];
        fry = [-3.2 -3.2 -3.9 -3.9];
        coorx = 1.25;
        coory = [-3.35 -3.55 -3.75];
      end
      fill3(frx, fry, ones(4,1), 'w');
            
      user.plot = 1;
      user.xPrev = x(1);
      user.yPrev = x(2);
      user.myPause = mypause;
      user.nCalls = 0;

      % Initialize the minimization routine e04uc using e04wb
      [cwsav,lwsav,iwsav,rwsav,ifail] = e04wb('e04uc');
  
      % Display e04wb ifail error messages
      if ifail==1
        errordlg('The function name supplied in routine is invalid.','e04wb ifail=1','modal');
      elseif ifail==2
        errordlg('One or more of the workspace array lengths lcwsav, llwsav, liwsav or lrwsav is too small.','e04wb ifail=2','modal');
      end

      if usederivs ~= 1
        % Specify optional parameters    
        [lwsav, iwsav, rwsav, ifail] = e04ue('Derivative Level = 0', lwsav, iwsav, rwsav);
        % Display e04ue ifail error message
        if ifail==5
          errordlg('The supplied option is invalid. Check that the keywords are neither ambiguous nor misspelt.','e04ue ifail=5','modal');
        end
      % else
        % [lwsav, iwsav, rwsav, ifail] = e04ue('Verify Level = 3', lwsav, iwsav, rwsav);          
        % [lwsav, iwsav, rwsav, ifail] = e04ue('Major Print Level = 10', lwsav, iwsav, rwsav);          
      end

      [lwsav, iwsav, rwsav, ifail] = e04ue(stlimit, lwsav, iwsav, rwsav);
      % Display e04ue ifail error message
      if ifail==5
         errordlg('The supplied option is invalid. Check that the keywords are neither ambiguous nor misspelt.','e04ue ifail=5','modal');
      end
        
      % Solve the problem using NAG routine e04uc.
      [iter, istateOut, cOut, cjacOut, clamdaOut, objf, objgrd, rOut, xOut, user, lwsavOut, iwsavOut, rwsavOut, ifail] = ...
         e04uc(a, bl, bu, 'confun', objfun, istate, cjac, clamda, r, x, lwsav, iwsav, rwsav, 'user', user);
     
      % Display e04uc ifail error messages
      if ifail==1
       errordlg('The final iterate x satisfies the first-order Kuhn-Tucker conditions to the accuracy requested, but the sequence of iterates has not yet converged. e04uc was terminated because no further improvement could be made in the merit function. x is still likely to be a solution. For more information see the documentation.','e04uc ifail=1','modal');
      elseif ifail==2
        errordlg('e04uc has terminated without finding a feasible point for the linear constraints and bounds.','e04uc ifail=2','modal');
      elseif ifail==3
        errordlg('No feasible point could be found for the nonlinear constraints.','e04uc ifail=3','modal');      
      elseif ifail==4
        errordlg('The limiting number of iterations (as determined by the optional parameter Major Iteration Limit) has been reached.','e04uc ifail=4','modal');      
      elseif ifail==6
        errordlg('x does not satisfy the first-order Kuhn-Tucker conditions and no improved point for the merit function could be found during the final linesearch.','e04uc ifail=6','modal'); 
      elseif ifail==7
        errordlg('The user-supplied derivatives of the objective function and/or nonlinear constraints appear to be incorrect.','e04uc ifail=7','modal'); 
      elseif ifail==9
        errordlg('An input parameter is invalid.','e04uc ifail=9','modal'); 
      end
  
      % Show the results  
      fill3(frx, fry, ones(4,1), 'w');
      t = sprintf('Number of evaluations of objective function = %d', user.nCalls);
      text(coorx, coory(1), 1, t, 'FontSize', 10);
      t = sprintf('Minimum point is at (%5.2f, %5.2f)', xOut(1), xOut(2));
      text(coorx, coory(2), 1, t, 'FontSize', 10);
      t = sprintf('Objective function at minimum = %13.08f', objf);
      text(coorx, coory(3), 1, t, 'FontSize', 10);

      % Do not rerun the demo unless "Reset" button is pressed
      run = 0;
      % The first run of demo is finished
      first = 0;

      if rolling ~= 1
        % Enable "Close" button once the animation is finished
        set(cbutton, 'Enable', 'On');
        % Change "Staring point" button to "Repeat problem" and enable the button
        set(pbutton, 'String', 'Repeat problem');
        set(pbutton, 'Enable', 'On');
        % Change "Start" button to "Reset"
        set(hbutton, 'String', 'Reset');
        uiwait(gcf); % Wait until "Reset" or "Close" button is pressed.
      end
    end
  end
end
end

function button_handler(hObject, eventData)
global constraints a bl bu nclin
global run repeat_problem ret
global x spoint buttons
global xmin xmax ymin ymax

label = get(hObject, 'String');
if strcmp(label, 'Close') || strcmp(label, 'Quit')
   close;
   ret = 1;
elseif strcmp(label, 'Pause')
   set(hObject, 'String', 'Resume');
   uiwait;
elseif strcmp(label, 'Start') || strcmp(label, 'Resume')
   set(hObject, 'String', 'Pause');
   uiresume;
elseif strcmp(label, 'Set starting point')
   % Store 'Enable' property of "Delete last constraint" button
   prop = get(buttons(7),'Enable');
   % Disable all the buttons
   set(buttons,'Enable','Off');
   % Delete the previous starting point
   delete(spoint);
   % Select new starting point using mouse click
   [x(1),x(2)] = ginput(1);
   while x(1) < xmin || x(1) > xmax || x(2) < ymin || x(2) > ymax
     [x(1),x(2)] = ginput(1);
   end
   % Plot the starting point with a blue circle and make it 'invisible' for
   % cla
   spoint = plot(x(1), x(2), 'bo', 'Linewidth', 2, 'MarkerSize', 16);
   set(spoint,'HandleVisibility','off');
   % Enable all the buttons
   set(buttons,'Enable','On');
   % Restore 'Enable' property of "Delete last constraint" button
   set(buttons(7),'Enable',prop);
elseif strcmp(label, 'Add linear constraint')
   % Disable all the buttons
   set(buttons,'Enable','Off');
   % Input linear constraints
   [c1,c2]=ginput(2);
   while c1(1)==c1(2) && c2(1)==c2(2)
     err = errordlg('The two points must differ to create a linear constraint.','Error','modal'); 
     uiwait(err);
     [c1,c2] = ginput(2);
   end
   nclin = nclin + 1;
   % Calculate gradient and y intercept of the new linear constraint
   a1 = (c2(1,1)-c2(2,1))/(c1(1,1)-c1(2,1));
   a2 = -1;
   a = [a;a1,a2];
   b = (c2(2,1)-a1*c1(2,1));
   % Draw the line representing the constraint
   lx = [];
   ly = [];
   % y for x=xmin
   ydown = a1*(xmin)+b;  
   % y for x=xmax
   yup = a1*xmax+b;
   if a1==0
      lx = [xmin xmax];
      ly = [ydown yup];
   else
     % x for y=ymin
     xdown = (ymin-b)/a1;
     % x for y=ymax
     xup = (ymax-b)/a1;
     if ydown >= ymin && ydown <=ymax 
       lx = [lx xmin];
       ly = [ly ydown];
     end   
     if yup >= ymin && yup <=ymax
       lx = [lx xmax];
       ly = [ly yup];
     end
     if xup >= xmin && xup <= xmax
       lx = [lx xup];
       ly = [ly ymax];
     end
     if xdown >= xmin && xdown <= xmax
       lx = [lx xdown];
       ly = [ly ymin];
     end
   end
   constr = line([lx(1),lx(2)], [ly(1),ly(2)], 'Color', [0 0 0], 'Linewidth', 2);
   % Store the new constraint
   constraints = [constraints constr];  
   points = [];
   [s1,s2]=ginput(1);
   % Update bounds on the variables and get list of points to shade not accessible area
   if -b <= a1*s1 - s2
     bl = [bl; -b];
     bu = [bu; inf];
     if a1*xmax-ymax <= -b
       points = [points; xmax,ymax];
     end
     if a1*xmax-ymin <= -b
       points = [points; xmax,ymin];
     end
     if a1*(xmin)-ymin <= -b
       points = [points; xmin,ymin];
     end
     if a1*(xmin)-ymax <= -b
       points = [points; xmin,ymax];
     end
   else
     bl = [bl; -inf];
     bu = [bu; -b];
     if -b <= a1*xmax-ymax
       points = [points; xmax,ymax];
     end
     if -b <= a1*xmax-ymin
       points = [points; xmax,ymin];
     end
     if -b <= a1*(xmin)-ymin
       points = [points; xmin,ymin];
     end
     if -b <= a1*(xmin)-ymax
       points = [points; xmin,ymax];
     end
   end
   % Sort points in the right order to create a polygon
   polx = [lx(1) lx(2)];
   poly = [ly(1) ly(2)];
   [m,n] = size(points);
   % Triangle
   if m==1
       polx = [polx points(1,1)];
       poly = [poly points(1,2)];
   end
   % Rectangle
   if m==2
     if points(1,1)==polx(2) || points(1,2)==poly(2)
       polx = [polx points(1,1) points(2,1)];
       poly = [poly points(1,2) points(2,2)];
     else
       polx = [polx points(2,1) points(1,1)];
       poly = [poly points(2,2) points(1,2)];
     end
   end
   j = 2;
   ignore=[];
   % Pentagon
   if m==3
     while length(polx) < 5
       for i=1:m
         valid = 1;
         for k=1:length(ignore)
           if i == ignore(k)
             valid = 0;
           end
         end
         if valid == 1
           if points(i,1)==polx(j) || points(i,2)==poly(j)
             polx = [polx points(i,1)];
             poly = [poly points(i,2)];
             ignore = [ignore i];
             j = j+1;
           end
         end
       end
     end
   end         
   % Create the polygon (shaded area that is not accessible due to the new constraint)
   polz = 0.1*ones(1,length(polx));
   pol=patch(polx,poly,polz,[1,1,0.5]);
   alpha(pol,0.5);
   % Store the polygon
   constraints = [constraints, pol];
   % Enable all the buttons, including "Delete last constraint"
   set(buttons,'Enable','On');
elseif strcmp(label, 'Repeat problem')
  % Set demo to be run again with the same problem
  run = 1;
  repeat_problem = 1;
 % Make constraints 'invisible', so they would not be deleted by cla
  len=length(constraints);
  for i = 1:len
    set(constraints(i),'HandleVisibility','off');
  end
  uiresume;
elseif strcmp(label, 'Reset')
   % Set demo to be run again with the initial problem
   run = 1;
   repeat_problem = 0;
   % Make constraints 'visible', so they would be deleted by cla
   set(constraints,'HandleVisibility','on');
   % Delete current starting point
   delete(spoint);
   uiresume;
elseif strcmp(label, 'Delete last constraint')
   len = length(constraints);
   for i= len: -1: (len-1)
     delete(constraints(i));
     constraints(i) = [];
   end
   a(nclin,:)=[];
   bl(nclin+2)=[];
   bu(nclin+2)=[];
   nclin = nclin - 1;
   % If there are no constraints left, disable "Delete last constraint" button
   if nclin==0
     set(hObject, 'Enable', 'Off');
   end
end
end

function usederivatives(hObject, eventData)
global usederivs peaks
val = get(hObject, 'Value');
if val == 1
  usederivs = 1;
  if peaks==1
    title('NAG routine e04uc: minimizing peaks Function (gradients provided)', 'FontSize', 14);
  else
    title('NAG routine e04uc: minimizing Rosenbrock''s Function (gradients provided)', 'FontSize', 14);
  end
elseif val == 2
  usederivs = 0;
  if peaks==1
    title('NAG routine e04uc: minimizing peaks Function (gradients not provided)', 'FontSize', 14);
  else
    title('NAG routine e04uc: minimizing Rosenbrock''s Function (gradients not provided)', 'FontSize', 14);
  end
end
end

function steplimit(hObject, eventdata, handles)
global stlimit buttons
slider_val = get(hObject,'Value');
t = sprintf('%4.2f', slider_val);
set(buttons(10),'String',t);
stlimit = sprintf('Step Limit = %4.2f', slider_val);
end

function editstep(hObject, eventdata, handles)
global stlimit buttons
slider_val = get(hObject,'String');
t = str2num(slider_val);
if (isempty(t) || t < 0.01 || t > 5)
  set(buttons(9),'Value',0.25);
  set(hObject,'String',0.25);
  stlimit = sprintf('Step Limit = 0.25');
else
  set(buttons(9),'Value',t);
  stlimit = sprintf('Step Limit = %4.2f', t);
end
end

function animspeed(hObject, eventData)
global mypause
val = get(hObject, 'Value');
if val == 1
  mypause = 0.5;
elseif val == 2
  mypause = 0.125;
else
  mypause = 0.01;
end
end

function choosefunc(hObject, eventData)
global fun_change peaks
val = get(hObject, 'Value');
if val == 1
  peaks = 0;
else
  peaks = 1;
end
fun_change=1;
uiresume;
end


