function [mode, objf, objgrd, user] = e04uc_rosenbrock_objfun(mode, n, x, objgrd, nstate, user)

% Set use_gradients to 0 if you don't want to supply
% gradients. Then e04uc needs more function evaluations.
use_gradients = user.useGradients;

objf = 100*(x(2)-x(1)*x(1))*(x(2)-x(1)*x(1)) + (1 - x(1))*(1 - x(1));

if mode == 1 || mode == 2

  % Derivatives: objgrd(i) is the derivative with
  % respect to x(i), for i = 1, 2.

  if use_gradients == 1
    objgrd(1) = 2*(1-200*x(2))*x(1) + 400*x(1)*x(1)*x(1) - 2;
    objgrd(2) = 200*(x(2) - x(1)*x(1));
  end

end

% Show progress on the graph plot, if required
if (user.plot == 1)
  oldx(1) = user.xPrev;
  oldx(2) = user.yPrev;

  % Draw a green line from previous to current point
  line([oldx(1),x(1)], [oldx(2),x(2)], 'Color', [0 1 0], 'Linewidth', 2);

  % Change the previous plotted point to blue
  plot(oldx(1), oldx(2), 'b*', 'Linewidth', 2, 'MarkerSize', 8);

  % Plot the new point in red
  plot(x(1), x(2), 'r*', 'Linewidth', 2, 'MarkerSize', 8);

  % Save the latest point in the user array for use next time around.
  user.xPrev = x(1);
  user.yPrev = x(2);

  % Display the current number of objective function evaluations
  frx = [1.2 3.9 3.9 1.2];
  fry = [-4.65 -4.65 -5.85 -5.85];
  fill3(frx, fry, ones(4,1), 'w');
  t = sprintf('Number of evaluations of objective function = %d', user.nCalls);
  t1 = text(1.3, -4.85, 1, t, 'FontSize', 10);
  t = sprintf('Current point is (%5.2f, %5.2f)', x(1), x(2));
  t2 = text(1.3, -5.25, 1, t, 'FontSize', 10);
  t = sprintf('Objective function at this point = %13.08f', objf);
  t3 = text(1.3, -5.65, 1, t, 'FontSize', 10);
  
  % Set savemovie = 1 to create a bunch of PNG files with a sequence
  % of movie frames.
  savemovie = 0;
  if savemovie == 1
    FF = getframe(gcf);
    [FX,Map] = frame2im(FF);
    filename = sprintf('frame%d.png', user.nCalls);
    imwrite(FX,filename,'png');
  end
  
  % Slow down the animation
  drawnow;
  pause(user.myPause);

  % One more function evaluation has been made
  user.nCalls = user.nCalls + 1;
  
  delete(t1);
  delete(t2);
  delete(t3);
end
