function c06fp_example

% diary c06fp_example.r
fprintf('c06fp example results\n\n');

% 3 real sequences stored as rows
m = nag_int(3);
n = nag_int(6);
x = [0.3854  0.6772  0.1138  0.6751  0.6362  0.1424;
     0.5417  0.2983  0.1181  0.7255  0.8638  0.8723;
     0.9172  0.0644  0.6037  0.6430  0.0428  0.4815];

% Transform to get Hermitian sequences
init = 'Initial';
trig = zeros(2*n,1);
[xt, trig, ifail] = c06fp(m, n, x, init, trig);
disp('Discrete Fourier transforms in Hermitian format:');
disp(xt);

for j = 1:m
  zt(j,:) = nag_herm2complex(xt(j,:));
end
title = 'Discrete Fourier transforms in full complex format:';
[ifail] = x04da('General','Non-unit', zt, title);

% Restore data by conjugation and back transform
init = 'Subsequent';
nd = double(n);
xt(1:m,floor(nd/2)+2:n) = -xt(1:m,floor(nd/2)+2:n);
[xr, trig, ifail] = c06fq(m, n, xt, init, trig);

fprintf('\n');
disp('Original data as restored by inverse transform:');
disp(xr);

% diary off

function [z] = nag_herm2complex(x);
  n = size(x,2);
  z(1) = complex(x(1));
  for j = 2:floor((n-1)/2) + 1
    z(j) = x(j) + i*x(n-j+2);
    z(n-j+2) = x(j) - i*x(n-j+2);
  end
  if (mod(n,2)==0)
    z(n/2+1) = complex(x(n/2+1));
  end
