function d02ga_example

% diary d02ga_example.r
fprintf('d02ga example results\n\n');

% For communication with fcn & display_plot.
global beta;

% Initialize variables and arrays.
u = [0, 10; 0, 1; 0, 0];
v = [0, 1; 0, 0; 1, 1];
a = 0;
b = 10;
tol = 0.001;
x = zeros(40, 1);
np = nag_int(26);
beta = double(0);
n = 3;
for i = 2:np-1
    x(i) = (double(np-i)*a+double(i-1)*b)/double(np-1);
end
x(np) = b;

for k = 1:2

  [xOut, y, npOut, ifail] = ...
    d02ga(...
           u, v, a, b, tol, @fcn, x, np);

  fprintf('Problem with beta = %1.2f \n\n',beta);
  fprintf('Solution on final mesh of %1.0f points\n\n',npOut);
  disp('  x         y_1           y_2           y_3');
  for i = 1:npOut;
    fprintf('%6.3f     %8.5f      %8.5f      %8.5f\n', x(i),y(1:3,i));
  end

  % Plot results.
  xplot = x(1:npOut);
  yplot = y(:,1:npOut); 
  if k == 1
    fig1 = figure;
    display_plot(xplot, yplot);
    % print(fig1,'-dpng','-r75','d02ga_fig1.png');
    % print(fig1,'-deps','-r75','d02ga_fig1.eps');
  else
    fig2 = figure;
    display_plot(xplot, yplot);
    % print(fig2,'-dpng','-r75','d02ga_fig2.png');
    % print(fig2,'-deps','-r75','d02ga_fig2.eps');
  end
  disp(' ');
  beta = double(beta + 0.2);
end

% diary off

function f = fcn(x,y)
  % For communication with the main routine.
  global beta;

  % Evaluate the derivatives.
  f = zeros(3,1);
  f(1) = y(2);
  f(2) = y(3);
  f(3) = -y(1)*y(3)- beta*(1.0-y(2)*y(2));

function display_plot(x, y)
  % For communication with the main routine.
  global beta;
  % Plot the three curves.
  plot(x, y(1,:), '-+', ...
       x, y(2,:), '--x', ...
       x, y(3,:), ':*');
  % Add title.
  title({['Two-point Boundary-value Problem using'],...
         ['Deferred Correction Technique (\beta = ',num2str(beta),')']});
  % Label the axes.
  xlabel('x');
  ylabel('Solution');
  % Add legend.
  legend('y','y''','y''''','Location','Best');
