function d02ke_example

% diary d02ke_example.r
fprintf('d02ke example results\n\n');

% For communication with display_plot.
global ykeep ncall xkeep pkeep;

% Set up initial values.
xpoint = [0; 0.1; 4d0^(1d0/3d0); 30; 30];
match = 0;
k = 11;
tol = 1d-4;
elam = 14;
delam = 1;
m = 5;
hmax = zeros(2,m);

ncall = 0;
ykeep = zeros(1,1);
xkeep = zeros(1,1);
pkeep = zeros(1,1);

% report outputs intermediate results.
[match, elam, delam, hmax, maxit, ifail] = ...
d02ke( ...
       xpoint, nag_int(match), @coeffn, @bdyval, ...
       nag_int(k), tol, elam, delam, hmax, 'd02kay', @report);

% Output final results.
fprintf('\n\n');
fprintf('Eigenvalue number, k = %d\n',k);
fprintf(' estimated value     = %6.3f\n',elam);
fprintf(' estimated error     = %9.2e\n',delam);
fprintf(' sensitivity to left  boundary = %6.3f\n',hmax(1,m-1));
fprintf(' sensitivity to right boundary = %6.3f\n\n', hmax(1,m));
fig1 = figure;
display_plot(xkeep,pkeep,ykeep)
% print(fig1,'-dpng','-r75','d02ke_fig1.png');
% print(fig1,'-deps','-r75','d02ke_fig1.eps');

% diary off

function [yl, yr] = bdyval(xl, xr, elam)
  % Define the boundary conditions.
  yl(1) = xl;
  yl(2) = 2;
  yl(3) = 0;
  yr(1) = 1;
  yr(2) = -sqrt(xr-elam);
  yr(3) = 0;

function [p, q, dqdl] = coeffn(x, elam, jint)
  % Compute the coefficient functions.
  p = 1;
  dqdl = 1;
  q = elam - x - 2/x/x;

function report(x, v, jint)
  % For communication with main routine.
  global ykeep ncall xkeep pkeep;

% Compute eigenfunction and its dervative at this mesh point.
  if (jint == 0)
    fprintf('\n A singular problem\n');
  end
  sqrtb = sqrt(v(1));
  if (0.5*v(3) >= log(x02am))
    r = exp(0.5*v(3));
  else
    r = 0d0;
  end
  pyp = r*sqrtb*cos(0.5*v(2));
  y = r/sqrtb*sin(0.5*v(2));

  % Output results and store them for plotting.
  ncall = ncall+1;
  ykeep(ncall,1) = y;
  xkeep(ncall,1) = x;
  pkeep(ncall,1) = pyp;

function display_plot(xkeep, pkeep, ykeep)

  % Re-order the arrays before plotting.
  n = length(xkeep);
  for i = 2:n
    if (xkeep(i-1) > xkeep(i))
      x1 = [xkeep(i-1:n);rot90(xkeep(1:i-2)')];
      p1 = [pkeep(i-1:n);rot90(pkeep(1:i-2)')];
      y1 = [ykeep(i-1:n);rot90(ykeep(1:i-2)')];
        break
    end
  end

  % Plot the two curves.
  plot(x1,y1,'-',x1,p1,'--');
  % Add title.
  title({'Regular Singular Second-order Sturm-Liouville System',...
         '11th Eigenfunction and Derivative'});
  % Label the axes.
  xlabel('x'); ylabel('Eigenfunction and Derivative');
  % Add a legend.
  legend('y','p(x)y''','Location','Best');
