function d02nm_example

% diary d02nm_example.r
fprintf('d02nm example results\n\n');

% Initialize setup setup variables and arrays.
neq    = nag_int(3);
neqmax = neq;
maxord = nag_int(5);
sdysav = maxord+1;
petzld = false;
tcrit  = 0;
hmin   = 1.0e-10;
hmax   = 10;
h0     = 0;
maxstp = nag_int(200);
mxhnil = nag_int(5);
const  = zeros(6, 1);
rwork  = zeros(50+4*neq, 1);

% BDF with Newton iterations.
[const, rwork, ifail] = d02nv( ...
                               neqmax, sdysav, maxord, 'Newton', petzld, ...
                               const, tcrit, hmin, hmax, h0, maxstp, ...
                               mxhnil, 'Average-L2', rwork);
% Numerical Jacobian
nwkjac = neqmax*(neqmax + 1);
[rwork, ifail] = d02ns( ...
                        neq, neqmax, 'Numerical', nwkjac, rwork);

% Initialize variables and arrays.
njcpvt = nag_int(1);
wkjac  = zeros(nwkjac, 1);
ydot   = zeros(neq, 1);
ysave  = zeros(neq, sdysav);
inform(1:23) = nag_int(0);
jacpvt(1) = nag_int(0);
algequ = zeros(neq, 1);

% Integrate to tout by overshooting (itask = 1).
% At monitoring stages output intermediate solutions after interpolating.
t     = 0;
tout  = 10.0;
itask = nag_int(1);
iout  = 1;
xout  = 2;
y     = [1; 0; 0];
itol  = nag_int(1);
rtol  = [1e-04];
atol  = [1e-07];

% Output header and initial results.
fprintf('\n      x           y(1)       y(2)          y(3)\n');
fprintf(' %8.3f %12.4f %12.2e %11.4f\n', t, y);

% Prepare to store results for plotting.
ncall = 1;
ykeep = y;
tkeep = [t];

% bogus initial entry values for unitialized input parameters.
irevcm = nag_int(-999); imon = irevcm; inln = irevcm; ires = irevcm;

% Ask for warning messages only.
itrace = nag_int(0);

% The reverse communication process is controlled by the value of irevcm
% (which is 0 for the final exit).
while (irevcm ~= 0)
  [t, y, ydot, rwork, inform, ysave, wkjac, jacpvt, imon, inln, ires, ...
   irevcm, ifail] = d02nm(...
                          t, tout, y, ydot, rwork, rtol, atol, itol, ...
                          inform, ysave, wkjac, jacpvt, imon, inln, ...
                          ires, irevcm, itask, itrace, 'neq', neq, ...
                          'sdysav', sdysav, 'nwkjac', nwkjac);

  % Evaluate the derivative, then use irevcm to place it.
  f(1) = -0.04*y(1) + 1.0e4*y(2)*y(3);
  f(2) =  0.04*y(1) - 1.0e4*y(2)*y(3) - 3.0e7*y(2)*y(2);
  f(3) =  3.0e7*y(2)*y(2);
  if (irevcm == 1 || irevcm == 3 )
    lrw = 50+2*neq;
    rwork(lrw+1:lrw+neq) = f;
  elseif (irevcm == 4)
    lrw = 50+neq;
    rwork(lrw+1:lrw+neq) = f;
  elseif (irevcm == 5)
    ydot = f;
  elseif (irevcm == 9 && imon == 1)
    % Extract useful information about the current step.
    tc    = rwork(19);
    hlast = rwork(15);
    hnext = rwork(16);
    nqu   = nag_int(rwork(10));

    % For low values of the independent variable, store the results
    % for plotting.
    if tc < 2.0
      ncall = ncall + 1;
      tkeep(ncall,1) = tc;
      ykeep(:,ncall) = y;
    end

    % xout is intermediate output point.
    while (xout <= tc)
      lrw = 50+neq;
      [sol, ifail] = d02xk(...
                           xout, neq, ysave, rwork(lrw+1:lrw+neq), ...
                           tc, nqu, hlast, hnext);
      % Output interpolated result, and save it for plotting.
      fprintf(' %8.3f %12.4f %12.2e %11.4f\n', xout, sol);
      ncall = ncall + 1;
      tkeep(ncall) = xout;
      ykeep(:,ncall) = sol;
      iout = iout + 1;
      xout = iout*2;
    end
  end
end

% d02ny is an integrator diagnostic routine which can be called
% after d02nm.  Output results.
[hu, h, tcur, tolsf, nst, nre, nje, nqu, nq, niter, imxer, algequ, ifail] ...
  = d02ny(...
          neq, neqmax, rwork, inform);
fprintf('\nDiagnostic information\n integration status:\n');
fprintf('   last and next step sizes = %8.5f, %8.5f\n',hu, h);
fprintf('   integration stopped at x = %8.5f\n',tcur);
fprintf(' algorithm statistics:\n');
fprintf('   number of time-steps and Newton iterations  = %5d %5d\n', ...
        nst,niter);
fprintf('   number of residual and jacobian evaluations = %5d %5d\n', ...
        nre,nje);
fprintf('   order of method last used and next to use   = %5d %5d\n', ...
        nqu,nq);
fprintf('   component with largest error                = %5d\n',imxer);
% Plot results.
fig1 = figure;
display_plot(tkeep, ykeep)
% print(fig1,'-dpng','-r75','d02nm_fig1.png');
% print(fig1,'-deps','-r75','d02nm_fig1.eps');

% diary off

function display_plot(x, y)
  % Plot one of the curves and then add the other two.
  hline1 = semilogx(x, y(1,:));
  hold on
  [haxes, hline2, hline3] = plotyy(x,y(3,:),x,y(2,:), ...
                                   'semilogx','semilogx');
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [-0.05 1.3]);
  set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [0.0:0.2:1.2]);
  set(haxes(2), 'YLim', [0.0 5e-5]);
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(2), 'YTick', [5e-6:5e-6:3.5e-5]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [0 12]);
    set(haxes(iaxis), 'XTick', [0.0001 0.001 0.01 0.1 1 10]);
  end
  set(gca, 'box', 'off');
  % Add title.
  title({'Stiff Robertson Problem (Explicit ODE):', ...
         'BDF with Newton Iterations'},'Position',[0.05,1.1]);
  % Label the x axis, and both y axes.
  xlabel('x');
  ylabel(haxes(1),'Solution (a,c)');
  ylabel(haxes(2),'Solution (b)');
  % Add a legend.
  legend({'a','c','b'},'Position',[0.45,0.45,0.15,0.15]);
  % Set some features of the three lines.
  set(hline1, 'Marker', '+','Linestyle','-','Color','red');
  set(hline2, 'Marker', 'o','Linestyle',':','Color','green');
  set(hline3, 'Marker', 'x','Linestyle','--','Color','blue');
  hold off
