function d02nu_example

% diary d02nu_example.r
fprintf('d02nu example results\n\n');

% Initialize variables and arrays for setup routine

n      = nag_int(3);
ord    = nag_int(5);
sdy    = nag_int(ord + 1);
petzld = false;
con    = zeros(6);
tcrit  = 0;
hmin   = 1.0e-10;
hmax   = 10;
h0     = 0;
maxstp = nag_int(200);
mxhnil = nag_int(5);
lrwork = 50 + 4*n;
rwork  = zeros(lrwork);

% Setup integration method using d02nv.
[const, rwork, ifail] = d02nv(...
                              n, sdy, ord, 'Newton', petzld, con, tcrit, ...
                              hmin, hmax, h0, maxstp, mxhnil, 'Average-L2', ...
                              rwork);

% Setup for sparse Jacbian using d02nu
nwkjac = nag_int(100);
njcpvt = nag_int(150);
ia     = nag_int(0);
ja     = nag_int(0);
sens   = 0;
u      = 0.1;
eta    = 1.0e-4;
lblock = true;
jacpvt = zeros(njcpvt, 1);
[jacpvt, rwork, ifail] = d02nu(...
                               n, n, 'Numerical', nwkjac, ia, ja, njcpvt, ...
                               sens, u, eta, lblock, rwork);

% Initialize variables and arrays for integration
t      = 0;
tout   = 10;
y      = [1; 0; 0];
rtol   = [0.0001];
atol   = [1e-07];
itol   = nag_int(1);
inform = zeros(23, 1, nag_int_name);
ysave  = zeros(n, sdy);
wkjac  = zeros(nwkjac, 1);
itask  = nag_int(1);
itrace = nag_int(0);

% Integrate ODE from t=0 to t=tout, no monitoring, using d02nd
[t, y, ydot, rwork, inform, ysave, wkjac, jacpvt, ifail] = ...
    d02nd(t, tout, y, rwork, rtol, atol, itol, inform, @fcn, ysave, @jac, ...
          wkjac, jacpvt, 'd02nby', itask, itrace);

fprintf('Solution y and derivative y'' at t = %7.4f is:\n',t);
fprintf('\n %10s %10s\n','y','y''');
for i=1:n
  fprintf(' %10.4f %10.4f\n',y(i),ydot(i));
end

% diary off

function [f, ires] = fcn(neq, t, y, ires)
% Evaluate derivative vector.
f = zeros(3,1);
f(1) = -0.04d0*y(1) + 1.0d4*y(2)*y(3);
f(2) =  0.04d0*y(1) - 1.0d4*y(2)*y(3) - 3.0d7*y(2)*y(2);
f(3) =                                  3.0d7*y(2)*y(2);
