function d02pc_example

% diary d02pc_example.r
fprintf('d02pc example results\n\n');

% Set initial conditions and input
method = nag_int(1);
tstart = 0;
tend   = 2*pi;
n      = nag_int(2);
errass = false;
lenwrk = nag_int(32*n);
yinit  = [0;1];
hstart = 0;
thresh = [1e-08; 1e-08];
npts = 40;
tol0 =  1.0E-3;
ygot = zeros(npts+1, 2);
tgot = zeros(npts+1, 1);
err1 = zeros(npts+1, 2);
err2 = zeros(npts+1, 2);
ymax = zeros(1, 2);

% Set control for output
tinc = (tend-tstart)/npts;
tol = 10.0*tol0;

% We run through the calculation twice with two tolerance values
for i = 1:2

  tol = tol*0.1;

  % Call setup function
  [work, ifail] = d02pv(tstart, yinit, tend, tol, thresh, method, 'Usual', ...
                        errass, lenwrk);


  tgot(1) = tstart;
  ygot(1,:) = yinit;
  twant = tstart;
  for j=1:npts
    twant = twant + tinc;
    [tgot(j+1), ygot(j+1,:), ypgot, ymax, work, ifail] = d02pc(@f, n, ...
                                            twant, ygot(j,:), ymax, work);
    err1(j+1, i) =  ygot(j+1, 1)-sin(tgot(j+1));
    err2(j+1, i) =  ygot(j+1, 2)-cos(tgot(j+1));
  end

  fprintf('\nCalculation with TOL = %8.1e:\n\n', tol);
  % d02py is a diagnostic routine.
  [fevals, stepcost, waste, stepsok, hnext, ifail] = d02py;
  fprintf('  Number of evaluations of f = %d\n', fevals);

end

% Plot results
fig1 = figure;
title('First-order ODEs using Runge-Kutta Low-order Method, Two Tolerances');
hold on;
axis([0 10 -1.2 1.2]);
xlabel('t');
ylabel('Solution (y, y'')');
plot(tgot, ygot(:, 1), '-xr');
text(ceil(tgot(npts+1)), ygot(npts+1, 1)-0.2, 'y', 'Color', 'r');
plot(tgot, ygot(:, 2), '-xg');
text(ceil(tgot(npts+1)), ygot(npts+1, 2), 'y''', 'Color', 'g');
% Plot errors with a different (log) scale
ax1 = gca;
ax2 = axes('Position',get(ax1,'Position'),...
           'XAxisLocation','bottom','YAxisLocation','right',...
           'YScale', 'log', 'Color','none','XColor','k','YColor','k');
hold on;
axis([0 10 1e-7 0.01]);
ylabel('abs(Error)');
plot(ax2, tgot, abs(err1(:, 1)), '-*b');
text(ceil(tgot(npts+1)), err1(npts+1, 1), 'y-error (tol=0.001)', 'Color', 'b');
plot(ax2, tgot, abs(err1(:, 2)), '-sm');
text(ceil(tgot(npts+1)), err1(npts+1, 2), 'y-error (tol=0.0001)', 'Color', 'm');
% print(fig1,'-dpng','-r75','d02pc_fig1.png');
% print(fig1,'-deps','-r75','d02pc_fig1.eps');
hold off

% diary off

function [yp] = f(t, y)
% Evaluate derivative vector.
yp = zeros(2, 1);
yp(1) =  y(2);
yp(2) = -y(1);
