function d03ea_example

% diary d03ea_example.r
fprintf('d03ea example results\n\n');


% This example illustrates using an interior Neumann problem to
% solve the Laplace equation in the domain bounded by two triangles.

% Stage 1: Set up phi and phid
stage1 = true;
ext = false;
dorm = false;
p = 0;
q = 0;
% Outer and inner triangles with mid-points
x = [ 3.0;  1.5;  0.0; -1.5; -3.0;  0.0;  3.0;
      3.0;
      2.0;  0.0; -2.0; -1.0;  0.0;  1.0;  2.0];
y = [ 0.0;  2.0;  4.0;  2.0;  0.0;  0.0;  0.0;
      0.0;
      1.0;  1.0;  1.0;  2.0;  3.0;  2.0;  1.0];
% Some interesting Neumann conditions
phi(1:6) = [3, 4, 4, 5, 4, 3];
phid(1:6) = [0, 0, 0, 0, 0, 0];
% Integral of solution around outer triangle
alpha = 12;
[phi, phid, alpha, ifail] = ...
    d03ea( ...
           stage1, ext, dorm, p, q, x, y, phi, phid, alpha);

% Stage 2 - solution at a set of points
stage1 = false;
phisol = zeros(401,601);
solmin = 1.e10;
solmax = -1.0e10;
i = 0;
for xx = -3:0.01:3
  i = i + 1;
  j = 0;
  xl = 3-abs(xx);
  y1 = (4/3)*xl;
  for yy = 0:0.01:4
    j = j + 1;
    if ( (yy<=y1) && ((yy<=1) || (yy>=xl)));
      % In Outer triangle and outside inner triangle
      [phi, phid, sol, ifail] = ...
      d03ea( ...
             stage1, ext, dorm, xx, yy, x, y, phi, phid, alpha);
      phisol(j,i) = sol;
      solmax = max(solmax,sol);
      solmin = min(solmin,sol);
    end
  end
end

fprintf('The maximum value of solution = %8.3f\n',solmax);
fprintf('The minimum value of solution = %8.3f\n',solmin);

% Plot Solution
fig1 = figure;
contourf(phisol,[solmin:-1,1,2:0.2:solmax]);
title('Laplace Equation Solution between two triangles');
set(gca,'XTick',[],'XTickLabel',[],'YTick',[],'YTickLabel',[]);
% print(fig1,'-dpng','-r75','d03ea_fig1.png');
% print(fig1,'-deps','-r75','d03ea_fig1.eps');

% diary off
