function d03ed_example

% diary d03ed_example.r
fprintf('d03ed example results\n\n');

ngx = nag_int(65); 
ngy = ngx; 
nn  = ngx*ngy;
lda = 4*(ngx+1)*(ngy+1)/3;
a   = zeros(lda,7);
rhs = zeros(lda, 1);
ub  = zeros(nn, 1);

maxit = nag_int(15); acc = 0.0001; iout = nag_int(0);
hx = 1/double(ngx+1);
hy = 1/double(ngy+1);
alpha = 1.7;

% Set up operator, right-hand side and initial guess for
% step-lengths hx and hy
a(1:nn,[1 3 5 7]) = 1 - 0.5*alpha;
a(1:nn,[2 6])     = 0.5*alpha;
a(1:nn,4)         = -4 + alpha;
rhs(1:nn)         = -4*hx*hy;

% Boundary conditions u(x,y=0,1) = 0
a(2:ngx-1,1:2)       = 0.0;
a(nn-ngx+2:nn-1,6:7) = 0.0;
for j = 2:ngy-1
  % Boundary condition u(x=0,y) = 0
  iy = (j-1)*ngx + 1;
  a(iy,[3 6]) = 0.0;
  % Boundary condition u(x=1,y) = 1
  iy = j*ngx;
  rhs(iy) = rhs(iy) - a(iy,5) - a(iy,2);
  a(iy,[2 5]) = 0.0;
end

% Bottom left corner
a(1,[1:3 6]) = 0.0;
% Top left corner
k = nn - ngx + 1;
a(k,[3 6:7]) = 0.0;
% Bottom right corner
% Use average value at discontinuity ( = 0.5 )
k = ngx;
rhs(k) = rhs(k) - a(k,2)*0.5 - a(k,5);
a(k,[1:2 5]) = 0.0;
% Top right corner
k = ngx*ngy;
rhs(k) = rhs(k) - a(k,2) - a(k,5);
a(k,[2 5:7]) = 0.0;
u = zeros(ngx*ngy,1);

[a, rhs, ub, resid, u, numit, ifail] = ...
d03ed(...
       ngx, ngy, a, rhs, ub, maxit, acc, iout);

 % Output solution statistics
fprintf('Number of iterations  %4d\n',numit);
fprintf('Maximum residual       %8.3e\n',resid(numit));

% Plot the solution u
fig1 = figure;
u2 = u(1:nn);
umat = reshape(u2,ngx,ngy);
x(1:ngx) = hx:hx:1-hx; y(1:ngy) = hy:hy:1-hy;
[xs,ys] = meshgrid(y,x);
mesh(xs,ys,umat);
xlabel('x'); ylabel('y'); zlabel ('u(x,y)');
title('d03ed example:  U_{xx}-\alpha U_{xy}+U_{yy}=-4, \alpha=1.7');
% print(fig1,'-dpng','-r75','d03ed_fig1.png');
% print(fig1,'-deps','-r75','d03ed_fig1.eps');

% diary off
