function d03nc_example

% diary d03nc_example.r
fprintf('d03nc example results\n\n');

% American put option with exercise price 50
kopt = nag_int(4);
x    = 50;

% Use uniform mesh of 21 stock prices [0:100] and 11 time-steps [0:0.416667]
mesh = 'U';
n    = 21;
nt   = 11;
s    = zeros(n,1);
s(n) = 100;
t    = zeros(nt,1);
t(nt) = 0.4166667;

% Parameters are not time-dependent
tdpar = [false; false; false];
r     = [0.1];
q     = [0];
sigma = [0.4];

% Keep 4 solutions in time and use backward Euler scheme.
ntkeep = nag_int(4);
alpha  = 1;

[s, t, f, theta, delta, gamma, lambda, rho, ifail] = ...
    d03nc(...
          kopt, x, mesh, s, t, tdpar, r, q, sigma, ntkeep,'alpha',alpha);

disp('         Option Values');
disp('  Stock Price  |   Time to Maturity (months)');
fprintf('%16s','|');
fprintf('%12.1f',12*(t(nt)-t(1:ntkeep))); 
fprintf('\n');
for i = 1:n
  fprintf('%12.0f%4s', s(i), '|');
  fprintf(' %12.5f', f(i,:));
  fprintf('\n');
end

fig1 = figure;
plot(s,f,s,theta(:,1),s,delta(:,1),s,gamma(:,1),s,...
     lambda(:,1),s,rho(:,1));
legend('value','theta','delta','gamma','lambda','rho');
title('Option values and greeks at 5 months to maturity');
xlabel('stock price');
ylabel('values and derivatives');

% print(fig1,'-dpng','-r75','d03nc_fig1.png');
% print(fig1,'-deps','-r75','d03nc_fig1.eps');

% diary off
