function d03nd_example

% diary d03nd_example.r
fprintf('d03nd example results\n\n');

% American 5-month call option, exercise price 50

kopt = nag_int(2);
x = 50;
ns    = 21;  nt    = 4;
s_beg = 0;   t_beg = 0;
s_end = 100; t_end = 0.125;
tmat  = 0.4166667;
tdpar = [false; false; false];
r     = [0.1];
q     = [0];
sigma = [0.4];

% Discretize s and t
ds = (s_end-s_beg)/(ns-1);
dt = (t_end-t_beg)/(nt-1);
s = [s_beg:ds:s_end];
t = [t_beg:dt:t_end];

f = zeros(ns,nt);
theta = f; delta = f; gamma = f; lambda = f; rho = f;

% Loop over times and prices
for j = 1:nt
  for i = 1:ns
    [f(i,j),theta(i,j),delta(i,j),gamma(i,j),lambda(i,j),rho(i,j),ifail] = ...
    d03nd( ...
           kopt, x, s(i), t(j), tmat, tdpar, r, q, sigma);
  end
end

% Tabulate option values only
print_greek(ns,nt,tmat,s,t,'Option Values',f);
% print_greek(ns,nt,tmat,s,t,'Theta',theta);
% print_greek(ns,nt,tmat,s,t,'Delta',delta);
% print_greek(ns,nt,tmat,s,t,'Gamma',gamma);
% print_greek(ns,nt,tmat,s,t,'Lambda',lambda);
% print_greek(ns,nt,tmat,s,t,'Rho',rho);

% plot initial and final option values and greeks 
fig1 = figure;
plot(s,f(:,1),s,theta(:,1),s,delta(:,1),s,gamma(:,1),s,lambda(:,1),s,rho(:,1));
legend('value','theta','delta','gamma','lambda','rho','Location','NorthWest');
title('Option values and greeks at 5 months to maturity');
xlabel('stock price');
ylabel('values and derivatives');
fig2 = figure;
plot(s,f(:,4),s,theta(:,4),s,delta(:,4),s,gamma(:,4),s,lambda(:,4),s,rho(:,4));
legend('value','theta','delta','gamma','lambda','rho','Location','NorthWest');
title('Option values and greeks at 3.5 months to maturity');
xlabel('stock price');
ylabel('values and derivatives');

% print(fig1,'-dpng','-r75','d03nd_fig1.png');
% print(fig1,'-deps','-r75','d03nd_fig1.eps');
% print(fig2,'-dpng','-r75','d03nd_fig2.png');
% print(fig2,'-deps','-r75','d03nd_fig2.eps');

% diary off

function print_greek(ns,nt,tmat,s,t,grname,greek)

  fprintf('\n%s\n\n',grname);
  fprintf('  Stock Price  |   Time to Maturity (months)\n');
  fprintf('%16s %12.4e%12.4e%12.4e%12.4e\n', '|', 12*(tmat-t));
  fprintf('%15s+%48s\n', '---------------', ...
          '-------------------------------------------------');
  for i = 1:ns
    fprintf('%12.4e%4s %12.4e%12.4e%12.4e%12.4e\n', s(i), '|', greek(i,:));
  end
