function d03pe_example

% diary d03pe_example.r
fprintf('d03pe example results\n\n');

% Initial conditions
ts    = 0;
dx    = 0.025;
x     = [0:dx:1];
npts  = 1 + 1/0.025;
u(1,1:npts) = exp(x);
u(2,1:npts) = sin(x);

% Input parameter values
nleft = nag_int(1);
acc   = 1e-06;
rsave = zeros(4000, 1);
isave = zeros(200, 1, nag_int_name);
itask = nag_int(1);
itrace = nag_int(0);
ind   = nag_int(0);

% Solve system at time-steps of 0.2
fprintf('%32s\n%6s    ','x','t');
fprintf('%10.4f',x(5:8:37));
fprintf('\n');

pr = false;
i = 0;
t = [0.1:0.1:1];
for tout  = t
  i = i+1;
  [ts, u, rsave, isave, ind, ifail] = ...
    d03pe(...
          ts, tout, @pdedef, @bndary, u, x, nleft, acc, ...
          rsave, isave, itask, itrace, ind);
  if pr
    fprintf('%6.1f %3s',ts,'U1:');
    fprintf('%10.4f',u(1,5:8:37));
    fprintf('\n%10s','U2:');
    fprintf('%10.4f',u(2,5:8:37));
    fprintf('\n');
  end
  pr = ~pr;
  v(:,i) = u(1,:);
  w(:,i) = u(2,:);
end

fig1 = figure;
hold on
mesh(t,x,v);
mesh(t,x,w);
xlabel('Time');
ylabel('x');
title('First-order solution using Keller box and DBF');
legend('u(1,x,t)','u(2,x,t)');
view(47,26);
hold off

% print(fig1,'-dpng','-r75','d03pe_fig1.png');
% print(fig1,'-deps','-r75','d03pe_fig1.eps');


% diary off

function [res, ires] = pdedef(npde, t, x, u, ut, ux, ires)
  if (ires == -1)
    res(1:2) = ut(1:2);
  else
    res(1) = ut(1) +   ux(1) + ux(2);
    res(2) = ut(2) + 4*ux(1) + ux(2);
  end

function [res, ires] = bndary(npde, t, ibnd, nobc, u, ut, ires)
  if (ibnd == 0)
    if (ires == -1)
      res(1) = 0;
    else
      res(1) = u(1) - (exp(t)+exp(-3*t))/2 - (sin(-3*t)-sin(t))/4;
    end
  else
    if (ires == -1)
      res(1) = 0;
    else
      res(1) = u(2) - exp(1-3*t) + exp(1+t) - (sin(1-3*t)+sin(1+t))/2;
    end
  end
