function d03pj_example

% diary d03pj_example.r
fprintf('d03pj example results\n\n');

npde   = nag_int(1);
m      = nag_int(0);
ts     = 0.0001;
tout   = 0.2;
u      = zeros(22,1);
xbkpts = [0:0.1:1];
npoly  = nag_int(2);
npts   = nag_int(21);
ncode  = nag_int(1);
xi     = [1];
rtol   = [0.0001];
atol   = rtol;
itol   = nag_int(1);
normt  = 'A';
laopt  = 'F';
algopt = zeros(30,1);
rsave  = zeros(900, 1);
isave  = zeros(24, 1, nag_int_name);
itask  = nag_int(1);
itrace = nag_int(0);
ind   = nag_int(0);
cwsav = {''; ''; ''; ''; ''; ''; ''; ''; ''; ''};
lwsav = false(100, 1);
iwsav = zeros(505, 1, nag_int_name);
rwsav = zeros(1100, 1);

t = [0.1:3.1/19:3.2];

for i_t = 1:numel(t)
  tout = t(i_t);
  [ts, u, x, rsave, isave, ind, user, cwsav, lwsav, iwsav, rwsav, ifail] = ...
  d03pj( ...
         npde, m, ts, tout, @pdedef, @bndary, u, xbkpts, ...
         npoly, npts, ncode, @odedef, xi, @uvinit, rtol, atol, itol, ...
         normt, laopt, algopt, rsave, isave, itask, itrace, ind, ...
         cwsav, lwsav, iwsav, rwsav);

  if (i_t==1)
    % with ind==1, x is not returned again
    xs = reshape(x,[7,3]);
  end
  if mod(i_t+4,6)==0
    fprintf('\nThe solution at t = %7.4f is:\n',ts);
    for j = 0:2
      fprintf('%10s%12s','x','u(x,t)');
    end
    fprintf('\n');

    us = reshape(u(1:21),[7,3]);
    for i = 1:7
      for j = 1:3
        fprintf('%12.2f%10.4f',xs(i,j),us(i,j));
      end
      fprintf('\n');
    end
  end
  v(:,i_t) = u(1:(end-1));
end

fig1 = figure;
x = reshape(xs,[21,1]);
mesh(t,x,v);
xlabel('t');
ylabel('x');
zlabel('u(x,t)');
title('Coupled Parabolic PDE/ODE using Collocation and BDF');
% print(fig1,'-dpng','-r75','d03pj_fig1.png');
% print(fig1,'-deps','-r75','d03pj_fig1.eps');

% diary off

function [p,q,r,ires,user] = pdedef(npde,t,x,nptl,u,ux,ncode,v,vdot,ires,user)
  p = zeros(npde,npde,nptl);
  q = zeros(npde,nptl);
  r = zeros(npde,nptl);

  p(1,1,:) = v(1)*v(1);
  r(1,:) = ux(1,1:nptl);
  q(1,1:nptl) = -x(1:nptl)'.*ux(1,1:nptl)*(v(1)*vdot(1));

function [beta,gamma,ires,user] = bndary(npde,t,u,ux,ncode,v,vdot,ibnd,...
                                         ires,user)
  beta = zeros(npde,1);
  gamma = zeros(npde,1);
  beta(1) = 1;
  if (ibnd == 0)
    gamma(1) = -v(1)*exp(t);
  else
    gamma(1) = -v(1)*vdot(1);
  end

function [f,ires,user] = odedef(npde,t,ncode,v,vdot,nxi,xi,ucp, ...
                                ucpx,rcp,ucpt,ucptx,ires,user)
  f = zeros(ncode,1);
  if (ires == 1)
    f(1) = vdot(1) - v(1)*ucp(1,1) - ucpx(1,1) - 1 - t;
  elseif (ires == -1)
    f(1) = vdot(1);
  end

function [u,v,user] = uvinit(npde,npts,x,ncode,user)
  u = zeros(npde,npts);
  ts = 1e-4;

  v(1) = ts;
  u = exp(ts*(1-x)) - 1;
