function d03pr_example

% diary d03pr_example.r
fprintf('d03pr example results\n\n');

npde  = nag_int(2);
ts    = 0;
npts  = 61;
u     = zeros(2,npts);
dx    = 1/(npts-1);
x     = [0:dx:1];
nleft = nag_int(1);
ncode = nag_int(0);
xi    = [];
rtol  = [5e-05];
atol  = [5e-05];
itol  = nag_int(1);
normt = 'A';
laopt = 'F';
algopt = zeros(30, 1);
remesh = true;
xfix   = [];
nrmesh = nag_int(3);
dxmesh = 0;
trmesh = 0;
ipminf = nag_int(0);
rsave  = zeros(17004, 1);
isave  = zeros(25, 1, nag_int_name);
itask  = nag_int(1);
itrace = nag_int(0);
ind    = nag_int(0);

u1sol = zeros(25,npts);
u2sol = zeros(25,npts);
xsol  = zeros(25,npts);
tsol  = zeros(25,npts);
for j = 1:25
  tout = j*0.01;
  [ts, u, x, rsave, isave, ind, ifail] = ...
  d03pr( ...
         npde, ts, tout, @pdedef, @bndary, @uvinit, u, x, nleft, ncode, ...
         @odedef, xi, rtol, atol, itol, normt, laopt, algopt, remesh, ...
         xfix, nrmesh, dxmesh, trmesh, ipminf, @monitf, rsave, isave, ...
         itask, itrace, ind, 'xratio', 1.2, 'con', 5/60);
  xsol(j,:) = x;
  tsol(j,:) = ts;
  u1sol(j,:) = u(1,:);
  u2sol(j,:) = u(2,:);

end

fprintf(' Number of time steps           = %6d\n',isave(1));
fprintf(' Number of function evaluations = %6d\n',isave(2));
fprintf(' Number of Jacobian evaluations = %6d\n',isave(3));
fprintf(' Number of iterations           = %6d\n',isave(5));

fig1=figure;
mesh(xsol,tsol,u1sol);
title('First-order system, moving mesh, u_1(x,t)');
xlabel('x');
ylabel('t');
zlabel('u_1(x,t)');
view(-30,40);

fig2=figure;
mesh(xsol,tsol,u2sol);
title('First-order system, moving mesh, u_2(x,t)');
xlabel('x');
ylabel('t');
zlabel('u_2(x,t)');
view(-30,48);

% print(fig1,'-dpng','-r75','d03pr_fig1.png');
% print(fig1,'-deps','-r75','d03pr_fig1.eps');
% print(fig2,'-dpng','-r75','d03pr_fig2.png');
% print(fig2,'-deps','-r75','d03pr_fig2.eps');

% diary off

function [res, ires] = pdedef(npde, t, x, u, udot, ux, ncode, v, vdot, ires)
  res = zeros(npde, 1);

  if (ires == -1)
    res(1) = udot(1);
    res(2) = udot(2);
  else
    res(1) = udot(1) + ux(1) + ux(2);
    res(2) = udot(2) + 4.0d0*ux(1) + ux(2);
  end


function [res, ires] = bndary(npde, t, ibnd, nobc, u, udot, ncode, v, ...
                              vdot, ires)
  res = zeros(nobc, 1);

  pp = 2*pi;
  if (ibnd == 0)
    if (ires ~= -1)
      pp1 = pp*t^2;
      res(1) = u(1) - (exp(t)+exp(-3*t))/2 - (sin(9*pp1)-sin(pp1))/4 - 2*t^2;
    end
  else
    if (ires ~= -1)
      pp1 = pp*(1-3*t)^2;
      pp2 = pp*(1+t)^2;
      e   = exp(1);
      qt  = 5*t^2-2*t + 1;
      res(1) = u(2) - e*(exp(-3*t)-exp(t)) - (sin(pp1)+sin(pp2))/2 - qt;
    end
  end


function [u, v] = uvinit(npde, npts, nxi, x, xi, ncode)
  u = zeros(npde, npts);
  v = zeros(ncode, 1);

  for i = 1:npts
    u(1,i) = exp(x(i));
    u(2,i) = x(i)^2 + sin(2*pi*x(i)^2);
  end


function [f, ires] = odedef(npde, t, ncode, v, vdot, nxi, xi, ucp, ucpx, ...
                            ucpt, ires)
  f = zeros(ncode, 1);

function [fmon] = monitf(t, npts, npde, x, u)
  fmon = zeros(npts, 1);

  for i = 2:npts - 1
    h1 = x(i) - x(i-1);
    h2 = x(i+1) - x(i);
    h3 = (x(i+1)-x(i-1))/2;
    % second derivatives ..
    d2x1 = abs(((u(1,i+1)-u(1,i))/h2-(u(1,i)-u(1,i-1))/h1)/h3);
    d2x2 = abs(((u(2,i+1)-u(2,i))/h2-(u(2,i)-u(2,i-1))/h1)/h3);
    fmon(i) = max(d2x1,d2x2);
  end
  fmon(1)    = fmon(2);
  fmon(npts) = fmon(npts-1);
