function d03ub_example

% diary d03ub_example.r
fprintf('d03ub example results\n\n');

n = [16 20 24];
b = [ 6 10 15];
delta = b./(n.*(n-1));
for j = 1:n(1)
  x(j) = (j*(j-1))*delta(1);
end
for j = 1:n(2)
  y(j) = (j*(j-1))*delta(2);
end
for j = 1:n(3)
  z(j) = (j*(j-1))*delta(3);
end

a = zeros(n);
b = a; c = a; e = a; f = a; g = a;
q = a; t = a;

aparam = 1;
it     = nag_int(1);

% Set up difference equation coefficients, source terms and
% initial approximation

% Specification for internal nodes
ii = 2:n(1)-1;
jj = 2:n(2)-1;
kk = 2:n(3)-1;
for k = kk
  a(ii,jj,k) = 2/((z(k)-z(k-1))*(z(k+1)-z(k-1)));
  g(ii,jj,k) = 2/((z(k+1)-z(k))*(z(k+1)-z(k-1)));
end
for j = jj
  b(ii,j,kk) = 2/((y(j)-y(j-1))*(y(j+1)-y(j-1)));
  f(ii,j,kk) = 2/((y(j+1)-y(j))*(y(j+1)-y(j-1)));
end
for i = ii
  c(i,jj,kk) = 2/((x(i)-x(i-1))*(x(i+1)-x(i-1)));
  e(i,jj,kk) = 2/((x(i+1)-x(i))*(x(i+1)-x(i-1)));
end
d = -a - b - c - e - f - g;

% Specification for boundary nodes
ex1 = exp((x(1)+1)/y(n(2)));
ex2 = exp((x(n(1))+1)/y(n(2)));
for j = 1:n(2);
  cy1 = cos(sqrt(2)*y(j)/y(n(2)));
  q(1,   j,:) = ex1*cy1*exp((-z(:)-1)/y(n(2)));;
  q(n(1),j,:) = ex2*cy1*exp((-z(:)-1)/y(n(2)));;
end
cy1 = cos(sqrt(2)*y(1)/y(n(2)));
cy2 = cos(sqrt(2)*y(n(2))/y(n(2)));
for i = 1:n(1);
  ex1 = exp((x(i)+1)/y(n(2)));
  q(i,1,   :) = ex1*cy1*exp((-z(:)-1)/y(n(2)));;
  q(i,n(2),:) = ex1*cy2*exp((-z(:)-1)/y(n(2)));;
end
ez1 = exp((-z(1)-1)/y(n(2)));
ez2 = exp((-z(n(3))-1)/y(n(2)));
for i = 1:n(1);
  ex1 = exp((x(i)+1)/y(n(2)));
  q(i,:,1)    = ex1*cos(sqrt(2)*y(:)/y(n(2)))*ez1;
  q(i,:,n(3)) = ex1*cos(sqrt(2)*y(:)/y(n(2)))*ez2;
end

nits = 10;
n = nag_int(n);

% Iterative loop
for it = nag_int(1:nits)
  [r] = resid(n(1),n(2),n(3),a,b,c,d,e,f,g,q,t);
  [r, ifail] =  d03ub( ...
                       n(1), n(2), n(3), a, b, c, d, e, f, g, aparam, it, r);
  t = t + r;
end

for i = 1:n(3)
  nrms(i) = norm(r(:,:,i));
end
  
fprintf('Final residual after %d iterations = %10.1e\n',nits,norm(nrms));
fprintf('\nApproximate solution is:\n\n');
for j=1:4:n(3)
  fprintf('            z = %7.4f\n    x/y',z(j));
  fprintf('%8.2f',x(1:4:n(1)));
  for i=1:4:n(2)
    fprintf('\n%8.2f',y(i));
    fprintf('%8.3f',t(1:4:n(1),i,j));
  end
  fprintf('\n\n');
end

fig1 = figure;
hold on
mesh(y,x,t(:,:,1));
mesh(y,x,t(:,:,9));
mesh(y,x,t(:,:,17));
mesh(y,x,t(:,:,24));
z1 = sprintf('z = %5.2f',z(1));
z2 = sprintf('z = %5.2f',z(9));
z3 = sprintf('z = %5.2f',z(17));
z4 = sprintf('z = %5.2f',z(24));
title({'Solution of 3D Laplace''s Equation in a Box',
       'Solutions in the xy-plane for various z values'});
xlabel('y');
ylabel('x');
zlabel('U(x,y,z=Z)');
legend(z1,z2,z3,z4);
view(-27,16);
hold off

% print(fig1,'-dpng','-r75','d03ub_fig1.png');
% print(fig1,'-deps','-r75','d03ub_fig1.eps');

% diary off

function [r] = resid(n1,n2,n3,a,b,c,d,e,f,g,q,t)
r  = zeros(n1, n2, n3);
for k = 1:n3
  for j = 1:n2
    for i = 1:n1
      if (d(i,j,k) ~= 0)
        % Seven point molecule formula
        r(i,j,k) = q(i,j,k) - a(i,j,k)*t(i,j,k-1) - b(i,j,k)*t(i,j-1,k) - ...
                   c(i,j,k)*t(i-1,j,k) - d(i,j,k)*t(i,j,k) - ...
                   e(i,j,k)*t(i+1,j,k) - f(i,j,k)*t(i,j+1,k) - ...
                   g(i,j,k)*t(i,j,k+1);
      else
        % Explicit equation
        r(i,j,k) = q(i,j,k) - t(i,j,k);
      end
    end
  end
end
