function d04bb_example

% diary d04bb_example.r
fprintf('d04bb example results\n\n');

fprintf('\nFind the derivatives of the polygamma (psi) function\n');
fprintf('using function values generated by s14ae.\n\n');
fprintf('Demonstrate the effect of successively reducing hbase.\n\n');

% Set the target location and calculate the objective value
x_0 = 0.05;
[f_0, ifail] = s14ae(x_0, nag_int(0));

% Compute the actual derivatives using s14ae for comparison
actder = zeros(3, 1);
for j=1:3
  [actder(j), ifail] = s14ae(x_0, nag_int(j));
end

der_comp = zeros(14, 3, 4);
% Attempt 4 applications, reducing hbase by factor 0.1 each time
for j=1:4
  % Generate the abscissa xval using d04bb
  hbase = 0.025*10^(-j);
  [xval] = d04bb(x_0, hbase);

  % Calculate the corresponding objective function values
  fval(11) = f_0;
  for k=[1:10,12:21]
    [fval(k), ifail] = s14ae(xval(k), nag_int(0));
  end

  % Call d04ba to calculate the derivative estimates
  [der, erest, ifail] = d04ba(xval, fval);

  % Store results in der_comp
  der_comp(:, 1, j) = hbase;
  der_comp(:, 2, j) = der;
  der_comp(:, 3, j) = erest;

end

% Display results for first 3 derivatives
for i=1:3
  fprintf('\nderivative %d calculated using s14ae: %11.4e\n', i, actder(i));
  fprintf('Derivative and error estimates for derivative %d\n', i);
  fprintf('      hbase        der(%d)      erest(%d)\n', i, i);
  for j=1:4
    fprintf('  %12.4e %12.4e %12.4e\n', der_comp(i,:,j));
  end
end

% diary off
