function d06ca_example

% diary d06ca_example.r
fprintf('d06ca example results\n\n');

imax = 20;
jmax = 20;
delta = 87;
nv = imax*jmax;

hx = 1/(imax-1);
hy = 1/(jmax-1);
rad = 0.01*delta*min(hx,hy)/2;

% Initialise the seed for the random number generator
seed = [nag_int(1762541)];
% genid and subid identify the base generator
genid = nag_int(1);
subid = nag_int(1);
% Initialise the generator to a repeatable sequence
[state, ifail] = g05kf(genid, subid, seed);

% Generate two sets of uniform random variates
[state, x1, ifail] = g05sq(nag_int(nv), 0, rad, state);
[state, x2, ifail] = g05sq(nag_int(nv), 0, 2*pi, state);

% Generate a simple uniform mesh and then distort it
% randomly within the distortion neighbourhood of each node.
coor = zeros(2, nv);
conn = zeros(3, 2*nv-1, nag_int_name);
k = 0;
ind = 0;
for j = 1:jmax
  for i = 1:imax
    k = k + 1;
    r = x1(k);
    theta = x2(k);

    if (i==1 || i==imax || j==1 || j==jmax)
      r = 0;
    end

    coor(1,k) = (i-1)*hx + r*cos(theta);
    coor(2,k) = (j-1)*hy + r*sin(theta);

    if (i<imax && j<jmax)
      ind = ind + 1;
      conn(1,ind) = k;
      conn(2,ind) = k + 1;
      conn(3,ind) = k + imax + 1;
      ind = ind + 1;
      conn(1,ind) = k;
      conn(2,ind) = k + imax + 1;
      conn(3,ind) = k + imax;
    end
  end
end

nelt = ind;

% Boundary edges
nedge = 0;
edge = zeros(3, 100, nag_int_name);
for i = 1:imax - 1
  nedge = nedge + 1;
  edge(1,nedge) = i;
  edge(2,nedge) = i + 1;
  edge(3,nedge) = 0;
end

for i = 1:jmax - 1
  nedge = nedge + 1;
  edge(1,nedge) = i*imax;
  edge(2,nedge) = (i+1)*imax;
  edge(3,nedge) = 0;
end

for i = 1:imax - 1
  nedge = nedge + 1;
  edge(1,nedge) = imax*jmax - i + 1;
  edge(2,nedge) = imax*jmax - i;
  edge(3,nedge) = 0;
end

for i = 1:jmax - 1
  nedge = nedge + 1;
  edge(1,nedge) = (jmax-i)*imax + 1;
  edge(2,nedge) = (jmax-i-1)*imax + 1;
  edge(3,nedge) = 0;
end

numfix = zeros(0, 0, nag_int_name);
itrace = nag_int(1);
nqint = nag_int(10);

% Plot original mesh
fig1 = figure;
subplot(1,2,1);
triplot(transpose(double(conn(:,1:nelt))), coor(1,:), coor(2,:));
title ('Original Mesh');

[coor, ifail] = d06ca(coor, edge(:, 1:nedge), conn(:, 1:nelt), ...
                      numfix, itrace, nqint);

fprintf('\nComplete smooth mesh characteristics:\n');
fprintf('  nv    = %d\n', nv);
fprintf('  nelt  = %d\n', nelt);

% Plot smoothed mesh
subplot(1,2,2);
triplot(transpose(double(conn(:,1:nelt))), coor(1,:), coor(2,:));
title ('Smoothed Mesh');

% print(fig1,'-dpng','-r75','d06ca_fig1.png'); 
% print(fig1,'-deps','-r75','d06ca_fig1.eps');

% diary off
