function e02bc_example

% diary e02bc_example.r
fprintf('e02bc example results\n\n');

knots = [ 1  3  3  3  4  4];
ncap = size(knots,2) + 1;
ncap7 = ncap + 7;

lamda = zeros(ncap7,1);
lamda(5:ncap+3) = knots;
lamda(ncap+4:ncap7) = 6;

% B-spline coefficients
c = zeros(ncap7,1);
c(1:ncap+3) = [10  12  13  15  22  26  24  18  14  12];

% Evaluate spline at values in lamda range
left  = nag_int(1);
right = nag_int(2);
k = 0;
for x = 0:0.2:6;
  k = k+1;
  [sl(:,k), ifail] = e02bc( ...
                            lamda, c, x, left);
  [sr(:,k), ifail] = e02bc( ...
                            lamda, c, x, right);
end
x = 0:0.2:6;
fprintf('Left hand values and derivatives\n');
fprintf('%5s%12s%12s%11s%11s\n', 'x', 'spline', '1st deriv', ...
        '2nd deriv', '3rd deriv');
sol = [ x; sl];
fprintf('%7.2f%11.4f%11.4f%11.4f%11.4f\n',sol(:,1:5:end));

fprintf('\nRight hand values and derivatives\n');
fprintf('%5s%12s%12s%11s%11s\n', 'x', 'spline', '1st deriv', ...
        '2nd deriv', '3rd deriv');
sol = [ x; sr];
fprintf('%7.2f%11.4f%11.4f%11.4f%11.4f\n', sol(:,1:5:end));

fig1 = figure;
plot(x,sl(1,:),x,sl(2,:),x,sl(3,:),x,sl(4,:));
xlabel('x');
title('Evaluation of Left-hand cubic spline and derivatives');
legend('cubic spline', '1st derivative', '2nd derivative', ...
       '3rd derivative', 'Location', 'NorthWest');
fig2 = figure;
plot(x,sr(1,:),x,sr(2,:),x,sr(3,:),x,sr(4,:));
xlabel('x');
title('Evaluation of Right-hand cubic spline and derivatives');
legend('cubic spline', '1st derivative', '2nd derivative', ...
       '3rd derivative', 'Location', 'NorthWest');
% print(fig1,'-dpng','-r75','e02bc_fig1.png');
% print(fig1,'-deps','-r75','e02bc_fig1.eps');
% print(fig2,'-dpng','-r75','e02bc_fig2.png');
% print(fig2,'-deps','-r75','e02bc_fig2.eps');

% diary off
