function e02dd_example

% diary e02dd_example.r
fprintf('e02dd example results\n\n');

% Data to fit
d = [11.16   1.24  22.15; 
     12.85   3.06  22.11; 
     19.85  10.72   7.97; 
     19.72   1.39  16.83; 
     15.91   7.74  15.30; 
      0.00  20.00  34.60; 
     20.87  20.00   5.74; 
      3.45  12.78  41.24; 
     14.26  17.87  10.74; 
     17.43   3.46  18.60; 
     22.80  12.39   5.47; 
      7.58   1.98  29.87; 
     25.00  11.87   4.40; 
      0.00   0.00  58.20; 
      9.66  20.00   4.73; 
      5.22  14.66  40.36; 
     17.25  19.57   6.43; 
     25.00   3.87   8.74; 
     12.13  10.79  13.71; 
     22.23   6.21  10.25; 
     11.52   8.53  15.74; 
     15.20   0.00  21.60; 
      7.54  10.69  19.31; 
     17.32  13.78  12.11; 
      2.14  15.03  53.10; 
      0.51   8.37  49.43; 
     22.69  19.63   3.25; 
      5.47  17.13  28.63; 
     21.67  14.36   5.52; 
      3.31   0.33  44.08];
x = d(:,1);
y = d(:,2);
f = d(:,3);
w = ones(size(x));

start = 'C';
s     = 10;
nx    = nag_int(0);
lamda = zeros(14,1);
ny    = nag_int(0);
mu    = zeros(14,1);
wrk   = zeros(11016, 1);
[nx, lamda, ny, mu, c, fp, rank, wrk, ifail] = ...
e02dd( ...
       start, x, y, f, w, s, nx, lamda, ny, mu, wrk);

% Print details of spline
fprintf('\nCalling with smoothing factor S = %5.2f\n', s);
fprintf('Rank deficiency = %4d\n\n',(nx-4)*(ny-4)-rank);
fprintf('Knots:   lamda      mu\n');
for j = 4:max(nx,ny)-3
  if j<=min(nx,ny)-3
    fprintf('%4d%10.4f%10.4f\n', j, lamda(j), mu(j));
  elseif j<=nx-3
    fprintf('%4d%10.4f\n', j, lamda(j));
  else
    fprintf('%4d%20.4f\n', j, mu(j));
  end
end

cp = c(1:(ny-4)*(nx-4));
cp = reshape(cp,[ny-4,nx-4]);
fprintf('\nB-spline coefficients:\n');
disp(cp);

fprintf('Weighted sum of squared residuals = %7.4f\n', fp);
if fp==0
  fprintf('(The spline is an interpolating spline)\n');
elseif nx==8 && ny==8
  fprintf('(The spline is the weighted least-squares bi-cubic polynomial)\n');
end
fprintf('\n');

% Evaluate spline on mesh
mx = [3:21];
my = [2:17];
[ff, ifail] = e02df( ...
                     mx, my, lamda(1:nx), mu(1:ny), c);

fig1 = figure;
ff = reshape(ff,[16,19]);
meshc(mx,my,ff);
xlabel('x');
ylabel('y');
title('Least-squares bi-cubic spline fit of scattered data');
view(32,40);
% print(fig1,'-dpng','-r75','e02dd_fig1.png');
% print(fig1,'-deps','-r75','e02dd_fig1.eps');

% diary off
