function e04gd_example

% diary e04gd_example.r
fprintf('e04gd example results\n\n');

% Model fitting data
m = nag_int(15);
y = [ 0.14, 0.18, 0.22, 0.25, 0.29,...
      0.32, 0.35, 0.39, 0.37, 0.58,...
      0.73, 0.96, 1.34, 2.10, 4.39];
t = [ 1.0, 15.0, 1.0;
      2.0, 14.0, 2.0;
      3.0, 13.0, 3.0;
      4.0, 12.0, 4.0;
      5.0, 11.0, 5.0;
      6.0, 10.0, 6.0;
      7.0,  9.0, 7.0;
      8.0,  8.0, 8.0;
      9.0,  7.0, 7.0;
     10.0,  6.0, 6.0;
     11.0,  5.0, 5.0;
     12.0,  4.0, 4.0;
     13.0,  3.0, 3.0;
     14.0,  2.0, 2.0;
     15.0,  1.0, 1.0];

% Input parameters
xtol   = sqrt(x02aj());
n = nag_int(3);
x = [0.5; 1; 1.5];
user = {y; t};

[x, fsumsq, fvec, fjac, s, v, niter, nf, user, ifail] = ...
    e04gd( ...
           m, @lsqfun, @lsqmon, xtol, x, 'user', user);

fprintf('Best fit model parameters are:\n');
for i = 1:n
  fprintf('        x_%d = %10.3f\n',i,x(i));
end
fprintf('\nResiduals for observed data:\n');
fprintf('  %8.4f  %8.4f  %8.4f  %8.4f  %8.4f\n',fvec);
fprintf('\nSum of squares of residuals    = %7.4f\n',fsumsq);
fprintf('Number of iterations           = %7d\n',niter);
fprintf('Number of function evaluations = %7d\n',nf);

% diary off

function [iflag, fvecc, fjacc, user] = lsqfun(iflag, m, n, xc, ljc, user)
  y = user{1};
  t = user{2};

  fvecc = zeros(m, 1);
  fjacc = zeros(ljc, n);

  for i = 1:double(m)
    denom = xc(2)*t(i,2) + xc(3)*t(i,3);
    if (iflag ~= 1)
      fvecc(i) = xc(1) + t(i,1)/denom - y(i);
    end
    if (iflag ~= 0)
      fjacc(i,1) = 1;
      dummy = -1/(denom*denom);
      fjacc(i,2) = t(i,1)*t(i,2)*dummy;
      fjacc(i,3) = t(i,1)*t(i,3)*dummy;
    end
  end

function [user] = ...
    lsqmon(m, n, xc, fvecc, fjacc, ljc, s, igrade, niter, nf, user)
