function e04kd_example

% diary e04kd_example.r
fprintf('e04kd example results\n\n');

global monitoring;
monitoring = false;

eta     = 0.5;
ibound  = nag_int(0);
bl      = [1; -2;-1000000; 1];
bu      = [3;  0; 1000000; 3];
x       = [3; -1;       0; 1];
lh      = nag_int(6);
iw(1:2) = nag_int(0);
w       = zeros(34,1);

% Catch warnings and assume ifail=3,5 gives a good estimate
wstat = warning();
warning('OFF');
[bl, bu, x, hesl, hesd, istate, f, g, iw, w, ifail] = ...
  e04kd(...
        @funct, @monit, eta, ibound, bl, bu, x, lh, iw, w);
if (ifail == 0 || ifail == 5 | ifail == 3)
  fprintf('\nMinimum found at x: ');
  fprintf(' %9.4f',x);
  fprintf('\nMinimum value     :  %9.4f\n\n',f);
else   
  fprintf('\n Error: e04kd returns ifail = %d\n',ifail);
end
warning(wstat);

% diary off

function [iflag, fc, gc] = funct(iflag, n, xc)
  gc = zeros(n, 1);
  fc = 0;
  x1 = xc(1) + 10*xc(2);
  x2 = xc(3) -    xc(4);
  x3 = xc(2) -  2*xc(3);
  x4 = xc(1) -    xc(4);
  if (iflag ~= 1)
    fc = x1^2 + 5*x2^2 + x3^4 + 10*x4^4;
  end
  gc(1) =   2*x1 + 40*x4^3;
  gc(2) =  20*x1 +  4*x3^3;
  gc(3) =  10*x2 -  8*x3^3;
  gc(4) = -10*x2 - 40*x4^3;


function [] = monit(n, xc, fc, gc, istate, gpjnrm, cond, posdef, niter, nf)

global monitoring;
if (monitoring)
  fprintf('\n Itn     Fn evals         Fn value     Norm of proj gradient\n');
  fprintf(' %3d      %5d    %15.4f      %13.4f\n', niter, nf, fc, gpjnrm);
  fprintf('\n J           X(J)           G(J)         Status\n');
  for j = 1:n
    isj = istate(j);
    if (isj > 0)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Free');
    elseif (isj == -1)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Upper Bound');
    elseif (isj == -2)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Lower Bound');
    elseif (isj == -3)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Constant');
    end
  end
  if (cond ~= 0)
    if (cond > 1.0d6)
      fprintf('\nEst. condition number of projected Hessian > 10^6\n');
    else
      fprintf('\nEst. condition number of projected Hessian = %10.2f\n', cond);
    end
    if ( not(posdef) )
      % The following statement is included so that this monit can be used
      % in conjunction with either of the functions e04kd or e04lb
      fprintf('\nProjected Hessian matrix is not positive definite\n');
    end
  end
end
