function e04lb_example

% diary e04lb_example.r
fprintf('e04lb example results\n\n');

global monitoring;
monitoring = false;

bl = [ 1; -2; -1000000; 1];
bu = [ 3;  0;  1000000; 3];
x  = [ 3; -1;        0; 1];
ibound  = nag_int(0);
lh      = nag_int(6);
iw(1:2) = nag_int(0);
w       = zeros(34,1);
% Catch warnings and assume ifail=3,5 gives a good estimate
wstat = warning();
warning('OFF');
[bl, bu, x, hesl, hesd, istate, f, g, iw, w, ifail] = ...
  e04lb(@funct, @hess, @monit, ibound, bl, bu, x, lh, iw, w);
warning(wstat);
if (ifail == 0 || ifail == 5 | ifail == 3)
  fprintf('\nMinimum found at x: ');
  fprintf(' %9.4f',x);
  fprintf('\nGradients at x,  g: ');
  fprintf(' %9.4f',g);
  fprintf('\nMinimum value     :  %9.4f\n\n',f);
else   
  fprintf('\n Error: e04lb returns ifail = %d\n',ifail);
end

% diary off

function [iflag, fc, gc] = funct(iflag, n, xc)
  gc = zeros(n, 1);
  fc = 0;
  x1 = xc(1) + 10*xc(2);
  x2 = xc(3) -    xc(4);
  x3 = xc(2) -  2*xc(3);
  x4 = xc(1) -    xc(4);
  fc = x1^2 + 5*x2^2 + x3^4 + 10*x4^4;
  gc(1) =   2*x1 + 40*x4^3;
  gc(2) =  20*x1 +  4*x3^3;
  gc(3) =  10*x2 -  8*x3^3;
  gc(4) = -10*x2 - 40*x4^3;


function [iflag, fhesl, fhesd] = hess(iflag, n, xc, lh, fhesd)
  fhesl = zeros(lh, 1);

  x3 = xc(2) -  2*xc(3);
  x4 = xc(1) -    xc(4);
  fhesd(1) =    2 + 120*x4^2;
  fhesd(2) =  200 +  12*x3^2;
  fhesd(3) =   10 +  48*x3^2;
  fhesd(4) =   10 + 120*x4^2;
  fhesl(1) =   20;
  fhesl(2) =    0;
  fhesl(3) =  -24*x3^2;
  fhesl(4) = -120*x4^2;
  fhesl(5) =    0;
  fhesl(6) =  -10;


function [] = monit(n, xc, fc, gc, istate, gpjnrm, cond, posdef, niter, nf)

global monitoring;
if (monitoring)
  fprintf('\n Itn     Fn evals         Fn value     Norm of proj gradient\n');
  fprintf(' %3d      %5d    %15.4f      %13.4f\n', niter, nf, fc, gpjnrm);
  fprintf('\n j           x(j)           g(j)         Status\n');
  for j = 1:double(n)
    isj = istate(j);
    if (isj > 0)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Free');
    elseif (isj == -1)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Upper Bound');
    elseif (isj == -2)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Lower Bound');
    elseif (isj == -3)
      fprintf('%2d %16.4f%15.4f   %s\n', j, xc(j), gc(j), '    Constant');
    end
  end
  if (cond ~= 0.0d0)
    if (cond > 1.0d6)
      fprintf('\nEst. condition number of projected Hessian > 10^6\n');
    else
      fprintf('\nEst. condition number of projected Hessian = %10.2f\n', cond);
    end
    if ( not(posdef) )
%     The following statement is included so that this MONIT
%     can be used in conjunction with either of the functions
%     e04kd or e04lb
      fprintf('\nProjected Hessian matrix is not positive definite\n');
    end
  end
end
