function e04us_example

% diary e04us_example.r
fprintf('e04us example results\n\n');

% Hoxk and Schittkowski problem specifications
m   = 44;
n   = 2;
n_l = 1;
n_n = 1;
a  = [1, 1];
bl = [0.4;   -4;    1;    0];
bu = [1e25;  1e25;  1e25; 1e25];
y  = [0.49;  0.49; 0.48; 0.47; 0.48; 0.47; 0.46; 0.46; 0.45; 0.43; 0.45;
      0.43;  0.43; 0.44; 0.43; 0.43; 0.46; 0.45; 0.42; 0.42; 0.43; 0.41;
      0.41;  0.40; 0.42; 0.40; 0.40; 0.41; 0.40; 0.41; 0.41; 0.40; 0.40;
      0.4;   0.38; 0.41; 0.40; 0.40; 0.41; 0.38; 0.40; 0.40; 0.39; 0.39];

istate = zeros(4, 1, nag_int_name);
cjac   = zeros(1,n);
fjac   = zeros(m,n);
clamda = zeros(n+n_l+n_n,1);
r      = zeros(n,n);

% Initialize
x      = [0.4;  0];
[cwsav,lwsav,iwsav,rwsav,ifail] = e04wb('e04us');
[iter, istate, c, cjac, f, fjac, clamda, objf, r, x, user, ...
 lwsav, iwsav, rwsav, ifail] = ...
     e04us(...
           a, bl, bu, y, @confun, @objfun, istate, cjac, ...
           fjac, clamda, r, x, lwsav, iwsav, rwsav);

fprintf('Minimum value     :  %9.4f\n\n',objf);
fprintf('Found after %3d iterations at x:\n     ',iter);
fprintf(' %9.4f',x);
fprintf('\nNonlinear contraints at x:\n   ');
fprintf(' %11.3e',c);
fprintf('\n');

% diary off

function [mode, c, cjac, user] = ...
    confun(mode, ncnln, n, ldcj, needc, x, cjac, nstate, user)
  c = zeros(ncnln, 1);

  if (nstate == 1)
%   first call to confun. set all jacobian elements to zero.
%   note that this will only work when 'derivative level = 3'
%   (the default; see section 11.2).
    cjac = zeros(ncnln, n);
  end
  if (needc(1) > 0)
    if (mode == 0 || mode == 2)
      c(1) = -0.09 - x(1)*x(2) + 0.49*x(2);
    end
    if (mode == 1 || mode == 2)
      cjac(1,1) = -x(2);
      cjac(1,2) = -x(1) + 0.49;
    end
  end


function [mode, f, fjac, user] = ...
      objfun(mode, m, n, ldfj, needfi, x, fjac, nstate, user)
  f = zeros(m, 1);

  a = [ 8,  8, 10, 10, 10, 10, 12, 12, 12, 12, 14, ...
       14, 14, 16, 16, 16, 18, 18, 20, 20, 20, 22, ...
       22, 22, 24, 24, 24, 26, 26, 26, 28, 28, 30, ...
       30, 30, 32, 32, 34, 36, 36, 38, 38, 40, 42];

  for i = 1:double(m)
    temp = exp(-x(2)*(a(i)-8));
    if (mode  ==  0 || mode  ==  2)
      f(i) = x(1) + (0.49-x(1))*temp;
    end
    if (mode  ==  1 || mode  ==  2)
      fjac(i,1) = 1 - temp;
      fjac(i,2) = -(0.49-x(1))*(a(i)-8)*temp;
    end
  end
