function f08bf_example

% diary f08bf_example.r
fprintf('f08bf example results\n\n');

a = [-0.09,  0.14, -0.46,  0.68,  1.29;
     -1.56,  0.2,   0.29,  1.09,  0.51;
     -1.48, -0.43,  0.89, -0.71, -0.96;
     -1.09,  0.84,  0.77,  2.11, -1.27;
      0.08,  0.55, -1.13,  0.14,  1.74;
     -1.59, -0.72,  1.06,  1.24,  0.34];

b = [ 7.4,  2.7;
      4.2, -3.0;
     -8.3, -9.6;
      1.8,  1.1;
      8.6,  4.0;
      2.1, -5.7];

jpvt = [nag_int(0);0;0;0;0];


% Compute the QR factorization of a
[a, jpvt, tau, info] = f08bf( ...
			      a, jpvt);

% Compute C = (C1) = (Q^T)*b, storing the result in c
%             (C2)
[c, info] = f08ag( ...
		   'Left', 'Transpose', a, tau, b);

% Choose tol to reflect the relative accuracy of the input data
tol = 0.01;

% Determine and print the rank, k, of r relative to tol
k = find(abs(diag(a)) <= tol*abs(a(1,1)));
if numel(k) == 0
  k = numel(diag(a));
else
  k = k(1)-1;
end

fprintf('Tolerance used to estimate the rank of a\n     %11.2e\n', tol);
fprintf('Estimated rank of a\n        %d\n', k);

% Compute least-squares solution by backsubstitution in r(1:k, 1:k)*c = c1
c1 = zeros(5, 2);
c1(1:k, :) = inv(triu(a(1:k,1:k)))*c(1:k,:);

% Compute estimates of the square roots of the residual sums of
% squares (2-norm of each of the columns of C2)
rnorm = [norm(c(k+1:6,1)), norm(c(k+1:6,2))];

% Permute the least-squares solutions stored in c1 to give x = p*y
x = zeros(5, 2);
for i=1:5
   x(jpvt(i), :) = c1(i, :);
end
fprintf('\nLeast-squares solution(s)\n');
disp(x);
fprintf('Square root(s) of the residual sum(s) of squares\n');
disp(rnorm);

% diary off
