function f08kf_example

% diary f08kf_example.r
fprintf('f08kf example results\n\n');

% Two cases of performing Singular Value Decomposition
% Case 1: m > n
ex1;
% Case 2: m < n
ex2;

% diary off

function ex1

  m = nag_int(6);
  n = nag_int(4);
  a = [-0.57  -1.28  -0.39   0.25;
       -1.93   1.08  -0.31  -2.14;
        2.30   0.24   0.40  -0.35;
       -1.93   0.64  -0.66   0.08;
        0.15   0.30   0.15  -2.13;
       -0.02   1.03  -1.43   0.50];

  % Reduce A to bidiagonal form
  [B, d, e, tauq, taup, info] =  ...
  f08ke(a);

  % Form P^T explicitly (n-by-n, k = m)
  vect = 'P';
  [PT, info] = f08kf( ...
                      vect, m, B, taup, 'm', n, 'n', n);
  % Form Q explicitly (m-by-n, k = n)
  vect = 'Q';
  [Q, info] = f08kf( ...
                     vect, n, B, tauq, 'm', m, 'n', n);
  
  % Compute SVD of A using its bidiagonal form.
  c = [];
  uplo = 'Upper';
  [s, ~, VT, U, c, info] = f08me( ...
                                  uplo, d, e, PT, Q, c);

  disp('Example 1: singular values');
  disp(s(1:n)');
  disp('Example 1: right singular vectors, by row');
  disp(VT(1:n,:));
  disp('Example 1: left singular vectors, by column');
  disp(U);

function ex2
  m = nag_int(4);
  n = nag_int(6);
  a = [-5.42   3.28  -3.68   0.27   2.06   0.46;
       -1.65  -3.40  -3.20  -1.03  -4.06  -0.01;
       -0.37   2.35   1.90   4.31  -1.76   1.13;
       -3.15  -0.11   1.99  -2.70   0.26   4.50];

  % Reduce A to bidiagonal form
  [B, d, e, tauq, taup, info] =  ...
  f08ke(a);

  % Form P^T explicitly (m-by-n, k = m)
  vect = 'P';
  [PT, info] = f08kf( ...
                      vect, m, B, taup, 'm', m, 'n', n);
  % Form Q explicitly (m-by-m, k = n)
  vect = 'Q';
  [Q, info] = f08kf( ...
                     vect, n, B, tauq, 'm', m, 'n', m);
  
  % Compute SVD of A using its bidiagonal form.
  c = [];
  uplo = 'Lower';
  [s, ~, VT, U, c, info] = f08me( ...
                                  uplo, d, e, PT, Q, c, 'n', m);

  disp('Example 2: singular values');
  disp(s(1:m)');
  disp('Example 2: right singular vectors, by row');
  disp(VT);
  disp('Example 2: left singular vectors, by column');
  disp(U(:,1:m));
