function f08kh_example

% diary f08kh_example.r
fprintf('f08kh example results\n\n');

% General 6-by-4 real matrix A
m = nag_int(6);
n = nag_int(4);
a = [2.27, -1.54,  1.15, -1.94;
     0.28, -1.67,  0.94, -0.78;
    -0.48, -3.09,  0.99, -0.21;
     1.07,  1.22,  0.79,  0.63;
    -2.35,  2.93, -1.45,  2.30;
     0.62, -7.39,  1.03, -2.57];

% Compute the SVD of A (A = U*S*V^T, m >= n)
joba = 'estimated condition number';
jobu = 'U required';
jobv = 'V required';
jobr = 'restricted range';
jobt = 'No entropy test';
jobp = 'No perturbations';
[~, s, u, v, work, iwork, info] = ...
  f08kh( ...
         joba, jobu, jobv, jobr, jobt, jobp, a);

% Approximate error bound for S, s(1) = norm(A).
eps = x02aj;
serrbd = eps*s(1);

% Print solution
if (abs(work(1)-work(2)) < 2*eps)
  % No scaling required
  fprintf('Singular values:\n');
  disp(transpose(s));
else
  fprintf('Scaled singular values:\n');
  disp(transpose(s));
  fprintf('\nFor true singular values, multiply by a/b\n');
  fprintf('   where a=%13.5e and b=%13.5e.\n', work(1), work(2));
end

[ifail] = x04ca( ...
                 'Gen', ' ', u, 'Left singular vectors');
fprintf('\n');
[ifail] = x04ca( ...
                 'Gen', ' ', v, 'Right singular vectors');

% Estimate reciprocal condition numbers for S
[rcondu, info] = f08fl( ...
                        'Left', m, n, s);
[rcondv, info] = f08fl( ...
                        'Right', m, n, s);

% display approximate error bounds
fprintf('\nEstimate of the condition number of column equilibrated A\n');
fprintf('%11.1e\n', work(3));
fprintf('\nError estimate  for S:\n');
fprintf('%11.1e\n', serrbd);
fprintf('\nError estimates for U:\n');
fprintf('%11.1e ',serrbd./rcondu);
fprintf('\n\nError estimates for V:\n');
fprintf('%11.1e ',serrbd./rcondv);
fprintf('\n');

% diary off
