function f08wb_example

% diary f08wb_example.r
fprintf('f08wb example results\n\n');

% Generalized eigenvalues and right eigenvectors of (A, B):
n = 4;
a = [3.9, 12.5, -34.5, -0.5;
     4.3, 21.5, -47.5,  7.5;
     4.3, 21.5, -43.5,  3.5;
     4.4, 26,   -46,    6];
b = [1,    2,    -3,    1;
     1,    3,    -5,    4;
     1,    3,    -4,    3;
     1,    3,    -4,    4];

balanc = 'Balance';
jobvl = 'No vectors (left)';
jobvr = 'Vectors (right)';
sense = 'Both reciprocal condition numbers';
[~, ~, alphar, alphai, beta, ~, VR, ~, ~, ~, ~, abnrm, bbnrm, ...
	     rconde, rcondv, info] = ...
  f08wb( ...
	 balanc, jobvl, jobvr, sense, a, b);

epsilon = x02aj;
small   = x02am;
absnrm  = sqrt(abnrm^2+bbnrm^2);
tol     = epsilon*absnrm;

for j=1:n

  % display information on the jth eigenvalue
  if (abs(alphar(j)) + abs(alphai(j)))*small >= abs(beta(j))
    fprintf('\n%4d: Eigenvalue is numerically infinite or undetermined\n',j);
    fprintf('%4d: alphar = %11.4e, alphai = %11.4e, beta = %11.4e\n', ...
	    j, alphar(j), alphai(j), beta(j));
  else
    fprintf('\n%4d: Eigenvalue =  ', j);
    if alphai(j)==0
      fprintf('%10.4e\n',alphar(j)/beta(j));
    elseif alphai(j)>=0
      fprintf('%10.4e + %10.4ei\n',alphar(j)/beta(j), alphai(j)/beta(j));
    else
      fprintf('%10.4e - %10.4ei\n',alphar(j)/beta(j), -alphai(j)/beta(j));
    end
  end

  if rconde(j) > 0
    fprintf('\n      Condition number            = %8.1e\n', 1/rconde(j));
    fprintf('      Error bound                 = %8.1e\n', tol/rconde(j));
  else
    fprintf('\n      Reciprocal condition number = %8.1e\n', rconde(j));
    fprintf('      Error bound is infinite\n');
  end

  fprintf('\n      Eigenvector:\n');
  if alphai(j) == 0
    fprintf('%30.4e\n',VR(:, j));
  else
    if alphai(j) > 0
      k = j;
    else   
      k = j-1;
    end
    for l = 1:n
      if VR(l,k+1)>=0
        fprintf('%30.4e + %10.4ei\n',VR(l, k), VR(l, k+1));
      else
        fprintf('%30.4e - %10.4ei\n',VR(l, k), -VR(l, k+1));
      end
    end
  end

  if rcondv(j) > 0
    fprintf('\n      Condition number            = %8.1e\n', 1/rcondv(j));
    fprintf('      Error bound                 = %8.1e\n', tol/rcondv(j));
  else
    fprintf('\n      Reciprocal condition number = %8.1e\n', rcondv(j));
    fprintf('      Error bound is infinite\n');
  end
end

% diary off
