function f08yx_example

% diary f08yx_example.r
fprintf('f08yx example results\n\n');

% Generalized eigenvalues of matrix pair (A,B) , where
a = [ 1.0+3.0i  1.0+4.0i  1.0+5.0i   1.0+6.0i;
      2.0+2.0i  4.0+3.0i  8.0+4.0i  16.0+5.0i;
      3.0+1.0i  9.0+2.0i 27.0+3.0i  81.0+4.0i;
      4.0+0.0i 16.0+1.0i 64.0+2.0i 256.0+3.0i];
b = [ 1.0+0.0i  2.0+1.0i  3.0+2.0i   4.0+3.0i;
      1.0+1.0i  4.0+2.0i  9.0+3.0i  16.0+4.0i;
      1.0+2.0i  8.0+3.0i 27.0+4.0i  64.0+5.0i;
      1.0+3.0i 16.0+4.0i 81.0+5.0i 256.0+6.0i];

% Balance matrix pair
job = 'B';
[a, b, ilo, ihi, lscale, rscale, info] = ...
  f08wv(job, a, b);
bbal = b(ilo:ihi,ilo:ihi);
abal = a(ilo:ihi,ilo:ihi);

% QR factorize balanced B
[QR, tau, info] = f08as(bbal);

% Perform C = Q^H*A
side = 'Left';
trans = 'Conjugate transpose';
[c, info] = f08au(...
                  side, trans, QR, tau, abal);

% Generalized Hessenberg form (C,R) -> (H,T)
compq = 'Vectors Q';
compz = 'Vectors Z';

% Form Q explicitly and let Z = I.
[q, info] = f08at(QR, tau);
z = complex(eye(4));

jlo = nag_int(1);
jhi = nag_int(ihi-ilo+1);
[H, T, q, z, info] = ...
  f08ws(...
        compq, compz, jlo, jhi, c, QR, q, z);


% Find eigenvalues of generalized Hessenberg form
%    = eigenvalues of (A,B).
% and return Schur form for computing eigenvectors
job = 'Schur form';
ilo = nag_int(1);
ihi = nag_int(4);
[HS, TS, alpha, beta, q, z, info] = ...
  f08xs(...
        job, compq, compz, jlo, jhi, H, T, q, z);

disp('Generalized eigenvalues of (A,B):');
disp(alpha./beta);

% Obtain scaled eigenvectors from Schur form
side = 'Both sides';
howmny = 'Backtransformed using Q and Z';
select = [false];
[q, z, m, info] = f08yx(...
                        side, howmny, select, HS, TS, q, z, jhi);

% rescale to obtain left and right eigenvectors of (A,B)
job  = 'Back scale';
side = 'Left';
[VL, info] = f08ww( ...
                    job, side, jlo, jhi, lscale, rscale, q);
side = 'Right';
[VR, info] = f08ww( ...
                    job, side, jlo, jhi, lscale, rscale, z);

% Normalize eigenvectors to have Euclidean norm 1 and largest component real.
incv = nag_int(1);
n = nag_int(4);
for j = 1:n
  [k, r] = f16js(n, VL(:,j), incv);
  scal = conj(VL(k,j))/abs(VL(k,j))/norm(VL(:,j),2);
  VL(:,j) = VL(:,j)*scal;
end
for j = 1:n
  [k, r] = f16js(n, VR(:,j), incv);
  scal = conj(VR(k,j))/abs(VR(k,j))/norm(VR(:,j),2);
  VR(:,j) = VR(:,j)*scal;
end

disp('Right Eigenvectors');
disp(VR);

disp('Left Eigenvectors');
disp(VL);

% diary off
