function f11bs_example

% diary f11bs_example.r
fprintf('f11bs example results\n\n');

% Solve sparse system Ax = b iteratively using ILU preconditioner

% Define sparse matrix A and RHS B
nz   = nag_int(24);
n    = nag_int(8);
a    = complex(zeros(3*nz,1));
irow = zeros(3*nz,1,nag_int_name);
icol = zeros(3*nz,1,nag_int_name);
a(1:nz) = [ 2 + 1i, -1 + 1i,  1 - 3i,  4 + 7i, -3 + 0i,  2 + 4i, ...
           -7 - 5i,  2 + 1i,  3 + 2i, -4 + 2i,  0 + 1i,  5 - 3i, ...
           -1 + 2i,  8 + 6i, -3 - 4i, -6 - 2i,  5 - 2i,  2 + 0i, ...
            0 - 5i, -1 + 5i,  6 + 2i, -1 + 4i,  2 + 0i,  3 + 3i];
b       = [ 7 + 11i;
            1 + 24i;
          -13 - 18i;
          -10 +  3i;
           23 + 14i;
           17 -  7i;
           15 -  3i;
           -3 + 20i];
irow(1:nz) = nag_int(...
             [1; 1; 1; 2; 2; 2; 3; 3; 4; 4; 4; 4;
              5; 5; 5; 6; 6; 6; 7; 7; 7; 8; 8; 8]);
icol(1:nz) = nag_int(...
             [1; 4; 8; 1; 2; 5; 3; 6; 1; 3; 4; 7;
              2; 5; 7; 1; 3; 6; 3; 5; 7; 2; 6; 8]);

% ILU preconditioner
%     Input parameters
lfill = nag_int(0);
dtol  = 0;
milu  = 'No modification';
ipivp = zeros(n, 1, nag_int_name);
ipivq = zeros(n, 1, nag_int_name);
% Compute preconditioner
[a, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm, ifail] = ...
    f11dn(...
          nz, a, irow, icol, lfill, dtol, milu, ipivp, ipivq);


% Iterative Solution Setup
%      Input parameters
method = 'TFQMR   ';
precon = 'P';
lpoly  = nag_int(1);
tol    = sqrt(x02aj);
maxitn = nag_int(20);
anorm  = 0;
sigmax = 0;
monit  = nag_int(2);
lwork  = nag_int(6000);

[lwreq, work, ifail] = ...
     f11br(...
           method, precon, n, lpoly, tol, maxitn, anorm, sigmax, ...
           monit, lwork, 'norm_p', '1');

irevcm = nag_int(0);
wgt    = zeros(n,1);
u      = complex(zeros(n,1));
v      = b;

while (irevcm ~= 4)
  [irevcm, u, v, work, ifail] = f11bs(...
                                      irevcm, u, v, wgt, work);

  if (irevcm == -1)
    % v = A^Hu
    [v, ifail] = f11xn(...
                       'T', a(1:nz), irow(1:nz), icol(1:nz), 'N', u);
  elseif (irevcm == 1)
    % v = Au
    [v, ifail] = f11xn(...
                       'N', a(1:nz), irow(1:nz), icol(1:nz), 'N', u);
  elseif (irevcm == 2)
    % Solve ILU v = u  
    [v, ifail] = f11dp(...
                       'N', a, irow, icol, ipivp, ipivq, istr, idiag, 'N', u);
  elseif (irevcm == 3)
    % Monitoring stage
    [itn, stplhs, stprhs, anorm, sigmax, work, ifail] = ...
       f11bt(work);
    fprintf('\nMonitoring at iteration number %2d\n',itn);
    fprintf('residual norm:              %14.4e\n', stplhs);
    fprintf('\n   Solution Vector\n');
    disp(u);
    fprintf('\n   Residual Vector\n');
    disp(v);
  end
end

% Get information about the computation
[itn, stplhs, stprhs, anorm, sigmax, work, ifail] = ...
      f11bt(work);

fprintf('\nNumber of iterations for convergence:     %4d\n', itn);
fprintf('Residual norm:                           %14.4e\n', stplhs);
fprintf('Right-hand side of termination criteria: %14.4e\n', stprhs);
fprintf('i-norm of matrix a:                      %14.4e\n', anorm);
fprintf('\n   Solution Vector\n');
disp(u);
fprintf('\n   Residual Vector\n');
disp(v);

% diary off
