function f11dk_example

% diary f11dk_example.r
fprintf('f11dk example results\n\n');

% Solve sparse system Ax = b iteratively using Jacobi preconditioner

% Define sparse matrix A and RHS B
n    = nag_int(8);
nz   = nag_int(24);
irow(1:nz) = nag_int(...
                     [1; 1; 1; 2; 2; 2; 3; 3; 4; 4; 4; 4;
                      5; 5; 5; 6; 6; 6; 7; 7; 7; 8; 8; 8]);
icol(1:nz) = nag_int(...
                     [1; 4; 8; 1; 2; 5; 3; 6; 1; 3; 4; 7;
                      2; 5; 7; 1; 3; 6; 3; 5; 7; 2; 6; 8]);
a(1:nz)    = [4;-1; 1; 4;-5; 2;-7; 2; 2;-1; 6; 2;
             -1; 8;-2;-2; 5; 8;-2;-1; 7;-1; 2; 6];
b          = [6; 8;-9;46;17;21;22;34];

% Iterative Solution Setup
%      Input parameters
method = 'BICGSTAB';
precon = 'Preconditioned';
lpoly  = nag_int(2);
tol    = sqrt(x02aj)^(3/8);
maxitn = nag_int(20);
anorm  = 0;
sigmax = 0;
monit  = nag_int(1);
lwork  = nag_int(200);

% Setup iterative method
[lwreq, work, ifail] = ...
  f11bd(method, precon, n, lpoly, tol, maxitn, anorm, sigmax, ...
        monit, lwork, 'norm_p', '1');

% Preconditioner parameters
store = 'Non symmetric';
trans = 'N';
init  = 'I';
niter = nag_int(4);
wgt   = zeros(n, 1);
dgnl  = zeros(n, 1);
x     = zeros(n, 1);

% Repeatedly call f11be to solve the equations.
% On final exit x will contain the solution and b the residual vector.
% Jacobi preconditioner is via f11dk when irevcm==2.

irevcm = nag_int(0);
init = 'i';
ifail = nag_int(0);

while irevcm ~= 4
  [irevcm, x, b, work, ifail] = ...
  f11be(...
        irevcm, x, b, wgt, work);

  if (irevcm == -1)
    % b = A^Tx
    [b, ifail1] = f11xa(...
        'Transpose', a, irow, icol, 'N', x);
  elseif (irevcm == 1)
    % b = Ax
    [b, ifail1] = f11xa(...
        'No Transpose', a, irow, icol, 'N', x);
  elseif (irevcm == 2)
    % Jacobi preconditioning Jb = x
    [b, dgnl, ifail1] = f11dk(...
        store, trans, init, niter, a(1:nz), irow, icol, 'Check', x, dgnl);
    init = 'n';
  elseif (irevcm == 3)
    % Monitoring
    [itn, stplhs, stprhs, anorm, sigmax, ifail1] = ...
        f11bf(work);
    fprintf('\nMonitoring at iteration number %2d\n',itn);
    fprintf('residual norm:              %14.4e\n', stplhs);
  end

end

% Get information about the computation
[itn, stplhs, stprhs, anorm, sigmax, ifail] = ...
    f11bf(work);

fprintf('\nNumber of iterations for convergence:     %4d\n', itn);
fprintf('Residual norm:                           %14.4e\n', stplhs);
fprintf('Right-hand side of termination criteria: %14.4e\n', stprhs);
fprintf('i-norm of matrix a:                      %14.4e\n', anorm);

fprintf('\n   Solution Vector  Residual Vector\n');
for i = 1:n
  fprintf('%16.4f %16.2e\n', x(i), b(i));
end

% diary off
