function f11gd_example

% diary f11gd_example.r
fprintf('f11gd example results\n\n');

% Sparse matrix A
n    = nag_int(7);
nz   = nag_int(16);
a    = zeros(1000,1);
irow = zeros(1000,1,nag_int_name);
icol = irow;

a(1:16)    = [4  1  5  2  2  3 -1  1  4  1 -2  3  2 -1 -2  5];
irow(1:16) = [1  2  2  3  4  4  5  5  5  6  6  6  7  7  7  7];  
icol(1:16) = [1  1  2  3  2  4  1  4  5  2  5  6  1  2  3  7];


% Incomplete Cholesky factorization, zero fill
lfill  = nag_int(0);
dtol   = 0;
mic    = 'N';
dscale = 0;
ipiv   = zeros(n, 1, nag_int_name);

[a, irow, icol, ipiv, istr, nnzc, npivm, ifail] = ...
f11ja( ...
       nz, a, irow, icol, lfill, dtol, mic, dscale, ipiv);

% Solver setup
method = 'CG';
precon = 'P';
tol    = 1e-6;
maxitn = nag_int(20);
anorm  = 0;
sigmax = 0;
maxits = nag_int(7);
monit  = nag_int(2);

[lwreq, work, ifail] = ...
  f11gd(method, precon, n, tol, maxitn, anorm, sigmax, ...
        maxits, monit, 'sigcmp', 'S', 'norm_p', '1');

% Solver inputs
irevcm = nag_int(0);
u      = zeros(n,1);
v      = [15; 18; -8; 21; 11; 10; 29];
wgt    = zeros(n,1);

% Reverse communication loop
while (irevcm ~= 4)
  [irevcm, u, v, work, ifail] = f11ge( ...
                                       irevcm, u, v, wgt, work);
  if (irevcm == 1)
    % v = Au
    [v, ifail] = f11xe( ...
                        a, irow, icol, 'N', u, 'nz', nz);
  elseif (irevcm == 2)
    % preconditioner
    [v, ifail] = f11jb( ...
                        a, irow, icol, ipiv, istr, 'N', u);
  elseif (irevcm == 3)
    [itn, stplhs, stprhs, anorm, sigmax, its, sigerr, ifail] = ...
    f11gf(work);

    fprintf('\nMonitoring at iteration number %d\n',itn);
    fprintf('residual norm: %14.4e\n\n', stplhs);
    fprintf('   Solution Vector  Residual Vector\n');
    fprintf('%16.4f  %16.4e\n', [u'; v']);
  end
end

% Get information about the computation
[itn, stplhs, stprhs, anorm, sigmax, its, sigerr, ifail] = f11gf(work);
fprintf('\nNumber of iterations for convergence: %d\n', itn);
fprintf('Residual norm:                           %14.4e\n', stplhs);
fprintf('Right-hand side of termination criteria: %14.4e\n', stprhs);
fprintf('i-norm of matrix a:                      %14.4e\n', anorm);
fprintf('\n   Solution Vector  Residual Vector\n');
fprintf('%16.4f  %12.2e\n', [u'; v']);

% diary off
