function f12aa_example

% diary f12aa_example.r
fprintf('f12aa example results\n\n');

% Discretization is on 12x12 grid, so spacing is h = 1/11;
% Dirichlet conditions reduce this to a system of order 10x10.
nx   = nag_int(10);
n    = nx^2;
nev  = nag_int(10);
ncv  = nag_int(30);

h    = 1/(double(nx)+1);
rho  = 100;

irevcm = nag_int(0);
resid = zeros(n,1);
v = zeros(n, ncv);
x = zeros(n, 1);
mx = zeros(n, 1);

dd = 4/h^2;
dl = -1/h^2 - rho/(2*h);
du = -1/h^2 + rho/(2*h);
y = zeros(n,1);

[icomm, comm, ifail] = f12aa( ...
                              n, nev, ncv);
[icomm, comm, ifail] = f12ad( ...
                              'Smallest Mag', icomm, comm);

while (irevcm ~= 5)
  [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    f12ab( ...
           irevcm, resid, v, x, mx, comm, icomm);
  if (irevcm == -1 || irevcm == 1)
    % -u_xx-rho*u_x 
    y(1:nx:n) = dd*x(1:nx:n) + du*x(2:nx:n);
    for j = 2:nx-1
      y(j:nx:n) = dl*x(j-1:nx:n) + dd*x(j:nx:n) + du*x(j+1:nx:n);
    end
    y(nx:nx:n) = dl*x(nx-1:nx:n) + dd*x(nx:nx:n);
    % -u_yy
    y(1:nx) = y(1:nx) - (1/h^2)*x(nx+1:2*nx);
    for j = 2:nx-1
      lo = (j-1)*nx;
      y(lo+1:lo+nx) =  y(lo+1:lo+nx) - (1/h^2)*x(lo+nx+1:lo+2*nx) ...
                       - (1/h^2)*x(lo-nx+1:lo);
    end
    lo = (nx-1)*nx;
    y(lo+1:lo+nx) =  y(lo+1:lo+nx) - (1/h^2)*x(lo-nx+1:lo);
    x = y;
  end
end

[nconv, dr, di, z, v, comm, icomm, ifail] = ...
  f12ac( ...
         0, 0, resid, v, comm, icomm);

fprintf('The %4d Ritz values of smallest magnitude are:\n\n',nconv);
fprintf('%9.3f %+9.3fi\n', [dr di]');

% diary off
