function g02dc_example

% diary g02dc_example.r
fprintf('g02dc example results\n\n');

x = [1, 0, 0, 0;
     0, 0, 0, 1;
     0, 1, 0, 0;
     0, 0, 1, 0;
     0, 0, 0, 1;
     0, 1, 0, 0;
     0, 0, 0, 1;
     1, 0, 0, 0;
     0, 0, 1, 0;
     1, 0, 0, 0;
     0, 0, 1, 0;
     1, 1, 1, 1];
y = [33.63;     39.62;     38.18;     41.46;     38.02;     35.83;
     35.99;     36.58;     42.92;     37.80;     40.43;     37.89];

[n,m]  = size(x);
isx    = ones(m,1,nag_int_name);
mean_p = 'Z';
ip     = nag_int(m);

% Fit initial general linear regression model
[rss, idf, b, se, covar, res, h, q, svd, irank, p, wk, ifail] = ...
  g02da(mean_p, x, isx, ip, y);

% Display initial results
fprintf('Results from initial model fit using g02da\n\n');
if svd
  fprintf('Model not of full rank, rank = %4d\n\n', irank);
end
fprintf('Residual sum of squares = %12.4e\n', rss);
fprintf('Degrees of freedom      = %4d\n', idf);
fprintf('\nVariable   Parameter estimate   Standard error\n\n');
ivar = double([1:ip]');
fprintf('%6d%20.4e%20.4e\n',[ivar b se]');

% Add or delete observations one at a time
action = {'dropping'; 'adding'};
xobs = [1   1   1   1;
        0   1   0   0];
yobs = [37.89;
        37.89];
nobs = numel(yobs);
ix = nag_int(1);

for j = 1:nobs
  % update regression
  [q, rss, ifail] = g02dc( ...
                           action{j}, mean_p, isx, q, xobs(j,:), ix, ...
                           yobs(j), rss, 'ip', ip);
  % Parameter estmate update
  
  if strcmp(action{j},'adding')
    n = n + 1;
  else
    n = n - 1;
  end
  [rss, idf, b, se, covar, svd, irank, p, ifail] = ...
  g02dd(nag_int(n), ip, q, rss);
  
  fprintf('\nResults from %s an observation\n',action{j});
  fprintf('Residual sum of squares = %12.4e\n', rss);
  fprintf('Degrees of freedom      = %4d\n', idf);
  fprintf('\nVariable   Parameter estimate   Standard error\n\n');
  fprintf('%6d%20.4e%20.4e\n',[ivar b se]');

end

% diary off
