function g05xa_example

% diary g05xa_example.r
fprintf('g05xa example results\n\n');


% Get information required to set up the bridge
[bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data();

% Make the bridge construction bgord
[times, ifail] = g05xe( ...
                        t0, tend, intime, move, 'bgord', bgord);

% Initialize the Brownian bridge generator
[rcomm, ifail] = g05xa( ...
                        t0, tend, times);

% Get additional information required by the bridge generator
[npaths,d,start,a,term,c] = get_bridge_gen_data();

% Generate the Z values
[z] = get_z(npaths, d, a, ntimes);

% Call the Brownian bridge generator routine
[z, b, ifail] = g05xb( ...
                       npaths, start, term, z, c, rcomm, 'a', a);

% Display the results
for i = 1:npaths
  fprintf('Weiner Path %d, %d time steps, %d dimensions\n', i, ntimes+1, d);
  w = transpose(reshape(b(:,i), d, ntimes+1));

  ifail = x04ca('G', ' ', w, '');

  fprintf('\n');
end

% diary off

function [bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data()
  % Set the basic parameters for a Wiener process
  t0 = 0;
  n = 10;
  ntimes = nag_int(n);

  % We want to generate the Wiener process at these time points
  intime = 1:n + t0;
  tend   = t0 + n + 1;

  nmove = nag_int(0);
  move  = zeros(nmove, 1, nag_int_name);
  bgord = nag_int(3);

function [npaths,d,start,a,term,c] = get_bridge_gen_data();
  % Set the basic parameters for a free Wiener process
  npaths = nag_int(2);
  d = 3;
  a = nag_int(0);
  start = zeros(d, 1);
  term  = zeros(d, 1);

  % As a=0, term need not be initialized

  % We want the following covariance matrix
  c = [ 6,   1,  -0.2; 
        1,   5,   0.3;
       -0.2, 0.3, 4  ];

  % Cholesky factorization of the covariance matrix c
  [c, info] = f07fd('l', c);

function [z] = get_z(npaths, d, a, ntimes)
  idim = d*(ntimes+1-a);

  % Generate the input pseudorandom points

  % First initialize the base pseudorandom number generator
  state = initialize_prng(nag_int(6), nag_int(0), [nag_int(1023401)]);

  % Generate the pseudorandom points from N(0,1)
  [state, z, ifail] = g05sk( ...
                             nag_int(idim*npaths), 0, 1, state);

  z = reshape(z, idim, npaths);

function [state] = initialize_prng(genid, subid, seed)
  % Initialize the generator to a repeatable sequence
  [state, ifail] = g05kf( ...
                          genid, subid, seed);
