function g05xb_example

% diary g05xb_example.r
fprintf('g05xb example results\n\n');

% Get information required to set up the bridge
[bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data();

% Make the bridge construction bgord
[times, ifail] = g05xe( ...
                        t0, tend, intime, move, 'bgord', bgord);

% Initialize the Brownian bridge generator
[rcomm, ifail] = g05xa( ...
                        t0, tend, times);

% Get additional information required by the bridge generator
[npaths, d, start, term, c] = get_bridge_gen_data();

% Generate the Z values
[z] = get_z(npaths, d, ntimes);

% Call the Brownian bridge generator routine
[z, b, ifail] = g05xb( ...
                       npaths, start, term, z, c, rcomm);

% Display the results
for i = 1:npaths
  fprintf('Weiner Path %d, %d time steps, %d dimensions\n', i, ntimes+1, d);
  w = transpose(reshape(b(:,i), d, ntimes+1));

  ifail = x04ca('G', ' ', w, '');

  fprintf('\n');
end

% diary off

function [bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data()
  % Set the basic parameters for a Wiener process
  t0 = 0;
  n = 10;
  ntimes = nag_int(n);

  % We want to generate the Wiener process at these time points
  intime = 1:n + t0;
  tend = t0 + n + 1;

  nmove= nag_int(0);
  move = zeros(nmove, 1, nag_int_name);
  bgord = nag_int(3);

function [npaths,d,start,term,c] = get_bridge_gen_data();
  % Set the basic parameters for a non-free Wiener process
  npaths = nag_int(2);
  d = 3;
  start = zeros(d, 1);
  term  = [1, 0.5, 0];

  % We want the following covariance matrix
  c = [ 6,   1,   -0.2; 
        1,   5,    0.3; 
       -0.2, 0.3,  4];

  % Cholesky factorize the covariance matrix c
  [c, info] = f07fd('l', c);

function [z] = get_z(npaths, d, ntimes)
  % Non-free Wiener process (the default)
  a = nag_int(1); 
  idim = d*(ntimes+1-a);

  % We now need to generate the input pseudorandom points

  % First initialize the base pseudorandom number generator
  state = initialize_prng(nag_int(6), nag_int(0), [nag_int(1023401)]);

  % Scrambled quasi-random sequences preserve the good discrepancy
  % properties of quasi-random sequences while counteracting the bias
  % some applications experience when using quasi-random sequences.
  % Initialize the scrambled quasi-random generator.
  [iref, state] = initialize_scrambled_qrng(nag_int(1), nag_int(2), ...
                                            idim, state);

  % Generate the quasi-random points from N(0,1)
  xmean = zeros(idim, 1);
  std   = ones(idim, 1);
  [z, iref, ifail] = g05yj( ...
                            xmean, std, npaths, iref);
  z = z';

function [state] = initialize_prng(genid, subid, seed)
  % Initialize the generator to a repeatable sequence
  [state, ifail] = g05kf( ...
                          genid, subid, seed);


function [iref, state] = initialize_scrambled_qrng(genid,stype,idim,state)
  iskip = nag_int(0);
  nsdigits = nag_int(32);
  [iref, state, ifail] = g05yn( ...
                                genid, stype, nag_int(idim), iskip, ...
                                nsdigits, state);
