function nag_contab_binary_example

% diary nag_contab_binary_example.r
fprintf('nag_contab_binary example results\n\n');

n = nag_int(1000);
x = [false, false, false, false;
     true,  false, false, false;
     false, false, false, true;
     false, true,  false, false;
     true,  false, false, true;
     true,  true,  false, false;
     false, true,  false, true;
     false, false, true,  false;
     true,  true,  false, true;
     true,  false, true,  false;
     false, false, true,  true;
     false, true,  true,  false;
     true,  false, true,  true;
     true,  true,  true,  false;
     false, true,  true,  true;
     true,  true,  true,  true];
irl = [nag_int(154);  11;  42;  49;
                  2;  10;  27;  84;
                 10;  25;  75; 129;
                 30;  50; 181; 121];

% Initial values
a = [0.5;     0.5;     0.5;     0.5];
c = [0;       0;       0;       0];

% Parameters
gprob  = false;
cgetol = 0.0001;
chisqr = true;
iprint = nag_int(-1);

% Fit a latent variable model
[x, irl, a, c, niter, alpha, pigam, cm, g, expp, obs, exf, y, ...
  iob, rlogl, chi, idf, siglev, ifail] = ...
  nag_contab_binary( ...
         n, gprob, x, irl, a, c, cgetol, chisqr, 'iprint', iprint);

% Display results
fprintf('Log likelihood kernel on exit = %14.4e\n\n', rlogl);

fprintf('Maximum likelihood estimates of item parameters are as follows\n');
fprintf('--------------------------------------------------------------\n\n');
fprintf('%8s%12s%11s%16s%10s%13s\n\n', 'item j', 'alpha(j)', 's.e.', ...
        'alpha(j,0)', 'pi(j)', 's.e.');
ivar = [1:numel(a)]';
se   = diag(cm);
results = [ivar alpha se(1:2:end) c pigam se(2:2:end)];
fprintf('%5d%13.3f%13.3f%13.3f%13.3f%13.3f\n',results');
fprintf('\n\n');

mtitle = 'Expected percentage of cases producing positive responses';
[ifail] = nag_file_print_matrix_real_gen( ...
                 'Lower', 'Non-unit', expp, mtitle);
fprintf('\n');

mtitle = 'Observed percentage of cases producing positive responses';
[ifail] = nag_file_print_matrix_real_gen( ...
                 'Lower', 'Non-unit', obs, mtitle);
fprintf('\n\n');

fprintf(' Observed   Expected   Theta   Component   Raw    Score\n');
fprintf(' frequency  frequency  score   score       score  pattern\n\n');
cs = double(x)*alpha;
results = [ double(irl)  exf  y  cs double(iob) double(x)];
fprintf('%7d%13.3f%8.3f%10.3f%8d    %1d%1d%1d%1d\n',results');
fprintf('---------  ---------\n');
fprintf('%7d%13.3f\n\n',n,n);

fprintf('Likelihood ratio goodness of fit statistic = %10.3f\n', chi);
fprintf('                        Significance level = %10.3f\n', siglev);
fprintf('(Based on %4d degrees of freedom)\n',idf)

% diary off
