function nag_correg_lars_param_example

% diary nag_correg_lars_param_example.r
fprintf('nag_correg_lars_param example results\n\n');

% Going to be fitting a LAR model via nag_correg_lars and getting nag_correg_lars
% to mean center y and normalise X around the mean
mtype = nag_int(1);
pred = nag_int(3);
prey = nag_int(1);

% Independent variables
d = [10.28  1.77  9.69 15.58  8.23 10.44;
      9.08  8.99 11.53  6.57 15.89 12.58;
     17.98 13.10  1.04 10.45 10.12 16.68;
     14.82 13.79 12.23  7.00  8.14  7.79;
     17.53  9.41  6.24  3.75 13.12 17.08;
      7.78 10.38  9.83  2.58 10.13  4.25;
     11.95 21.71  8.83 11.00 12.59 10.52;
     14.60 10.09 -2.70  9.89 14.67  6.49;
      3.63  9.07 12.59 14.09  9.06  8.19;
      6.35  9.79  9.40 12.79  8.38 16.79;
      4.66  3.55 16.82 13.83 21.39 13.88;
      8.32 14.04 17.17  7.93  7.39 -1.09;
     10.86 13.68  5.75 10.44 10.36 10.06;
      4.76  4.92 17.83  2.90  7.58 11.97;
      5.05 10.41  9.89  9.04  7.90 13.12;
      5.41  9.32  5.27 15.53  5.06 19.84;
      9.77  2.37  9.54 20.23  9.33  8.82;
     14.28  4.34 14.23 14.95 18.16 11.03;
     10.17  6.80  3.17  8.57 16.07 15.93;
      5.39  2.67  6.37 13.56 10.68  7.35];

% Dependent variable
y = [-46.47; -35.80; -129.22;  -42.44; -73.51;
     -26.61; -63.90;  -76.73;  -32.64; -83.29;
     -16.31;  -5.82;  -47.75;   18.38; -54.71;
     -55.62; -45.28;  -22.76; -104.32; -55.94];

% nag_correg_lars can issue warnings, but return sensible results,
% so save current warning state and turn warnings on
warn_state = nag_issue_warnings();
nag_issue_warnings(true);

% Call the model fitting routine
[b,fitsum,ifail] = nag_correg_lars(mtype,d,y);

% Reset the warning state to its initial value
nag_issue_warnings(warn_state);

% Set how the additional estimates will be specified

% Location of additional parameter estimates (as defined by the
% LARS step number)
ktype = nag_int(1);
nk = [0.2; 1.2; 3.2; 4.5; 5.2];

% Calculate the additional parameter estimates
[nb,ifail] = nag_correg_lars_param(b,fitsum,ktype,nk);

% Print the results
ip = size(b,1);
K = size(b,2) - 2;
lnk = size(nk,1);

fprintf(' Parameter Estimates from nag_correg_lars\n');
fprintf('  Step %s Parameter Estimate\n ',repmat(' ',1,max(ip-2,0)*5));
fprintf(repmat('-',1,5+ip*10));
fprintf('\n');
for k = 1:K
  fprintf('  %3d',k);
  for j = 1:ip
    fprintf(' %9.3f',b(j,k));
  end
  fprintf('\n');
end
fprintf('\n');

fprintf(' Additional Parameter Estimates from nag_correg_lars_param\n');
fprintf('   nk  %s Parameter Estimate\n ',repmat(' ',1,max(ip-2,0)*5));
fprintf(repmat('-',1,5+ip*10));
fprintf('\n');
for k = 1:lnk
  fprintf('  %4.1f',nk(k));
  for j = 1:ip
    fprintf(' %9.3f',nb(j,k));
  end
  fprintf('\n');
end

% diary off
