function nag_correg_lars_xtx_example

% diary nag_correg_lars_xtx_example.r
fprintf('nag_correg_lars_xtx example results\n\n');

% Going to be fitting a LAR model via nag_correg_lars_xtx
mtype = nag_int(1);

% Augmented matrix [D y]
dy = [10.28  1.77  9.69 15.58  8.23 10.44  -46.47;
       9.08  8.99 11.53  6.57 15.89 12.58  -35.80;
      17.98 13.10  1.04 10.45 10.12 16.68 -129.22;
      14.82 13.79 12.23  7.00  8.14  7.79  -42.44;
      17.53  9.41  6.24  3.75 13.12 17.08  -73.51;
       7.78 10.38  9.83  2.58 10.13  4.25  -26.61;
      11.95 21.71  8.83 11.00 12.59 10.52  -63.90;
      14.60 10.09 -2.70  9.89 14.67  6.49  -76.73;
       3.63  9.07 12.59 14.09  9.06  8.19  -32.64;
       6.35  9.79  9.40 12.79  8.38 16.79  -83.29;
       4.66  3.55 16.82 13.83 21.39 13.88  -16.31;
       8.32 14.04 17.17  7.93  7.39 -1.09   -5.82;
      10.86 13.68  5.75 10.44 10.36 10.06  -47.75;
       4.76  4.92 17.83  2.90  7.58 11.97   18.38;
       5.05 10.41  9.89  9.04  7.90 13.12  -54.71;
       5.41  9.32  5.27 15.53  5.06 19.84  -55.62;
       9.77  2.37  9.54 20.23  9.33  8.82  -45.28;
      14.28  4.34 14.23 14.95 18.16 11.03  -22.76;
      10.17  6.80  3.17  8.57 16.07 15.93 -104.32;
       5.39  2.67  6.37 13.56 10.68  7.35  -55.94];

% Number of observations in the dataset
n = nag_int(size(dy,1));

% Calculate the means and cross-product matrix around the mean
mean_p = 'M';
[~,wmean,dtd,ifail] = nag_correg_ssqmat( ...
                             dy,'mean_p',mean_p);

% Number of variables
m = nag_int(size(dy,2) - 1);

% The first pm elements of dtd contain the cross-products of D
% with itself, the next m elements hold the cross-product of D
% and y and the last element holds the cross-product of y with
% itself
pm = m*(m+1)/2;

% nag_correg_lars_xtx can issue warnings, but return sensible results,
% so save current warning state and turn warnings on
warn_state = nag_issue_warnings();
nag_issue_warnings(true);

[b,fitsum,ifail] = nag_correg_lars_xtx( ...
                          mtype,n,dtd(1:pm),dtd((pm+1):(pm+m)),dtd(end));

% Reset the warning state to its initial value
nag_issue_warnings(warn_state);

% Print the results
ip = size(b,1);
nstep = size(b,2) - 1;

fprintf('  Step %s Parameter Estimate\n ',repmat(' ',1,max(ip-2,0)*5));
fprintf(repmat('-',1,5+ip*10));
fprintf('\n');
for k = 1:nstep
  fprintf('  %3d',k);
  for j = 1:ip
    fprintf(' %9.3f',b(j,k));
  end
  fprintf('\n');
end
fprintf('\n');
fprintf(' alpha: %9.3f\n', wmean(m+1));
fprintf('\n');
fprintf('  Step     Sum      RSS       df       Cp       Ck     Step Size\n ');
fprintf(repmat('-',1,64));
fprintf('\n');
for k = 1:nstep
  fprintf('  %3d %9.3f %9.3f %6.0f  %9.3f %9.3f %9.3f\n', ...
          k,fitsum(1,k),fitsum(2,k),fitsum(3,k), ...
          fitsum(4,k),fitsum(5,k),fitsum(6,k));
end
fprintf('\n');
fprintf(' sigma^2: %9.3f\n', fitsum(5,nstep+1));

% Plot the parameter estimates
fig1 = figure;
ip = size(b,1);
nstep = size(b,2) - 2;

% Extract the sum of the absolute parameter estimates
xpos = transpose(repmat(fitsum(1,1:nstep),ip,1));

% Extract the parameter estimates
ypos = transpose(b(1:ip,1:nstep));

% Start both xpos and ypos at zero
xpos = [zeros(1,ip);xpos];
ypos = [zeros(1,ip);ypos];

% Get min and max for X and Y
xmin = min(min(xpos));
xmax = max(max(xpos));
ymin = min(min(ypos));
ymax = max(max(ypos));

% Get a range that is 10% past the data
ext = 1 + [-0.1 0.1];
xrng = [min(xmin*ext),max(xmax*ext)];
yrng = [min(ymin*ext),max(ymax*ext)];

% Extend the end of the lines we plot to cover this range
xpos = [xpos;xrng(2)*ones(1,ip)];
ypos = [ypos;ypos(end,:)];

% Produce the plot
plot(xpos,ypos);

% Change the axis limits
xlim(xrng);
ylim(yrng);

% Add title and labels
title({'{\bf nag_correg_lars_xtx Example Plot}'; ...
       'Estimates for LAR model fitted to simulated dataset'});
xlabel('{\bf \Sigma_j |\beta_{kj} |}');
ylabel('{\bf Parameter Estimates (\beta_{kj})}');

% Add legend
label = [repmat('\beta_{k',ip,1) num2str(transpose(linspace(1,ip,ip))) ...
         repmat('}',ip,1)];
h = legend(label,'Location','SouthOutside','Orientation','Horizontal');
set(h,'FontSize',get(h,'FontSize')*0.8);

% print(fig1,'-dpng','-r75','nag_correg_lars_xtx_fig1.png');
% print(fig1,'-deps','-r75','nag_correg_lars_xtx_fig1.eps');

% diary off
