function nag_correg_ridge_example

% diary nag_correg_ridge_example.r
fprintf('nag_correg_ridge example results\n\n');

% Data
x = [19.5, 43.1, 29.1;
     24.7, 49.8, 28.2;
     30.7, 51.9, 37.0;
     29.8, 54.3, 31.1;
     19.1, 42.2, 30.9;
     25.6, 53.9, 23.7;
     31.4, 58.5, 27.6;
     27.9, 52.1, 30.6;
     22.1, 49.9, 23.2;
     25.5, 53.5, 24.8;
     31.1, 56.6, 30.0;
     30.4, 56.7, 28.3;
     18.7, 46.5, 23.0;
     19.7, 44.2, 28.6;
     14.6, 42.7, 21.3;
     29.5, 54.4, 30.1;
     27.7, 55.3, 25.7;
     30.2, 58.6, 24.6;
     22.7, 48.2, 27.1;
     25.2, 51.0, 27.5];

[n,m] = size(x);
isx   = ones(m,1,nag_int_name);
ip    = nag_int(m);
y = [11.9; 22.8; 18.7; 20.1; 12.9; 21.7; 27.1; 25.4; 21.3; 19.3;
     25.4; 27.2; 11.7; 17.8; 12.8; 23.9; 22.6; 25.4; 14.8; 21.1];

% Ridge coeficients
h = [0;         0.002;     0.004;     0.006;
     0.008;     0.01;      0.012;     0.014;
     0.016;     0.018;     0.02;      0.022;
     0.024;     0.026;     0.028;     0.030];
lh = numel(h);
% Parameters
wantb = nag_int(1);
wantvf = nag_int(1);
pec = {'L'; 'G'; 'U'; 'F'; 'B'};

% Fit ridge regression model
[nep, b, vf, pe, ifail] = nag_correg_ridge( ...
                                 x, isx, ip, y, h, wantb, wantvf, pec);

% Display results
fprintf('Number of parameters used           : %5d\n', ip+1);
fprintf('Number of effective parameters (nep): \n');
fprintf('   Ridge coeff     nep\n');
ivar = double([1:lh]);
fprintf('     %4d      %10.4f\n',[ivar; nep(ivar)']);
fprintf('\nParameter estimates (original scalings)\n');
fprintf(' Ridge coef Intercept');
fprintf('%9d ',[1:ip]);
fprintf('\n');
for j = 1:lh
  fprintf('%10.4f',h(j),b(1:ip+1,j));
  fprintf('\n');
end
% Variance inflation factors
fprintf('\nVariance Inflation Factors\n');
fprintf(' Ridge coef');
fprintf('%9d ',[1:ip]);
fprintf('\n');
for j = 1:lh
  fprintf('%10.4f',h(j),vf(1:ip,j));
  fprintf('\n');
end
% Prediction error criterion
fprintf('\nPrediction error criterion\n');
pl = numel(pec);
fprintf(' Ridge coef');
fprintf('%9d ',[1:pl]);
fprintf('\n');
for j = 1:lh
  fprintf('%10.4f',h(j),pe(1:pl,j));
  fprintf('\n');
  if pl<ip
    fprintf('%10s',' ');
    fprintf('%10.4f',pe(pl+1:ip,j));
    fprintf('\n');
  end
end
fprintf('\nKey:\n');
for i = 1:pl
  switch pec{i}
    case {'L'}
      fprintf(' %5d %s\n', i, 'Leave one out cross-validation');
    case {'G'}
      fprintf(' %5d %s\n', i, 'Generalised cross-validation');
    case {'U'}
      fprintf(' %5d %s\n', i, 'Unbiased estimate of variance');
    case {'F'}
      fprintf(' %5d %s\n', i, 'Final prediction error');
    case {'B'}
      fprintf(' %5d %s\n', i, 'Bayesian information criterion');
  end
end

% diary off
