function nag_correg_robustm_user_example

% diary nag_correg_robustm_user_example.r
fprintf('nag_correg_robustm_user example results\n\n');

global dchi
dchi = 1.5;

x = [1, -1, -1;
     1, -1,  1;
     1,  1, -1;
     1,  1,  1;
     1,  0,  3];
y   = [10.5;    11.3;     12.6;     13.4;     17.1   ];
wgt = [0.4039;   0.5012;   0.4039;   0.5012;   0.3862];

[n,m] = size(x);

% Calculate beta
[beta] = betcal(wgt);

% Control Parameters
psip0 = 1;
indw = nag_int(1);
isigma = nag_int(1);

% Intial values
sigma = 1;
theta = zeros(m,1);

% Perform bounded influence regression
[x, y, wgt, theta, k, sigma, rs, nit, ifail] = ...
  nag_correg_robustm_user( ...
         @chi, @psi, psip0, beta, indw, isigma, x, y, wgt, theta, sigma);

fprintf(' iterations to convergence = %4d\n', nit);
fprintf('                         k = %4d\n',k);
fprintf('                     sigma = %9.4f\n',sigma);
fprintf('Theta:\n');
disp(theta');
fprintf('\n  Weights  Residuals\n');
fprintf('%9.4f%9.4f\n', [wgt rs]');

% diary off

function [result] = chi(t)
  global dchi

  if (abs(t) < dchi)
    ps=t;
  else
    ps=dchi;
  end
  result = ps*ps/2;


function [result] = psi(t)
  global dchi

  if t < -dchi
    result = -dchi;
  elseif abs(t) < dchi
    result = t;
  else
    result = dchi;
  end;

function [beta] = betcal(wgt)
  %  Calculate beta for Schweppe type regression
  global dchi

  n = numel(wgt);
  amaxex = -log(nag_machine_real_smallest);
  anormc = sqrt(2*pi);
  d2 = dchi*dchi;
  beta = 0;
  for i = 1:n
    w2 = wgt(i)*wgt(i);
    dw = wgt(i)*dchi;
    [pc, ifail] = nag_specfun_cdf_normal(dw);
    dw2 = dw*dw;
    if dw2<amaxex
      dc = exp(-dw2/2)/anormc;
    else
      dc = 0;
    end
    b = (-dw*dc+pc-0.5)/w2 + (1-pc)*d2;
    beta = b*w2/n + beta;
  end
