function nag_fit_2dspline_grid_example

% diary nag_fit_2dspline_grid_example.r
fprintf('nag_fit_2dspline_grid example results\n\n');

start = 'C';
x = [0:0.5:5];
y = [0:0.5:4];
f = [ 1         1.5      2.06     2.57     3        3.5;
      0.88758   1.3564   1.7552   2.124    2.6427   3.1715;
      0.5403    0.82045  1.0806   1.3508   1.6309   1.8611;
      0.070737  0.10611  0.15147  0.17684  0.21221  0.24458;
     -0.41515  -0.62422 -0.83229 -1.0404  -1.2484  -1.4565;
     -0.80114  -1.2317  -1.6023  -2.0029  -2.2034  -2.864;
     -0.97999  -1.485   -1.97    -2.475   -2.97    -3.265;
     -0.93446  -1.3047  -1.8729  -2.3511  -2.8094  -3.2776;
     -0.65664  -0.98547 -1.4073  -1.6741  -1.9809  -2.2878];

f(:,7:11) = [   4.04     4.5      5.04     5.505    6;
                3.5103   3.9391   4.3879   4.8367   5.2755;  
                2.0612   2.4314   2.7515   2.9717   3.2418;  
                0.28595  0.31632  0.35369  0.38505  0.42442; 
               -1.6946  -1.8627  -2.0707  -2.2888  -2.4769;  
               -3.2046  -3.6351  -4.0057  -4.4033  -4.8169;  
               -3.96    -4.455   -4.97    -5.445   -5.93;    
               -3.7958  -4.2141  -4.6823  -5.1405  -5.6387;  
               -2.6146  -2.9314  -3.2382  -3.595   -3.9319];

% Input argument initializations
s     = 0.1;
nx    = nag_int(0); ny = nx;
mx    = size(x,2);
my    = size(y,2);
lamda = zeros(mx+4, 1);
mu    = zeros(my+4, 1);
wrk   = zeros(1000, 1);
iwrk = zeros(100, 1, nag_int_name);

% Compute bi-cubic spline for grid data
[nx, lamda, ny, mu, c, fp, wrk, iwrk, ifail] = ...
nag_fit_2dspline_grid( ...
       start, x, y, f, s, nx, lamda, ny, mu, wrk, iwrk);

% Print details of spline
fprintf('\nCalling with smoothing factor S = %5.2f\n\n', s);
fprintf('Knots:   lamda      mu\n');
for j = 4:max(nx,ny)-3
  if j<=min(nx,ny)-3
    fprintf('%4d%10.4f%10.4f\n', j, lamda(j), mu(j));
  elseif j<=nx-3
    fprintf('%4d%10.4f\n', j, lamda(j));
  else
    fprintf('%4d%20.4f\n', j, mu(j));
  end
end

cp = c(1:(ny-4)*(nx-4));
cp = reshape(cp,[ny-4,nx-4]);
fprintf('\nB-spline coefficients:\n');
disp(cp);

fprintf('Weighted sum of squared residuals = %7.4f\n', fp);
if fp==0
  fprintf('(The spline is an interpolating spline)\n');
elseif nx==8 && ny==8
  fprintf('(The spline is the weighted least-squares cubic polynomial)\n');
end
fprintf('\n');

% Evaluate spline on mesh
mx = [0:0.2:5];
my = [0:0.2:4];
[ff, ifail] = nag_fit_2dspline_evalm( ...
                     mx, my, lamda(1:nx), mu(1:ny), c);

fig1 = figure;
ff = reshape(ff,[21,26]);
meshc(mx,my,ff);
xlabel('x');
ylabel('y');
title('Least-squares bi-cubic spline fit');
% print(fig1,'-dpng','-r75','nag_fit_2dspline_grid_fig1.png');
% print(fig1,'-deps','-r75','nag_fit_2dspline_grid_fig1.eps');

% diary off
