function nag_glopt_nlp_multistart_sqp_example

% diary nag_glopt_nlp_multistart_sqp_example.r
fprintf('nag_glopt_nlp_multistart_sqp example results\n\n');

npts   = nag_int(1000);
repeat = true;
nclin  = nag_int(1);
ncnln  = nag_int(2);
nb     = nag_int(10);
n      = nag_int(2);

bl = [-500; -500; -10000; -1;      -0.9];
bu = [ 500;  500;     10;  500000;  0.9];
a  = [3, -2];

% Initialise nag_glopt_nlp_multistart_sqp
iopts = zeros(740, 1, nag_int_name);
opts  = zeros(485, 1);
[iopts, opts, ifail] = nag_glopt_optset(...
                             'Initialize = nag_glopt_nlp_multistart_sqp', iopts, opts);
[iopts, opts, ifail] = nag_glopt_optset(...
                             'Derivative Level = 3', iopts, opts);

wstat = warning();
warning('OFF');
% Solve the problem
[x, objf, objgrd, iter, c, cjac, r, clamda, istate, ...
                          iopts, opts, user, info, ifail] = ...
  nag_glopt_nlp_multistart_sqp(...
        n, ncnln, a, bl, bu, @confun, @objfun, ...
        npts, 'nag_glopt_multistart_start_points', repeat, nb, iopts, opts);

warning(wstat);

if ifail == 8
  l = double(info(10));
  fprintf('\nOnly %d solutions found\n', l);
else
  l = 10;
end

l = min(l,3);
% List details of first 3 solutions only
for i=1:l
  fprintf('\nSolution number %d\n\n', i);
  fprintf('nag_opt_nlp1_solve returned with ifail = %d\n\n', info(i));
  fprintf(' Variable Istate           Value Lagrange Multiplier\n');
  for j=1:double(n)
    fprintf(' %3d%9d%19.6g%13.4g\n', j, istate(j,i), x(j,i), clamda(j,i));
  end

  if nclin > 0
    ax = a*x(:,i);
    fprintf('\n L Con    Istate           Value Lagrange Multiplier\n');
    for k=double(n+1):double(n+nclin)
      j=k-n;
      fprintf(' %3d%9d%19.6g%13.4g\n', j, istate(k,i), ax(j), clamda(k,i));
    end
  end

  if ncnln > 0
    fprintf('\n NL Con  Istate            Value Lagrange Multiplier\n');
    for k=double(n+nclin+1):double(n+nclin +ncnln)
      j=k-n-nclin;
      fprintf(' %3d%9d%19.6g%13.4g\n', j, istate(k,i), c(j,i), clamda(j,i));
    end
  end

  fprintf('\nFinal objective value = %15.7g\n', objf(i));
  fprintf('clamda: ');
  disp(transpose(clamda(1:double(n+nclin+ncnln),i)));
  fprintf('\n ------------------------------------------------------\n');
end

% diary off

function [mode, objf, objgrd, user] = objfun(mode, n, x, objgrd, nstate, user)
  if mode==0 || mode==2
    % Evaluate the objective function.
    objf = x(1)*sin(sqrt(abs(x(1)))) + x(2)*sin(sqrt(abs(x(2))));
  else
    objf = 0;
  end

  if mode==1 || mode==2
    % Calculate the gradient of the objective function.
    t = sqrt(abs(x(1)));
    objgrd(1) = sin(t) + 0.5*t*cos(t);
    t = sqrt(abs(x(2)));
    objgrd(2) = sin(t) + 0.5*t*cos(t);
  end

function [mode, c, cjsl, user] = ...
    confun(mode, ncnln, n, ldcj1, needc, x, cjsl, nstate, user)
  c = zeros(ncnln, 1);
  dncnln = double(ncnln);

  if (mode==0 || mode==2)
    % Constraint values are required.
    % Only those for which needc is non-zero need be set.
    for k = 1:dncnln
      if (needc(k)>0)
        switch k
          case {1}
            c(k) = x(1)^2 - x(2)^2 + 3.0*x(1)*x(2);
          case {2}
            c(k) = cos((x(1)/200.0)^2+(x(2)/100.0));
        end
      end
    end
  end

  if (mode==1 || mode==2)
    % Constraint derivatives (cjsl) are required.
    for k = 1:dncnln
      switch k
        case {1}
          cjsl(k,1) = 2.0*x(1) + 3.0*x(2);
          cjsl(k,2) = -2.0*x(2) + 3.0*x(1);
        case {2}
          t1 = x(1)/200.0;
          t2 = x(2)/100.0;
          cjsl(k,1) = -sin(t1^2+t2)*2.0*t1/200.0;
          cjsl(k,2) = -sin(t1^2+t2)/100.0;
      end
    end
  end
