function nag_lapack_dgeqlf_example

% diary nag_lapack_dgeqlf_example.r
fprintf('nag_lapack_dgeqlf example results\n\n');

% Find least squares solution of Ax=B (m>n) via QL factorization.
m = 6;
n = 4;
a = [-0.57, -1.28, -0.39,  0.25;
     -1.93,  1.08, -0.31, -2.14;
      2.30,  0.24,  0.40, -0.35;
     -1.93,  0.64, -0.66,  0.08;
      0.15,  0.3,   0.15, -2.13;
     -0.02,  1.03, -1.43,  0.5];
b = [-2.67,  0.41;
     -0.55, -3.10;
      3.34, -4.01;
     -0.77,  2.76;
      0.48, -6.17;
      4.10,  0.21];

% Compute the QL factorization of A
[ql, tau, info] = nag_lapack_dgeqlf(a);

% LX = Q^T B = C; compute C = (Q^T)*B
[c, info] = nag_lapack_dormql( ...
		   'Left', 'Transpose', ql, tau, b);

% Least-squares solution X = L^-1 C (lower n part)
il = m-n+1;
[x, info] = nag_lapack_dtrtrs( ...
		   'Lower', 'NoTrans', 'Non-Unit', ql(il:m,:), c(il:m,:));

% Print least-squares solutions
[ifail] = nag_file_print_matrix_real_gen( ...
		 'General', ' ', x, 'Least-squares solution(s)');
% Compute estimates of the square roots of the residual sums of squares
rnorm = zeros(2,1);
for j=1:2
  rnorm(j) = norm(c(1:2,j));
end
fprintf('\nSquare root(s) of the residual sum(s) of squares\n');
fprintf('\t%11.2e    %11.2e\n', rnorm(1), rnorm(2));

% diary off
