function nag_lapack_dgeqrf_example

% diary nag_lapack_dgeqrf_example.r
fprintf('nag_lapack_dgeqrf example results\n\n');

a = [-0.57, -1.28, -0.39,  0.25;
     -1.93,  1.08, -0.31, -2.14;
      2.30,  0.24,  0.40, -0.35;
     -1.93,  0.64, -0.66,  0.08;
      0.15,  0.30,  0.15, -2.13;
     -0.02,  1.03, -1.43,  0.50];
b = [-2.67,  0.41;
     -0.55, -3.10;
      3.34, -4.01;
     -0.77,  2.76;
      0.48, -6.17;
      4.10,  0.21];
% Compute the QR Factorisation of A
[a, tau, info] = nag_lapack_dgeqrf(a);

% Compute C = (C1) = (Q^T)*B, storing the result in B (C2)
[b, info] = nag_lapack_dormqr(...
                  'Left', 'Transpose', a, tau, b);

% Compute least-squares solutions by backsubstitution in R*X = C1
[b, info] = nag_lapack_dtrtrs(...
                  'Upper', 'No Transpose', 'Non-Unit', a, b, 'n', nag_int(4));
if (info >0)
  fprintf('The upper triangular factor, R, of A is singular,\n');
  fprintf('the least squares solution could not be computed.\n');
else
  % Print least-squares solutions
  [ifail] = nag_file_print_matrix_real_gen('General', ' ', b(1:4,:), 'Least-squares solution(s)');
  % Compute and print estimates of the square roots of the residual
  % sums of squares
  rnorm = zeros(2,1);
  for j=1:2
    rnorm(j) = norm(b(5:6,j));
  end
  fprintf('\nSquare root(s) of the residual sum(s) of squares\n');
  fprintf('\t%11.2e    %11.2e\n', rnorm(1), rnorm(2));
end

% diary off
