function nag_lapack_dggbak_example

% diary nag_lapack_dggbak_example.r
fprintf('nag_lapack_dggbak example results\n\n');

n = 5;
a = [ 1.0   1.0    1.0    1.0     1.0;
      2.0   4.0    8.0   16.0    32.0;
      3.0   9.0   27.0   81.0   243.0;
      4.0  16.0   64.0  256.0  1024.0;
      5.0  25.0  125.0  625.0  3125.0];
b = a';

%' Balance A and B
job = 'B';
[a, b, ilo, ihi, lscale, rscale, info] = ...
  nag_lapack_dggbal(job, a, b);

bbal = b(ilo:ihi,ilo:ihi);
abal = a(ilo:ihi,ilo:ihi);

% QR factorize balanced B
[QR, tau, info] = nag_lapack_dgeqrf(bbal);

% Perform C = Q^T*A
side = 'Left';
trans = 'Transpose';
[c, info] = nag_lapack_dormqr( ...
		   side, trans, QR, tau, abal);

% Generalized Hessenberg form (C,R) -> (H,T)
% Form Q explicitly and let Z = I.
[q, info] = nag_lapack_dorgqr(QR, tau);

z         = eye(n);
jlo       = nag_int(1);
jhi       = nag_int(ihi-ilo+1);
compq     = 'Vectors Q';
compz     = 'Vectors Z';
[H, T, q, z, info] = ...
  nag_lapack_dgghrd( ...
         compq, compz, jlo, jhi, c, QR, q, z);

% Find eigenvalues of generalized Hessenberg form
%    = eigenvalues of (A,B).
% and return Schur form for computing eigenvectors
job = 'Schur form';
[HS, TS, alphar, alphai, beta, q, z, info] = ...
  nag_lapack_dhgeqz( ...
	 job, compq, compz, jlo, jhi, H, T, q, z);

disp('Generalized eigenvalues of (A,B):');
w = complex(alphar+i*alphai);
disp(w./beta);

% Obtain scaled eigenvectors from Schur form
side = 'Both sides';
howmny = 'Backtransformed using Q and Z';
select = [false];
[q, z, m, info] = nag_lapack_dtgevc( ...
			 side, howmny, select, HS, TS, q, z, jhi);

% rescale to obtain left and right eigenvectors of (A,B)
job  = 'Back scale';
side = 'Left';
[VL, info] = nag_lapack_dggbak( ...
		    job, side, jlo, jhi, lscale, rscale, q);
side = 'Right';
[VR, info] = nag_lapack_dggbak( ...
		    job, side, jlo, jhi, lscale, rscale, z);

disp('Left Eigenvectors');
disp(VL);

disp('Right Eigenvectors');
disp(VR);

% diary off
