function nag_lapack_dggqrf_example

% diary nag_lapack_dggqrf_example.r
fprintf('nag_lapack_dggqrf example results\n\n');

a = [-0.57, -1.28, -0.39;
     -1.93,  1.08, -0.31;
      2.3,   0.24, -0.4;
     -0.02,  1.03, -1.43];
b = [0.5, 0, 0, 0;
     0,   1, 0, 0;
     0,   0, 2, 0;
     0,   0, 0, 5];
d = [1.32; -4.00; 5.52; 3.24];

n = 4;
m = 3;
p = 4;

% Compute the generalized QR factorization of (A,B) as
% A = Q*(R),   B = Q*(T11 T12)*Z
%       (0)          ( 0  T22)

[a, taua, b, taub, info] = nag_lapack_dggqrf( ...
                                  a, b);

% Compute c = (c1) = (Q**T)*d
%             (c2)

[c, info] = nag_lapack_dormqr( ...
                   'Left', 'Transpose', a, taua, d);

% Putting Z*y = w = (w1), set w1 = 0, storing the result in y1
%                   (w2)
y = zeros(p, 1);
if n > m
  % Solve T22*w2 = c2 for w2, storing result in y2
  y2 = m+p-n+1:p;

  [y(y2), info] = ...
  nag_lapack_dtrtrs( ...
         'Upper', 'No transpose', 'Non-unit', b(m+1:n, y2), c(m+1:n));

  % Compute estimate of the square root of the residual sum of squares
  % norm(y) = norm(w2)
  rnorm = norm(y(y2), 1);

  % Form c1 - T12*w2 in c
  c = c - b(:,y2)*y(y2);
end

% Solve R*x = c1 - T12*w2 for x
[c(1:m), info] = nag_lapack_dtrtrs( ...
                        'Upper', 'No transpose', 'Non-unit', a(1:m,:), c(1:m));

% Compute y = (Z^T)*w
b1 = max(1, n-p+1):n;
[b(b1,:), y, info] = nag_lapack_dormrq( ...
                            'Left', 'Transpose', b(b1,:), taub, y);

fprintf('Generalized least squares solution\n');
fprintf('%12.4f',c(1:m));

fprintf('\n\nResidual vector\n  ');
fprintf('%12.2e',y);

fprintf('\n\nSquare root of the residual sum of squares\n  %12.2e\n', ...
        rnorm);

% diary off
