function nag_lapack_dggrqf_example

% diary nag_lapack_dggrqf_example.r
fprintf('nag_lapack_dggrqf example results\n\n');

% Find x that minimizes norm(c-Ax) subject to Bx = d .

m = nag_int(6);
n = nag_int(4);
p = nag_int(2);
a = [-0.57, -1.28, -0.39,  0.25;
     -1.93,  1.08, -0.31, -2.14;
      2.30,  0.24,  0.40, -0.35;
     -1.93,  0.64, -0.66,  0.08;
      0.15,  0.30,  0.15, -2.13;
     -0.02,  1.03, -1.43,  0.50];
b = [1, 0, -1, 0;
     0, 1, 0, -1];
c = [-1.50; -2.14;  1.23; -0.54; -1.68; 0.82];
d = [0; 0];

% Compute the generalized RQ factorization of (B,A) as
% A = ZRQ, B = TQ
[TQ, taub, ZR, taua, info] = nag_lapack_dggrqf( ...
                                    b, a);

% Set Qx = y. The problem reduces to:
% minimize (Ry - Z^Tc) subject to Ty = d

% Update c = Z^T*c -> minimize (Ry-c)
[cup, info] = nag_lapack_dormqr( ...
                     'Left','Transpose',ZR,taua,c);

% Solve Ty = d for last p elements
T12 = triu(TQ(1:p,n-p+1:n));

[y2, info] = nag_lapack_dtrtrs( ...
                    'Upper', 'No transpose', 'Non-unit', T12, d);

% (from Ry-c) R11*y1 + R12*y2 = c1 --> R11*y1 = c1 - R12*y2
% Update c1
c1 = cup(1:n-p) - ZR(1:n-p,n-p+1:n)*y2;

% Solve R11*y1 = c1 for y1
R11 = triu(ZR(1:n-p,1:n-p));
[y1, info] = nag_lapack_dtrtrs( ...
                    'Upper', 'No transpose', 'Non-unit', R11, c1);

% Contruct y and backtransform for x = Q^Ty
y = [y1;y2];
[~, x, info] = nag_lapack_dormrq( ...
                      'Left', 'Transpose', TQ, taub, y);

fprintf('Constrained least squares solution\n');
disp(x);
res = a*x - c;
fprintf('Residual norm\n%12.2e\n', norm(res));

% diary off
