function nag_lapack_dggsvd_example

% diary nag_lapack_dggsvd_example.r
fprintf('nag_lapack_dggsvd example results\n\n');

% Generalized SVD of (A, B), where
m = 4;
n = 3;
p = 2;
a = [ 1, 2, 3;
      3, 2, 1;
      4, 5, 6;
      7, 8, 8];
b = [-2, -3, 3;
      4,  6, 5];

% Compute the GSVD of matrix pair A, B
% (a, b) = (U*D1, V*D2)*(0 R)*(Q^T), m>=n
jobu = 'U';
jobv = 'V';
jobq = 'Q';
[k, l, DR, b, alpha, beta, U, V, Q, iwork, info] = ...
  nag_lapack_dggsvd( ...
	 jobu, jobv, jobq, a, b);

irank = k + l;
fprintf('\nInfinite generalized singular values = %5d\n', k);
fprintf('  Finite generalized singular values = %5d\n', l);
fprintf('  Numerical rank of (a'' b'')''         = %5d\n', irank);

fprintf('\nFinite generalized singular values\n');
w = alpha(k+1:irank)./beta(k+1:irank);
disp(transpose(w));

fprintf('Orthogonal matrix U\n');
disp(U);

fprintf('Orthogonal matrix V\n');
disp(V);

fprintf('Orthogonal matrix Q\n');
disp(Q);

fprintf('Non singular upper triangular matrix R\n');
R = DR(1:n,n-(irank-1):n);
disp(R);

% Estimate the reciprocal condition number of R
[rcond, info] = nag_lapack_dtrcon( ...
		       'Inf-norm','Upper','Non-unit', R);

fprintf('Condition number for R:\n%11.1e\n', 1/rcond);

% So long as irank = n, compute error bound for singular values
if (irank==n)
  serrbd = nag_machine_precision/rcond;
  fprintf('\nError bound for the generalized singular values\n%11.1e\n', ...
          serrbd);
else
  fprintf('\n(A'' B'')'' is not of full rank\n');
end

% diary off
