function nag_lapack_dormbr_example

% diary nag_lapack_dormbr_example.r
fprintf('nag_lapack_dormbr example results\n\n');

% Two cases of preceding reduction to bidiagonal form by QR or LQ
% Case 1: m > n, precede by QR
ex1;
% Case 2: m < n, precede by LQ
ex2;

% diary off

function ex1
  m = 6;
  n = nag_int(4);
  a = [-0.57  -1.28  -0.39   0.25;
       -1.93   1.08  -0.31  -2.14;
        2.30   0.24   0.40  -0.35;
       -1.93   0.64  -0.66   0.08;
        0.15   0.30   0.15  -2.13;
       -0.02   1.03  -1.43   0.50];
  % Factorize A = QR
  [QR, tau, info] = nag_lapack_dgeqrf(a);

  % Generate Q from QR
  [Q, info] = nag_lapack_dorgqr(QR, tau);

  % Extract R from QR
  R = triu(QR(1:n,1:n));

  % Bidiagonalize R = Q1 B P^T
  [B, d, e, tauq, taup, info] = ...
  nag_lapack_dgebrd(R);

  % Update Q: Q2 = Q*Q1 (so A = QR = Q2 B P^T)
  vect = 'Q';
  side = 'Right';
  trans = 'No transpose';
  [Q2, info] = nag_lapack_dormbr( ...
                      vect, side, trans, n, B, tauq, Q);

  fprintf('Example 1: bidiagonal matrix B\n   Main diagonal  ');
  fprintf(' %7.3f',d);
  fprintf('\n   super-diagonal ');
  fprintf(' %7.3f',e);
  fprintf('\n\n');
  disp('Example 1: Orthogonal matrix Q');
  disp(Q2);

function ex2
  m = nag_int(4);
  n = nag_int(6);
  a = [ -5.42   3.28  -3.68   0.27   2.06   0.46;
        -1.65  -3.40  -3.20  -1.03  -4.06  -0.01;
        -0.37   2.35   1.90   4.31  -1.76   1.13;
        -3.15  -0.11   1.99  -2.70   0.26   4.50];

  % Factorize A = LQ
  [LQ, tau, info] = nag_lapack_dgelqf(a);

  % Generate Q from LQ
  [Q, info] = nag_lapack_dorglq(LQ, tau);

  % Extract L from LQ
  L = tril(LQ(1:m,1:m));

  % Bidiagonalize L = Q1 B P^T
  [B, d, e, tauq, taup, info] = ...
  nag_lapack_dgebrd(L);

  % Update Q: P2 = P^T*Q (so A = LQ = Q1 B P2)
  vect = 'P';
  side = 'Left';
  trans = 'Transpose';
  [P2, info] = nag_lapack_dormbr( ...
                      vect, side, trans, n, B, taup, Q);

  fprintf('Example 2: bidiagonal matrix B\n   Main diagonal  ');
  fprintf(' %7.3f',d);
  fprintf('\n   super-diagonal ');
  fprintf(' %7.3f',e);
  fprintf('\n\n');
  disp('Example 2: Orthogonal matrix P^T');
  disp(P2);
