function nag_lapack_dspgvd_example

% diary nag_lapack_dspgvd_example.r
fprintf('nag_lapack_dspgvd example results\n\n');

% Symmetric matrices A and B stored in packed (Upper) format
n = nag_int(4);
uplo = 'U';
ap = [0.24;
      0.39; -0.11;
      0.42;  0.79; -0.25;
     -0.16;  0.63;  0.48; -0.03];
bp = [4.16;
     -3.12;  5.03;
      0.56; -0.83;  0.76;
      -0.10;  1.09;  0.34;  1.18];

% Generalized eigenvalues for ABz = lambda z
itype = nag_int(2);
jobz = 'No vectors';
[~, L, w, z, info] = nag_lapack_dspgvd( ...
                            itype, jobz, uplo, n, ap, bp);

disp('Eigenvalues');
disp(w');

% Calculate condition number and error estimates
% Unpack ap and bp to calculate 1 norms of A and B
[a, ap, ifail] = nag_matop_real_tri_pack( ...
                        'Unpack', uplo, 'N', zeros(n,n), ap);
[b, bp, ifail] = nag_matop_real_tri_pack( ...
                        'Unpack', uplo, 'N', zeros(n,n), bp);
anorm = norm(a + a' - diag(diag(a)),1);
bnorm = norm(b + b' - diag(diag(b)),1);

% Estimate condition number
[rcond, info] = nag_lapack_dtpcon( ...
                       '1', uplo, 'N', n, L);

rcondb = rcond^2;
fprintf('Estimate of reciprocal condition number for B\n%12.1e\n', rcondb);

% Error bounds
t1 = nag_machine_precision/rcondb;
t2 = anorm*bnorm;
errbnd(1:n) = nag_machine_precision*t2 + t1.*abs(w);

fprintf('\n');
disp('Error estimate for the eigenvalues');
fprintf('%12.1e',errbnd);
fprintf('\n');

% diary off
