function nag_lapack_zgeqp3_example

% diary nag_lapack_zgeqp3_example.r
fprintf('nag_lapack_zgeqp3 example results\n\n');

a = [ 0.47 - 0.34i, -0.40 + 0.54i,  0.60 + 0.01i,  0.80 - 1.02i;
     -0.32 - 0.23i, -0.05 + 0.20i, -0.26 - 0.44i, -0.43 + 0.17i;
      0.35 - 0.60i, -0.52 - 0.34i,  0.87 - 0.11i, -0.34 - 0.09i;
      0.89 + 0.71i, -0.45 - 0.45i, -0.02 - 0.57i,  1.14 - 0.78i;
     -0.19 + 0.06i,  0.11 - 0.85i,  1.44 + 0.80i,  0.07 + 1.14i];

b = [-1.08 - 2.59i,  2.22 + 2.35i;
     -2.61 - 1.49i,  1.62 - 1.48i;
      3.13 - 3.61i,  1.65 + 3.43i;
      7.33 - 8.01i, -0.98 + 3.08i;
      9.12 + 7.63i, -2.84 + 2.78i];
[m,n] = size(a);
nrhs = size(b,2);
jpvt = zeros(n,1,nag_int_name);

% Compute the QR factorization of a
[qr, jpvt, tau, info] = nag_lapack_zgeqp3( ...
			      a, jpvt);

% Compute C = (C1) = (Q^H)*b, storing the result in c
%             (C2)
[c, info] = nag_lapack_zunmqr( ...
		   'Left', 'Conjugate Transpose', qr, tau, b);

% Choose tol to reflect the relative accuracy of the input data
tol = 0.01;

% Determine and print the rank, k, of r relative to tol
k = find(abs(diag(qr)) <= tol*abs(qr(1,1)));
if numel(k) == 0
  k = numel(diag(qr));
else
  k = k(1)-1;
end

fprintf('\nTolerance used to estimate the rank of a\n     %11.2e\n', tol);
fprintf('Estimated rank of a\n        %d\n', k);

% Compute least-squares solution by backsubstitution in r(1:k, 1:k)*c = c1
c1 = zeros(m, nrhs);
c1(1:k, :) = inv(triu(qr(1:k,1:k)))*c(1:k,:);

% Compute estimates of the square roots of the residual sums of
% squares (2-norm of each of the columns of C2)
for j = 1:nrhs
  rnorm(j) = norm(c(k+1:m,j));
end

% Permute the least-squares solutions stored in c1 to give x = p*y
x = zeros(n, nrhs);
for i=1:n
   x(jpvt(i), :) = c1(i, :);
end
fprintf('\nLeast-squares solution(s)\n');
disp(x);
fprintf('Square root(s) of the residual sum(s) of squares\n');
disp(rnorm);

% diary off
