function nag_lapack_zggevx_example

% diary nag_lapack_zggevx_example.r
fprintf('nag_lapack_zggevx example results\n\n');

% Generalized eigenvalues and right eigenvectors of (A, B):
n = 4;
a = [ -21.10 - 22.50i,  53.5 - 50.5i, -34.5 + 127.5i,    7.5 +  0.5i;
       -0.46 -  7.78i,  -3.5 - 37.5i, -15.5 +  58.5i,  -10.5 -  1.5i;
        4.30 -  5.50i,  39.7 - 17.1i, -68.5 +  12.5i,   -7.5 -  3.5i;
        5.50 +  4.40i,  14.4 + 43.3i, -32.5 -  46.00i, -19.0 - 32.5i];
b = [   1    -  5i,      1.6 +  1.2i,  -3   +   0i,      0   -  1i;
        0.8  -  0.6i,    3   -  5i,    -4   +   3i,     -2.4 -  3.2i;
        1    +  0i,      2.4 +  1.8i,  -4   -   5i,      0   -  3i;
        0    +  1i,     -1.8 +  2.4i,   0   -   4i,      4   -  5i];

balanc = 'Balance';
jobvl = 'No left vectors';
jobvr = 'Vectors (right)';
sense = 'Both reciprocal condition numbers';
[~, ~, alpha, beta, ~, VR, ilo, ihi, lscale, rscale, abnrm, bbnrm, ...
	    rconde, rcondv, info] = ...
  nag_lapack_zggevx( ...
	 balanc, jobvl, jobvr, sense, a, b);

epsilon = nag_machine_precision;
small = nag_machine_real_safe;
absnrm = sqrt(abnrm^2+bbnrm^2);
tol = epsilon*absnrm;

for j=1:n

  % display information on the jth eigenvalue
  if (abs(alpha(j))*small >= abs(beta(j)))
    fprintf('\n%4d: Eigenvalue is numerically infinite or undetermined\n',j);
    fprintf('%4d: alpha = (%11.4e,%11.4e), beta = (%11.4e,%11.4e)\n', ...
            j, real(alpha(j)), imag(alpha(j)), real(beta(j)),imag(beta(j)));
  else
    fprintf('Eigenvalue  (%d)\n', j);
    disp(alpha(j)/beta(j));
  end

  if rconde(j) > 0
    fprintf(' Condition number            = %8.1e\n', 1/rconde(j));
    fprintf(' Error bound                 = %8.1e\n', tol/rconde(j));
  else
    fprintf(' Reciprocal condition number = %8.1e\n', rconde(j));
    fprintf(' Error bound is infinite\n');
  end

  fprintf('\nEigenvector (%d):\n', j);
  disp(VR(:, j));

  if rcondv(j) > 0
    fprintf(' Condition number            = %8.1e\n', 1/rcondv(j));
    fprintf(' Error bound                 = %8.1e\n\n', tol/rcondv(j));
  else
    fprintf(' Reciprocal condition number = %8.1e\n', rcondv(j));
    fprintf(' Error bound is infinite\n\n');
  end
end

% diary off
