function nag_lapack_zggsvd_example

% diary nag_lapack_zggsvd_example.r
fprintf('nag_lapack_zggsvd example results\n\n');

% Generalized SVD of (A, B), where
m = 6;
n = 4;
a = [ 0.96 - 0.81i, -0.03 + 0.96i, -0.91 + 2.06i, -0.05 + 0.41i;
     -0.98 + 1.98i, -1.20 + 0.19i, -0.66 + 0.42i, -0.81 + 0.56i;
      0.62 - 0.46i,  1.01 + 0.02i,  0.63 - 0.17i, -1.11 + 0.60i;
      0.37 + 0.38i,  0.19 - 0.54i, -0.98 - 0.36i,  0.22 - 0.20i;
      0.83 + 0.51i,  0.20 + 0.01i, -0.17 - 0.46i,  1.47 + 1.59i;
      1.08 - 0.28i,  0.20 - 0.12i, -0.07 + 1.23i,  0.26 + 0.26i];
b = complex([ 1,  0, -1,  0;
              0,  1,  0, -1]);

% Compute the GSVD of (a, b)
  % (a,b) = (U*D1,v*D2)*(0 R)*(Q^H), m>=n
jobu = 'U';
jobv = 'V';
jobq = 'Q';
[k, l, DR, b, alpha, beta, U, V, Q, iwork, info] = ...
  nag_lapack_zggsvd( ...
	 jobu, jobv, jobq, a, b);

irank = k + l;
fprintf('\nInfinite generalized singular values = %5d\n', k);
fprintf('  Finite generalized singular values = %5d\n', l);
fprintf('  Numerical rank of (a'' b'')''         = %5d\n', irank);

fprintf('\nFinite generalized singular values\n');
w = alpha(k+1:irank)./beta(k+1:irank);
disp(transpose(w));

fprintf('Orthogonal matrix u\n');
disp(U);

fprintf('Orthogonal matrix v\n');
disp(V);

fprintf('Orthogonal matrix q\n');
disp(Q);

fprintf('Non singular upper triangular matrix R\n');
R = DR(1:n,n-(irank-1):n);
disp(R);

% Estimate the reciprocal condition number of R
[rcond, info] = nag_lapack_ztrcon( ...
		       'Infinity-norm','Upper','Non-unit', R);

fprintf('Estimate of reciprocal condition number for R\n%11.1e\n', rcond);

% If irank = n, compute error bound for generalized singular values
if (irank==n)
  serrbd = nag_machine_precision/rcond;
  fprintf('\nError bound for the generalized singular values\n%11.1e\n', ...
          serrbd);
else
  fprintf('\n(A^H B^H)^H is not of full rank\n');
end

% diary off
