function nag_lapack_zhbev_example

% diary nag_lapack_zhbev_example.r
fprintf('nag_lapack_zhbev example results\n\n');

% Hermitian band matrix A, stored on symmetric banded format
uplo = 'U';
kd   = nag_int(2);
n    = nag_int(4);
ab = [0,       0 + 0i,  3 - 1i,  4 - 2i,  5 - 3i;
      0 + 0i,  2 - 1i,  3 - 2i,  4 - 3i,  5 - 4i;
      1 + 0i,  2 + 0i,  3 + 0i,  4 + 0i,  5 + 0i];

% All eigenvalues and eigenvectors of A
jobz = 'Vectors';
[~, w, z, info] = nag_lapack_zhbev( ...
                         jobz, uplo, kd, ab);

% Normalize: largest elements are real
for i = 1:n
  [~,k] = max(abs(real(z(:,i)))+abs(imag(z(:,i))));
  z(:,i) = z(:,i)*conj(z(k,i))/abs(z(k,i));
end

disp('Eigenvalues');
disp(w');
[ifail] = nag_file_print_matrix_complex_gen( ...
                 'General', ' ', z, 'Eigenvectors');

% Eigenvalue error bound
errbnd = nag_machine_precision*max(abs(w(1)),abs(w(end)));
% Eigenvector condition numbers
[rcondz, info] = nag_lapack_ddisna( ...
		        'Eigenvectors', n, n, w);

% Eigenvector error bounds
zerrbd = errbnd./rcondz;

fprintf('\nError estimate for the eigenvalues\n');
fprintf('%12.1e\n',errbnd);
disp('Error estimates for the eigenvectors');
fprintf('%12.1e',zerrbd);
fprintf('\n');

% diary off
