function nag_lapack_ztgsja_example

% diary nag_lapack_ztgsja_example.r
fprintf('nag_lapack_ztgsja example results\n\n');

% Compute generalized SVD of (A,B)
A = [ 0.96-0.81i -0.03+0.96i -0.91+2.06i -0.05+0.41i;
     -0.98+1.98i -1.20+0.19i -0.66+0.42i -0.81+0.56i;
      0.62-0.46i  1.01+0.02i  0.63-0.17i -1.11+0.60i;
      0.37+0.38i  0.19-0.54i -0.98-0.36i  0.22-0.20i;
      0.83+0.51i  0.20+0.01i -0.17-0.46i  1.47+1.59i;
      1.08-0.28i  0.20-0.12i -0.07+1.23i  0.26+0.26i];
B = complex([ 1 0 -1  0;
              0 1  0 -1]); 
[m, n] = size(A);
p      = size(B,1);

% Reduce A and B to upper triangular form S = U^H A Q, T = V^H B Q
tola = max(m,n)*norm(A,1)*nag_machine_precision;
tolb = max(p,n)*norm(B,1)*nag_machine_precision;
[S, T, k, l, U, V, Q, info] = ...
  nag_lapack_zggsvp(...
        'U', 'V', 'Q', A, B, tola, tolb);

% Compute singular values
[S, T, alpha, beta, U, V, Q, ncycle, info] = ...
  nag_lapack_ztgsja('U', 'V', 'Q', k, l, S, T, tola, tolb, U, V, Q);

fprintf('Number of infinite generalized singular values = %3d\n',k);
fprintf('Number of   finite generalized singular values = %3d\n',l);
fprintf('Effective rank of the matrix pair  (A^T B^T)^T = %3d\n',k+l);
fprintf('Number of cycles of the Kogbetliantz method    = %3d\n\n',ncycle);
disp('Finite generalized singular values');
disp(alpha(k+1:k+l)./beta(k+1:k+l));

% diary off
