function nag_lapack_ztzrzf_example

% diary nag_lapack_ztzrzf_example.r
fprintf('nag_lapack_ztzrzf example results\n\n');

% Find Least squares solution of Ax = B, m>n
m = 5;
n = 4;
a = [  0.47 - 0.34i, -0.40 + 0.54i,  0.60 + 0.01i,  0.80 - 1.02i;
      -0.32 - 0.23i, -0.05 + 0.2i,  -0.26 - 0.44i, -0.43 + 0.17i;
       0.35 - 0.60i, -0.52 - 0.34i,  0.87 - 0.11i, -0.34 - 0.09i;
       0.89 + 0.71i, -0.45 - 0.45i, -0.02 - 0.57i,  1.14 - 0.78i;
      -0.19 + 0.06i,  0.11 - 0.85i,  1.44 + 0.80i,  0.07 + 1.14i];

b = [ -1.08 - 2.59i,  2.22 + 2.35i;
      -2.61 - 1.49i,  1.62 - 1.48i;
       3.13 - 3.61i,  1.65 + 3.43i;
       7.33 - 8.01i, -0.98 + 3.08i;
       9.12 + 7.63i, -2.84 + 2.78i];

% QR factorization of A with column pivoting = Q*(R1 R2 )*(P^T)
%                                                (0  R22)
[qr, jpvt, tau, info] = nag_lapack_zgeqp3( ...
			       a, zeros(n,1,nag_int_name));

% QRP'X = B, => RP'X = Q^HB = C; Compute C = Q^H B
[c, info] = nag_lapack_zunmqr( ...
		   'Left', 'Conjugate Transpose', qr, tau, b);

% Determine the rank, k, of R relative to tol;
% Choose tol to reflect the relative accuracy of the input data
tol = 0.01;
k = find(abs(diag(qr)) <= tol*abs(qr(1,1)));
if numel(k) == 0
  k = numel(diag(qr));
else
  k = k(1)-1;
end

fprintf('\nTolerance used to estimate the rank of a\n     %11.2e\n', tol);
fprintf('Estimated rank of a\n        %d\n', k);

% Compute the RZ (TZ) factorization of the first k rows of (R1 R2)
[rz, taurz, info] = nag_lapack_ztzrzf( ...
			   qr(1:k,:));

% Now, (TZ)P'X = C on first k rows of C
% Let ZP'X = T^{-1}C = Y (on first k rows)
y = zeros(n, 2);
y(1:k, :) = inv(triu(rz(1:k,1:k)))*c(1:k,:);

% ZP^T X = Y => P^T X = Z^H Y = W; Form W = Z^H Y.
[w, info] = nag_lapack_zunmrz( ...
		   'Left', 'ConjTrans', nag_int(n-k), rz, taurz, y);

% P^T X = W => X = PW,
x = zeros(n, 2);
for i=1:n
   x(jpvt(i), :) = w(i, :);
end
fprintf('\nLeast-squares solution(s)\n');
disp(x);

% Compute estimates of the square roots of the residual sums of
% squares (2-norm of each of the columns of C2)
rnorm = [norm(c(k+1:m,1)), norm(c(k+1:m,2))];
fprintf('Square root(s) of the residual sum(s) of squares\n');
disp(rnorm);

% diary off
