function nag_linsys_real_norm_rcomm_example

% diary nag_linsys_real_norm_rcomm_example.r
fprintf('nag_linsys_real_norm_rcomm example results\n\n');

ex1;
ex2;

% diary off

function ex1
  fprintf('Example 1:\n\n');
  % Estimate condition number of A
  a = [ 1.5,  2.0,  3.0, -2.1,  0.3;
        2.5,  3.0, -4.0,  2.3, -1.1;
        3.5,  4.0,  0.5, -3.1, -1.4;
       -0.4, -3.2, -2.1,  3.1,  2.1;
        1.7,  3.7,  1.9, -2.2, -3.3];
  anorm = norm(a,1);
  ainv = inv(a);

  % reverse communication initializations
  icase = nag_int(0);
  estnrm = 0;
  x     = zeros(5, 1);
  work  = zeros(5, 1);
  iwork = zeros(5, 1, nag_int_name);

  % reverse communication loop
  done = false;
  while (~done)
    [icase, x, estnrm, work, iwork, ifail] = ...
    nag_linsys_real_norm_rcomm(icase, x, estnrm, work, iwork);
    if (icase == 0)
      done = true;
    elseif (icase == 1)
      x = ainv*x;
    elseif (icase == 2)
      x = ainv'*x;
    end
  end

  fprintf('Computed norm of A              = %8.4f\n', anorm);
  fprintf('Estimated norm of inverse(A)    = %8.4f\n', estnrm);
  cond = anorm*estnrm;
  fprintf('Estimated condition number of A = %8.1f\n', cond);

function ex2
  fprintf('\nExample 2:\n\n');
  % Estimate condition number of sparse A
  n  = nag_int(6);
  nz = nag_int(15);
  a   = zeros(150,1);
  irn = zeros(75,1,nag_int_name);
  icn = zeros(150,1,nag_int_name);
  a(1:nz) = [ 5                       ...
                  2  -1   2           ...
                      3               ...
             -2           1   1       ...
             -1          -1   2  -3   ...
             -1  -1               6];
  anorm = 9;
  irn(1:15) = [nag_int(1); 2;2;2; 3; 4;4;4; 5;5;5;5; 6;6;6];
  icn(1:15) = [nag_int(1); 2;3;4; 3; 1;4;5; 1;4;5;6; 1;2;6];

  % Factorize A
  abort     = [true;     true;     false;     true];
  [a, irn, icn, ikeep, w, idisp, ifail] = ...
  nag_matop_real_gen_sparse_lu( ...
         n, nz, a, irn, icn, abort);

  % reverse communication initializations
  icase = nag_int(0);
  estnrm = 0;
  x     = zeros(n, 1);
  work  = zeros(n, 1);
  iwork = zeros(n, 1, nag_int_name);
  
  % reverse communication loop
  done = false;
  while (~done)
    [icase, x, estnrm, work, iwork, ifail] = ...
    nag_linsys_real_norm_rcomm(icase, x, estnrm, work, iwork);
    if (icase == 0)
      done = true;
    else
      % Solve Ax = b or A'x = b
      [x, resid] = nag_linsys_real_sparse_fac_solve( ...
                          a, icn, ikeep, x, icase, idisp);
    end
  end
  
  fprintf('Computed norm of A              = %8.4f\n', anorm);
  fprintf('Estimated norm of inverse(A)    = %8.4f\n', estnrm);
  cond = anorm*estnrm;
  fprintf('Estimated condition number of A = %8.1f\n', cond);
