function nag_nonpar_prob_mwu_ties_example

% diary nag_nonpar_prob_mwu_ties_example.r
fprintf('nag_nonpar_prob_mwu_ties example results\n\n');

x = [13;  6; 12;  7; 12;  7; 10;  7;
     10;  7; 16;  7; 10;  8;  9;  8];
y = [17;  6; 10;  8; 15;  8; 15; 10;
     15; 10; 14; 10; 14; 11; 14; 11;
     13; 12; 13; 12; 13; 12; 12];
n1 = nag_int(numel(x));
n2 = nag_int(numel(y));

fprintf('Mann-Whitney U test\n\n');
fprintf('Sample size of group 1 = %5d\n', n1);
fprintf('Sample size of group 2 = %5d\n\n', n2);
fprintf('Data values\n');
fprintf('\n     Group 1 ');
for j = 1:floor(n1/8)
  i1 = (j-1)*8 + 1;
  i2 = min(n1,i1+7);
  fprintf('%4.0f',x(i1:i2));
  fprintf('\n             ');
end
fprintf('\n     Group 2 ');
for j = 1:floor(n2/8)
  i1 = (j-1)*8 + 1;
  i2 = min(n2,i1+7);
  fprintf('%4.0f',y(i1:i2));
  fprintf('\n             ');
end

% Perform test
tail = 'Lower-tail';
[u, unor, p, ties, ranks, ifail] =  ...
  nag_nonpar_test_mwu(x, y, tail);

% Calculate exact probabilities
if ties
  [pexact, ifail] = nag_nonpar_prob_mwu_ties( ...
                      n1, n2, tail, ranks, u);
else
  [pexact, ifail] = nag_nonpar_prob_mwu_noties( ...
                      n1, n2, tail, u);
end
   
fprintf('\nTest statistic            = %8.4f\n', u);
fprintf('Normalized test statistic = %8.4f\n', unor);
fprintf('Approx. tail probability  = %8.4f\n\n', p);

fprintf('\nRanks\n\n');
fprintf('     Group 1 ');
for j = 1:floor(n1/8)
  i1 = (j-1)*8 + 1;
  i2 = min(n1,i1+7);
  fprintf('%5.1f',ranks(i1:i2));
  fprintf('\n             ');
end
fprintf('\n     Group 2 ');
for j = 1:floor(n2/8)
  i1 = (j-1)*8 + 1;
  i2 = min(n2,i1+7);
  fprintf('%5.1f',ranks(i1+n1:i2+n1));
  fprintf('\n             ');
end

fprintf('\nExact tail probability    = %8.4f\n', pexact);

% diary off
