function nag_numdiff_rcomm_example

% diary nag_numdiff_rcomm_example.r
fprintf('nag_numdiff_rcomm example results\n\n');

fprintf('Find the derivatives of the polygamma (psi) function\n');
fprintf('using function values generated by nag_specfun_psi_deriv_real.\n\n');
fprintf('Demonstrate the effect of successively reducing hbase.\n\n');

% Set the target location and calculate the objective value
x_0 = 0.05;
[f_0, ifail] = nag_specfun_psi_deriv_real(x_0, nag_int(0));

% Compute the actual derivatives using nag_specfun_psi_deriv_real for comparison
actder = zeros(3, 1);
for j=1:3
  [actder(j), ifail] = nag_specfun_psi_deriv_real(x_0, nag_int(j));
end

der_comp = zeros(14, 3, 4);
% Attempt 4 applications, reducing hbase by factor 0.1 each time
for j=1:4
  % Generate the abscissa xval using nag_numdiff_sample
  hbase = 0.025*10^(-j);
  [xval] = nag_numdiff_sample(x_0, hbase);

  % Calculate the corresponding objective function values
  fval(11) = f_0;
  for k=[1:10,12:21]
    [fval(k), ifail] = nag_specfun_psi_deriv_real(xval(k), nag_int(0));
  end

  % Call nag_numdiff_rcomm to calculate the derivative estimates
  [der, erest, ifail] = nag_numdiff_rcomm(xval, fval);

  % Store results in der_comp
  der_comp(:, 1, j) = hbase;
  der_comp(:, 2, j) = der;
  der_comp(:, 3, j) = erest;

end

% Display results for first 3 derivatives
for i=1:3
  fprintf('\nderivative %d calculated using nag_specfun_psi_deriv_real: %11.4e\n', i, actder(i));
  fprintf('Derivative and error estimates for derivative %d\n', i);
  fprintf('      hbase        der(%d)      erest(%d)\n', i, i);
  for j=1:4
    fprintf('  %12.4e %12.4e %12.4e\n', der_comp(i,:,j));
  end
end

% diary off
