function nag_ode_bvp_coll_nth_comp_example

% diary nag_ode_bvp_coll_nth_comp_example.r
fprintf('nag_ode_bvp_coll_nth_comp example results\n\n');

global b x0 x1 ndegree

% Initialize variables and arrays.
n       = 2;
nderiv  = nag_int([1 2]);
nbc     = nderiv;
x0      = -1;
x1      = 1;
ncoeff  = nag_int(9);
ncolloc = nag_int(15);
ndegree = ncoeff-1;

% Iterate to covergence within tolerance of 1.0e0-5.
iter = 0;
emax = 1.0;
b = zeros(ncoeff, n);
b(1,2) = 6.0;
while emax >= 1.0e-05
  iter = iter + 1;

  [c, ifail] = nag_ode_bvp_coll_nth_comp(...
                     nderiv, nbc, x0, x1, ncoeff, ncolloc, ...
                     @coeff, @bdyc);
  % Output coefficients.
  fprintf('\n Iteration %2d. Chebyshev coefficients are:\n',iter);
  for j = 1:n
    fprintf('%8.4f',c(1:ncoeff,j));
    fprintf('\n');
  end
  
  emax = max(abs(c - b));
  b = c;
end

% Use these coefficients to evaluate the solution.
npt = 9;
fprintf('\n Solution evaluated at %d equally spaced points.\n\n', npt);
fprintf('%7s%16s\n','x','y');

% Prepare to store results for plotting.
dx = (x1-x0)/(double(npt-1));
xarray = [x0:dx:x1];
yarray = zeros(npt, n);

for ipt = 1:npt
  xx = xarray(ipt);
  fprintf('%10.4f', xx);
  for jeqn = 1:n
    [yarray(ipt, jeqn), ifail] = ...
    nag_fit_1dcheb_eval2(...
           ndegree, x0, x1, c(:,jeqn), nag_int(1), xx);
    fprintf('%10.4f', yarray(ipt, jeqn));
  end
  fprintf('\n');
end

% Plot results.
fig1 = figure;
display_plot(xarray, yarray);
% print(fig1,'-dpng','-r75','nag_ode_bvp_coll_nth_comp_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_coll_nth_comp_fig1.eps');

% diary off

function [x, a, rhs] = bdyc(ii, jj, a, ia, ia1, rhs)
  % Evaluate coefficient functions and rhs of boundary conditions.

  x = -1;
  a(ii,jj) = 1;
  if (ii == 2 && jj == 1)
    rhs = 3;
  end

function [a, rhs] = coeff(x, ii, a, ia, ia1, rhs)
  % Evaluate coefficient functions and rhs of differential equations.

  global b x0 x1 ndegree

  if (ii <= 1)
    [z1, ifail] = nag_fit_1dcheb_eval2(ndegree, x0, x1, b(:,1), nag_int(1), x);
    [z2, ifail] = nag_fit_1dcheb_eval2(ndegree, x0, x1, b(:,2), nag_int(1), x);
    a(1,1) = z2*z2 - 1;
    a(1,2) = 2;
    a(2,1) = 2*z1*z2 + 1;
    rhs = 2*z1*z2*z2;
  else
    a(1,2) = -1;
    a(2,3) =  2;
  end

function display_plot(x, y)
  % Plot both curves.
  [haxes, hline1, hline2] = plotyy(x, y(:,1), x, y(:,2));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [-0.5 0.1]);
  set(haxes(1), 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [-0.5:0.1:0]);
  set(haxes(2), 'YLim', [2.6 3.2]);
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(2), 'YTick', [2.6:0.1:3.2]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [-1 1]);
  end
  % Add title.
  title('Solution by Chebyshev Collocation');
  % Label the axes.
  xlabel('x');
  ylabel(haxes(1), 'y_1');
  ylabel(haxes(2), 'y_2');
  % Add a legend.
  legend('y_1','y_2','Location','Best')
  % Set some features of the three lines.
  set(hline1, 'Linewidth', 0.25, 'Marker', '+', 'LineStyle', '-');
  set(hline2, 'Linewidth', 0.25, 'Marker', 'x', 'LineStyle', '--');
