function nag_ode_bvp_ps_lin_solve_example

% diary nag_ode_bvp_ps_lin_solve_example.r
fprintf('nag_ode_bvp_ps_lin_solve example results\n\n');

n = nag_int(16);
a = -pi/2;
b =  pi/2;


% Set up boundary condition on left side of domain
y = [a, b];
% Set up Dirichlet condition using exact solution at x=a.
bmat = zeros(2, 3);
bmat(1, 1:2) = [1, 1];
bmat(2, 1:2) = [1, 1];
bvec = [cos(a) - sin(a), cos(b) - sin(b)];

% Set up problem definition
f = [1, 2, 3];

% Set up solution grid
[x, ifail] = nag_ode_bvp_ps_lin_cgl_grid(n, a, b);

% Set up problem right hand sides for grid and transform
f0 = -2*sin(x) - 2*cos(x);
[c, ifail] = nag_ode_bvp_ps_lin_coeffs(n, f0);

% Solve in coefficient space
[bmat, f, uc, resid, ifail] = nag_ode_bvp_ps_lin_solve(n, a, b, c, bmat, y, bvec, f);

% Evaluate solution and derivatives on Chebyshev grid
u = zeros(n+1, 3);
for q=0:2
  [u(:, q+1),  ifail] = nag_ode_bvp_ps_lin_cgl_vals(n, a, b, nag_int(q), uc(:, q+1));
end

% Print Solution
fprintf('\nNumerical solution U and its first two derivatives\n');
fprintf('      x          U          Ux         Uxx\n');
fprintf('%10.4f %10.4f %10.4f %10.4f\n', [x u]');

% diary off
