function nag_ode_ivp_2nd_rkn_example

% diary nag_ode_ivp_2nd_rkn_example.r
fprintf('nag_ode_ivp_2nd_rkn example results\n\n');

% Initialize variables and arrays.
ecc = 0.5;
tend = 20;
tstart = 0;
tinc = 0.1;

neq = 2;
lrwork = 16+20*neq;
nwant = neq;
thres = zeros(neq, 1);
thresp = zeros(neq, 1);

fprintf('nag_ode_ivp_2nd_rkn example program results \n\n');

for itol = 4:5
    tol = 10^(-itol);
    thres(1) = 0.0;
    thresp(1) = 0.0;
    h = 0.0;
    maxstp = 0;
    start = true;
    high = false;
    onestp = true;
    rwork = zeros(lrwork,1);
    t = 0;
    y = [1-ecc; 0];
    yp = [0; sqrt((1+ecc)/(1-ecc))];
    ydp = zeros(neq, 1);

    % nag_ode_ivp_2nd_rkn_setup is a setup routine to be called prior to nag_ode_ivp_2nd_rkn.
    [start, rwork, ifail] = nag_ode_ivp_2nd_rkn_setup(h, tol, thres, thresp, nag_int(maxstp),...
        start, onestp, high, rwork, 'neq', nag_int(neq));

    fprintf('Calculation with tol = %1.1e \n\n',tol);
    % fprintf('   t          y_1           y_2 ');
    t = tstart;
    tnext = t + tinc;
    % fprintf('\n %4.1f    %10.5f    %10.5f\n', t, y);

    if itol == 4
        ncall = 1;
        tkeep = tnext;
        ykeep = y;
        nkeep = 1;
    end

    while (t < tend)

        [tout, y, yp, ydp, rwork, ifail] = nag_ode_ivp_2nd_rkn(@fcn, t, tend, y,...
            yp, ydp, rwork);
        t = tout;

        while (tnext <= t)
            % nag_ode_ivp_2nd_rkn_interp interpolates components of solution provided by nag_ode_ivp_2nd_rkn.

             [ywant, ypwant, ifail] = nag_ode_ivp_2nd_rkn_interp(t, y, yp, nag_int(neq), tnext,...
                rwork, 'neq', nag_int(neq));

            ncall = ncall+1;
            % fprintf(' %4.1f    %10.5f    %10.5f\n', tnext, ywant);

            if itol == 4
                tkeep(ncall) = tnext;
                ykeep(:,ncall) = ywant;
            end

            tnext = tnext + tinc;
        end
    end

    % nag_ode_ivp_2nd_rkn_diag is a diagnostic routine for use after calling nag_ode_ivp_2nd_rkn.

    [hnext, hused, hstart, nsucc, nfail, natt, thres, thresp, ifail] =...
        nag_ode_ivp_2nd_rkn_diag(nag_int(neq), rwork);

    fprintf('\nNumber of successful steps = %1.0f\n',nsucc);
    fprintf('Number of failed     steps = %1.0f\n',nfail);
    fprintf('\n\n');
end

% Find the index of the point halfway through the first orbit (i.e. the
% first point at which the y coord is negative).  This is used to calculate
% the end points of the orbits in display_plot.
nhalf = 1;
while ykeep(2,nhalf) >= 0
    nhalf = nhalf+1;
end
% Plot results.
fig1 = figure;
display_plot(ykeep, nhalf);
% print(fig1,'-dpng','-r75','nag_ode_ivp_2nd_rkn_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_ivp_2nd_rkn_fig1.eps');

% diary off

function ydp = fcn(neq, t, y)
% Evaluate second derivatives.
r = sqrt(y(1)^2+y(2)^2)^3;
f = zeros(2,1);
ydp(1) = -y(1)/r;
ydp(2) = -y(2)/r;

function display_plot(ykeep, nhalf)

% Find the end points of the three orbits.
norb1 = 2*(nhalf-1); norb2 = 2*norb1; norb3 = 3*norb1;

% Plot the orbits, including displacements.
plot(ykeep(1,1:norb1), ykeep(2,1:norb1), '-+', ...
       ykeep(1,norb1+1:norb2), ykeep(2,norb1+1:norb2)+0.1, '--x', ...
       ykeep(1,norb2+1:norb3), ykeep(2,norb2+1:norb3)+0.2, ':*');
% Add title.
title({'Second-order ODE Solution using Runge-Kutta-Nystrom.', ...
    'The Two-body Problem (using shifts to distinguish orbits)'});
% Label the axes.
xlabel('x'); ylabel('y');
% Add a legend.
legend('1st orbit', '2nd orbit+(0,0.1)', '3rd orbit+(0,0.2)', ...
       'Location', 'Best');
