function nag_ode_ivp_adams_roots_revcom_example

% diary nag_ode_ivp_adams_roots_revcom_example.r
fprintf('nag_ode_ivp_adams_roots_revcom example results\n\n');

% Initialize setup variables and arrays.
statef = 'S';
neqf   = nag_int(3);
vectol = false;
atol   = [1e-4];
rtol   = [1e-7];
onestp = false;
crit   = true;
tcrit  = 10;
hmax   = 2;
maxstp = nag_int(500);
neqg   = nag_int(1);
alterg = false;
sophst = true;
rwork  = zeros(106, 1);
iwork  = zeros(25, 1, nag_int_name);

% nag_ode_ivp_adams_setup is a setup routine to be called prior to nag_ode_ivp_adams_roots_revcom.
[statef, alterg, rwork, iwork, ifail] = ...
         nag_ode_ivp_adams_setup(...
               statef, neqf, vectol, atol, rtol, onestp, crit, tcrit, ...
               hmax, maxstp, neqg, alterg, sophst, rwork, iwork);

%Initialize integrator input variables
t    = 0;
y    = [0.5; 0.5; 0.2*pi];
grv  = 0;
kgrv = nag_int(0);

% Initialize output variables and arrays.
tinc = 2;
ncall = 1;
tkeep = zeros(1,1);
ykeep = zeros(1,neqf);


% Output initial results, then store them for plotting.
fprintf('   t        y(1)      y(2)      y(3)\n');
fprintf('%7.4f   %7.4f   %7.4f   %7.4f\n', t, y(1), y(2), y(3));
tkeep(ncall)   = t;
ykeep(ncall,:) = y;

for j=1:5

    tout = double(j)*tinc;
    % Initial call of nag_ode_ivp_adams_roots_revcom for this t value.
    irevcm     = nag_int(0);
    first_call = true;

    % nag_ode_ivp_adams_roots_revcom uses reverse communication.  Keep re-entering the function
    % until a final exit (irevcm = 0) occurs.
    while (irevcm ~= 0 || first_call)
      first_call = false;
        [t, y, root, irevcm, trv, yrv, yprv, kgrv, rwork, iwork, ifail] = ...
            nag_ode_ivp_adams_roots_revcom(...
                  t, y, tout, neqg, irevcm, grv, kgrv, rwork, iwork);
        if (irevcm > 0 && irevcm < 8)
            % Set current values for derivatives.
            if (yrv == 0)
                rwork(yprv)   = tan(y(3));
                rwork(yprv+1) = -0.032*tan(y(3))/y(2) - 0.02*y(2)/cos(y(3));
                rwork(yprv+2) = -0.032/y(2)^2;
            else
                y2 = rwork(yrv+1);
                y3 = rwork(yrv+2);
                rwork(yprv)   = tan(y3);
                rwork(yprv+1) = -0.032*tan(y3)/y2 - 0.02*y2/cos(y3);
                rwork(yprv+2) = -0.032/y2^2;
            end
        elseif (irevcm > 8)
            grv = y(1);
        end
    end

    % Output results, then store them for plotting.
    fprintf('%7.4f   %7.4f   %7.4f   %7.4f\n', t, y(1), y(2), y(3));
    ncall          = ncall + 1;
    tkeep(ncall)   = t;
    ykeep(ncall,:) = y;
end
% Plot results.
fig1 = figure;
display_plot(tkeep, ykeep)
% print(fig1,'-dpng','-r75','nag_ode_ivp_adams_roots_revcom_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_ivp_adams_roots_revcom_fig1.eps');

% diary off

function display_plot(tkeep, ykeep)
% Plot the results.
plot(tkeep, ykeep(:,1), '-+', tkeep, ykeep(:,2), '--x', tkeep, ...
     ykeep(:,3), ':*');
%Add title.
title('ODE Solution using Adams Method with Root-finding');
% Label the axes.
xlabel('x');
ylabel('Solution');
% Add a legend.
legend('height','velocity','angle','Location','Best');
