function nag_ode_ivp_rkts_interp_example

% diary nag_ode_ivp_rkts_interp_example.r
fprintf('nag_ode_ivp_rkts_interp example results\n\n');

% Set initial conditions and input
method = nag_int(2);
tstart = 0;
tend   = 2*pi;
yinit  = [0;1];
hstart = 0;
thresh = [1e-08; 1e-08];
n      = nag_int(2);
nwant  = nag_int(1);
tol    =  1.0E-2;
npts   = 32;
wcomm  = zeros(n+5*nwant, 1);
twant  = zeros(npts+1, 1);
ywant  = zeros(npts, nwant);
ypwant = zeros(npts, nwant);

% Set control for printing solution
tinc   = (tend-tstart)/npts;

% Run through the calculation twice with two tolerance values
for i = 1:2

  tol = tol*0.1;

  % Call setup function
  [iwsav, rwsav, ifail] = nag_ode_ivp_rkts_setup(tstart, tend, yinit, tol, thresh, method);

  % Set up first point at which solution is required
  twant(1) = tstart;
  twant(2) = tstart + tinc;
  ywant(1,1) = yinit(1);
  ypwant(1,1) = yinit(2);
  t = tstart;
  j=1;

  % Integrate by steps until tend is reached or error is encountered.
  while t < tend

  % Integrate one step from t, updating t.
    [t, y, yp, user, iwsav, rwsav, ifail] = nag_ode_ivp_rkts_onestep(@f, n, iwsav, rwsav);

    % Interpolate at required additional points up to t
    while twant(j+1) < t
      j=j+1;
      % Interpolate and print solution at t = twant.
      ideriv = nag_int(2);
      [ywant(j, :), ypwant(j, :), wcomm, user, iwsav, rwsav, ifail] = ...
        nag_ode_ivp_rkts_interp(n, twant(j), ideriv, nwant, @f, wcomm, iwsav, rwsav);

      % Set next required solution point
      twant(j+1) = twant(j) + tinc;
    end
  end

  % Get integration statistics.
  [fevals, stepcost, waste, stepsok, hnext, iwsav, ifail] = nag_ode_ivp_rkts_diag(iwsav,rwsav);
  fprintf('\nCalculation with TOL = %8.1e\n\n', tol);
  fprintf('     Number of evaluations of f = %d\n', fevals);

end

% Plot results
fig1 = figure;
title('Simple Sine Solution, tol=1e-4');
hold on;
xlabel('t');
ylabel('Solution');
plot(twant(1:npts), ywant(:, 1), '-xr');
text(3, 0.25, 'Solution', 'Color', 'r');
plot(twant(1:npts), ypwant(:, 1), '-xg');
text(1.45, 0.25, 'Derivative', 'Color', 'g');
% print(fig1,'-dpng','-r75','nag_ode_ivp_rkts_interp_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_ivp_rkts_interp_fig1.eps');
hold off

% diary off

function [yp, user] = f(t, n, y, user)
  yp = [y(2); -y(1)];
