function nag_ode_ivp_stiff_fulljac_setup_example

% diary nag_ode_ivp_stiff_fulljac_setup_example.r
fprintf('nag_ode_ivp_stiff_fulljac_setup example results\n\n');

% Initialize variables and arrays for setup routine

n      = nag_int(3);
ord    = nag_int(5);
sdy    = nag_int(ord + 1);
petzld = false;
con    = zeros(6);
tcrit  = 10;
hmin   = 1.0e-10;
hmax   = 10;
h0     = 0;
maxstp = nag_int(200);
mxhnil = nag_int(5);
lrwork = 50 + 4*n;
rwork  = zeros(lrwork);

% Setup integration method using nag_ode_ivp_stiff_bdf.
[const, rwork, ifail] = nag_ode_ivp_stiff_bdf(n, sdy, ord, 'Newton', petzld, con, tcrit, ...
                              hmin, hmax, h0, maxstp, mxhnil, 'Average-L2', ...
                              rwork);
% Setup for analytical Jacbian using nag_ode_ivp_stiff_fulljac_setup
nwkjac = nag_int(n*(n+1));
[rwork, ifail] = nag_ode_ivp_stiff_fulljac_setup(n, n, 'Analytical', nwkjac, rwork);

% Initialize variables and arrays for integration
t      = 0;
tout   = 10;
y      = [1; 0; 0];
rtol   = [0.0001];
atol   = [1e-07];
itol   = nag_int(1);
inform = zeros(23, 1, nag_int_name);
ysave  = zeros(n, sdy);
wkjac  = zeros(nwkjac, 1);
itask  = nag_int(4);
itrace = nag_int(0);

% Integrate ODE from t=0 to t=tout, no monitoring, using nag_ode_ivp_stiff_exp_fulljac
[t, y, ydot, rwork, inform, ysave, wkjac, ifail] = nag_ode_ivp_stiff_exp_fulljac(t, tout, y, rwork, ...
          rtol, atol, itol, inform, @fcn, ysave, @jac, wkjac, 'nag_ode_ivp_stiff_exp_fulljac_dummy_monit', ...
          itask, itrace);

fprintf('Solution y and derivative y'' at t = %7.4f is:\n',t);
fprintf('\n %10s %10s\n','y','y''');
for i=1:n
  fprintf(' %10.4f %10.4f\n',y(i),ydot(i));
end

% diary off

function [f, ires] = fcn(neq, t, y, ires)
% Evaluate derivative vector.
f = zeros(3,1);
f(1) = -0.04d0*y(1) + 1.0d4*y(2)*y(3);
f(2) =  0.04d0*y(1) - 1.0d4*y(2)*y(3) - 3.0d7*y(2)*y(2);
f(3) =                                  3.0d7*y(2)*y(2);

function p = jac(neq, t, y, h, d, p)
% Evaluate the Jacobian.
p = zeros(neq, neq);
hxd = h*d;
p(1,1) = -hxd*(-0.04d0    ) + 1;
p(1,2) = -hxd*( 1.0d4*y(3));
p(1,3) = -hxd*( 1.0d4*y(2));
p(2,1) = -hxd*( 0.04d0    );
p(2,2) = -hxd*(-1.0d4*y(3)-6.0d7*y(2)) + 1;
p(2,3) = -hxd*(-1.0d4*y(2));
p(3,2) = -hxd*( 6.0d7*y(2));
p(3,3) = -hxd*( 0.0d0     ) + 1;
