function nag_ode_ivp_stiff_imp_bandjac_example

% diary nag_ode_ivp_stiff_imp_bandjac_example.r
fprintf('nag_ode_ivp_stiff_imp_bandjac example results\n\n');

neq    = nag_int(3);
maxord = nag_int(5);
sdysav = maxord+1;
petzld = false;
tcrit  = 0;
hmin   = 1.0e-10;
hmax   = 10;
h0     = 0;
maxstp = nag_int(200);
mxhnil = nag_int(5);
const = zeros(6, 1);
rwork = zeros(50+4*neq, 1);

% Setup for nag_ode_ivp_stiff_imp_bandjac.
[const, rwork, ifail] = nag_ode_ivp_stiff_bdf( ...
                               neq, sdysav, maxord, 'Newton', petzld, ...
                               const, tcrit, hmin, hmax, h0, maxstp, ...
                               mxhnil, 'Average-L2', rwork);
ml     = nag_int(1);
mu     = nag_int(2);
nwkjac = nag_int(neq*(2*ml+mu+1));

% Numerical Jacobian
[rwork, ifail] = nag_ode_ivp_stiff_bandjac_setup( ...
			neq, neq, 'Numerical', ml, mu, nwkjac, neq, rwork);

% Initialize variables and arrays for integration
t    = 0;
tout = 10;
y    = [1; 0; 0];
ydot = zeros(neq, 1);
rtol = [0.0001];
atol = [1e-06; 1e-07; 1e-06];
itol = nag_int(2);
inform(1:23) = nag_int(0);
ysave  = zeros(neq, sdysav);
wkjac  = zeros(nwkjac, 1);
jacpvt(1:neq) = nag_int(0);
lderiv = [false; true];
itask  = nag_int(6);
itrace = nag_int(0);

% Solve for the initial values y and dy/dx as check
[t, tout, y, ydot, rwork, inform, ysave, wkjac, jacpvt, lderiv, ifail] = ...
nag_ode_ivp_stiff_imp_bandjac( ...
       t, tout, y, ydot, rwork, rtol, atol, itol, inform, @resid, ysave, ...
       'nag_ode_ivp_stiff_imp_bandjac_dummy_jac', wkjac, jacpvt, 'nag_ode_ivp_stiff_exp_fulljac_dummy_monit', lderiv, itask, itrace);

fprintf ('Initial y   : %8.4f %8.4f %8.4f\n',y);
fprintf ('Initial ydot: %8.4f %8.4f %8.4f\n',ydot);

% Use these initial estimates and integrate to tout 
% (overshoot and interpolate)
itask = nag_int(1);
tout = 10;
[t, tout, y, ydot, rwork, inform, ysave, wkjac, jacpvt, lderiv, ifail] = ...
nag_ode_ivp_stiff_imp_bandjac( ...
       t, tout, y, ydot, rwork, rtol, atol, itol, inform, @resid, ysave, ...
       'nag_ode_ivp_stiff_imp_bandjac_dummy_jac', wkjac, jacpvt, 'nag_ode_ivp_stiff_exp_fulljac_dummy_monit', lderiv, itask, itrace);

fprintf ('\n    x          y\n');
fprintf ('%8.3f %13.5f %13.5f %13.5f\n',tout,y);

[hu, h, tcur, tolsf, nst, nre, nje, nqu, nq, niter, imxer, algequ, ifail] = ...
  nag_ode_ivp_stiff_integ_diag( ...
	 neq, neq, rwork, inform);

fprintf('\nDiagnostic information\n integration status:\n');
fprintf('   last and next step sizes = %8.5f, %8.5f\n',hu, h);
fprintf('   integration stopped at x = %8.5f\n',tcur);
fprintf(' algorithm statistics:\n');
fprintf('   number of time-steps and Newton iterations  = %5d %5d\n', ...
        nst,niter);
fprintf('   number of residual and jacobian evaluations = %5d %5d\n', ...
        nre,nje);
fprintf('   order of method last used and next to use   = %5d %5d\n', ...
        nqu,nq);
fprintf('   component with largest error                = %5d\n',imxer);

% diary off

function [f, ires] = fcn(neq, t, y, ires)
  % Evaluate derivative vector.
  f = zeros(3,1);
  f(1) = -0.04*y(1) + 10000*y(2)*y(3);
  f(2) =  0.04*y(1) - 10000*y(2)*y(3) - 30000000*y(2)*y(2);
  f(3) =  30000000*y(2)*y(2);

function p = jac(neq, t, y, h, d, p)
  % Evaluate the Jacobian.
  p = zeros(neq, neq);
  hxd = h*d;
  p(1,1) = 1 - hxd*(-0.04);
  p(1,2) = -hxd*(10000*y(3));
  p(1,3) = -hxd*(10000*y(2));
  p(2,1) = -hxd*(0.04);
  p(2,2) = 1 - hxd*(-10000*y(3)-60000000*y(2));
  p(2,3) = -hxd*(-10000*y(2));
  p(3,2) = -hxd*(60000000*y(2));
  p(3,3) = 1 - hxd*(0);

function [r, ires] = resid(neq, t, y, ydot, ires)

  r = zeros(neq,1);
  r(1) = -ydot(1) - ydot(2) - ydot(3);
  r(2) = -ydot(2);
  r(3) = -ydot(3);
  if (ires == 1) ;
     r(1) = 0.0d0 + r(1);
     r(2) = 0.04d0*y(1) - 1.0d4*y(2)*y(3) - 3.0d7*y(2)*y(2) + r(2);
     r(3) = 3.0d7*y(2)*y(2) + r(3);
  end ;
