function nag_ode_ivp_stiff_integ_diag_example

% diary nag_ode_ivp_stiff_integ_diag_example.r
fprintf('nag_ode_ivp_stiff_integ_diag example results\n\n');

% Initialize integration method setup variables and arrays.
neq    = nag_int(3);
neqmax = nag_int(neq);
nwkjac = nag_int(neqmax*(neqmax + 1));
maxord = nag_int(5);
sdysav = nag_int(maxord+1);
maxstp = nag_int(200);
mxhnil = nag_int(5);

h0    = 0;
hmax  = 10;
hmin  = 1.0e-10;
tcrit = 20;
petzld = false;

const  = zeros(6, 1);
rwork  = zeros(50+4*neqmax, 1);

[const, rwork, ifail] = nag_ode_ivp_stiff_bdf(neqmax, sdysav, maxord, 'Newton', petzld, ...
                              const, tcrit, hmin, hmax, h0, maxstp, ...
                              mxhnil, 'Average-L2', rwork);

% Setup for numerical Jacbian using nag_ode_ivp_stiff_fulljac_setup
nwkjac = nag_int(neq*(neq+1));
[rwork, ifail] = nag_ode_ivp_stiff_fulljac_setup( ...
			neq, neq, 'Numerical', nwkjac, rwork);

% Initialize variables and arrays for integration
t      = 0;
tout   = 10;
y      = [1; 0; 0];
rtol   = [0.0001];
atol   = [1e-07];
itol   = nag_int(1);
inform = zeros(23, 1, nag_int_name);
ysave  = zeros(neq, sdysav);
wkjac  = zeros(nwkjac, 1);
itask  = nag_int(4);
itrace = nag_int(0);

% Integrate ODE from t=0 to t=tout, no monitoring, using nag_ode_ivp_stiff_exp_fulljac
[t, y, ydot, rwork, inform, ysave, wkjac, ifail] = ...
  nag_ode_ivp_stiff_exp_fulljac( ...
	 t, tout, y, rwork, rtol, atol, itol, inform, @fcn, ysave, ...
	 @jac, wkjac, 'nag_ode_ivp_stiff_exp_fulljac_dummy_monit', itask, itrace);

% Get diagnostics
[hu, h, tcur, tolsf, nst, nre, nje, nqu, nq, nit, imxer, algequ, ifail] = ...
  nag_ode_ivp_stiff_integ_diag( ...
         neq, neqmax, rwork, inform);

% Get solution at tout
[y, ifail] = nag_ode_ivp_stiff_interp( ...
                    tout, neq, neq, ysave, rwork);

% Print solution and diagnostics
fprintf(' At t = %8.3f, y(1:3) = %5.1f, %5.1f, %5.1f\n', t, y);
fprintf('\nDiagnostic information\n integration status:\n');
fprintf('   last and next step sizes = %8.5f, %8.5f\n',hu, h);
fprintf('   integration stopped at x = %8.5f\n',tcur);
fprintf(' algorithm statistics:\n');
fprintf('   number of time-steps and Newton iterations  = %5d %5d\n',nst,nit);
fprintf('   number of residual and jacobian evaluations = %5d %5d\n',nre,nje);
fprintf('   order of method last used and next to use   = %5d %5d\n',nqu,nq);
fprintf('   component with largest error                = %5d\n',imxer);

% diary off

function [f, ires] = fcn(neq, t, y, ires)
  % Evaluate derivative vector.
  f = zeros(3,1);
  f(1) = -0.04d0*y(1) + 1.0d4*y(2)*y(3);
  f(2) = 0.04d0*y(1) - 1.0d4*y(2)*y(3) - 3.0d7*y(2)*y(2);
  f(3) = 3.0d7*y(2)*y(2);

function p = jac(neq, t, y, h, d, p)
  % Evaluate the Jacobian.
  p = zeros(neq, neq);
  hxd = h*d;
  p(1,1) = 1.0d0 - hxd*(-0.04d0);
  p(1,2) = -hxd*(1.0d4*y(3));
  p(1,3) = -hxd*(1.0d4*y(2));
  p(2,1) = -hxd*(0.04d0);
  p(2,2) = 1.0d0 - hxd*(-1.0d4*y(3)-6.0d7*y(2));
  p(2,3) = -hxd*(-1.0d4*y(2));
  p(3,2) = -hxd*(6.0d7*y(2));
  p(3,3) = 1.0d0 - hxd*(0.0d0);
