function nag_pde_2d_ellip_fd_example

% diary nag_pde_2d_ellip_fd_example.r
fprintf('nag_pde_2d_ellip_fd example results\n\n');


% This example demonstrates the solution of the Laplace Equation with
% Dirichlet boundary conditions using a finite-difference 5-point stencil.

% Initialize variables and arrays.
aparam = 1;
itmax = nag_int(100);
itcoun = nag_int(0);
ndir = nag_int(1);
ixn = nag_int(0);
iyn = nag_int(0);
conres = 1e-06;
conchn = 1e-06;
n1 = nag_int(31);
n2 = 46;
x = 0:0.5:15;
y = 0:1:45;

% Set up difference equation coefficients, source terms and
% initial conditions.
a = zeros(n1, n2);
e = a; b = a; d = a; q = a; t = a;

for j = 2:n2-1
  a(2:n1-1,j) = 2/((y(j)-y(j-1))*(y(j+1)-y(j-1)));
  e(2:n1-1,j) = 2/((y(j+1)-y(j))*(y(j+1)-y(j-1)));
end
for i = 2:n1-1
  b(i,2:n2-1) = 2/((x(i)-x(i-1))*(x(i+1)-x(i-1)));
  d(i,2:n2-1) = 2/((x(i+1)-x(i))*(x(i+1)-x(i-1)));
end
c = -a - b - d - e;

q(1,1:n2)  = exp(6*(x(1)   +1.0)/y(n2))*cos(7*y(1:n2)/y(n2));
q(n1,1:n2) = exp(6*(x(n1)  +1.0)/y(n2))*cos(7*y(1:n2)/y(n2));
q(1:n1,1)  = exp(6*(x(1:n1)+1.0)/y(n2))*cos(7*y(1)   /y(n2));
q(1:n1,n2) = exp(6*(x(1:n1)+1.0)/y(n2))*cos(7*y(n2)  /y(n2));

[u, itcoun, itused, resids, chngs, ifail] = ...
nag_pde_2d_ellip_fd( ...
       n1, a, b, c, d, e, q, t, aparam, itmax, ...
       itcoun, ndir, ixn, iyn, conres, conchn);

% Output solution statistics.
fprintf('Number of iterations  %4d\n',itused);
fprintf('Maximum residual       %8.3e\n',resids(itused));

% Plot results.
fig1 = figure;
display_plot(x, y, u);
% print(fig1,'-dpng','-r75','nag_pde_2d_ellip_fd_fig1.png');
% print(fig1,'-deps','-r75','nag_pde_2d_ellip_fd_fig1.eps');

% diary off

function display_plot(x, y, tOut)
  % Plot array as a mesh and contours.
  meshc(y, x, tOut);
  % Set the title.
  title({'Solution to Laplace''s Equation', ...
         'using the Strongly Implicit Procedure', ...
         'on a Five-point Molecule Discretisation'});
  % Label the axes.
  xlabel('y');
  ylabel('x');
  zlabel('U(x,y)');
  % Set the view to something nice (determined empirically).
  view(35, 20);
