function nag_pde_2d_gen_order2_rectangle_example

% diary nag_pde_2d_gen_order2_rectangle_example.r
fprintf('nag_pde_2d_gen_order2_rectangle example results\n\n');

global ssav1 ssav2;
global xsav ysav tsav;

ex1;
fig1 = figure;
ex1_plot;


ex2;
fig2 = figure;
ex2_plot(1);
fig3 = figure;
ex2_plot(2);

% print(fig1,'-dpng','-r75','nag_pde_2d_gen_order2_rectangle_fig1.png');
% print(fig1,'-deps','-r75','nag_pde_2d_gen_order2_rectangle_fig1.eps');
% print(fig2,'-dpng','-r75','nag_pde_2d_gen_order2_rectangle_fig2.png');
% print(fig2,'-deps','-r75','nag_pde_2d_gen_order2_rectangle_fig2.eps');
% print(fig3,'-dpng','-r75','nag_pde_2d_gen_order2_rectangle_fig3.png');
% print(fig3,'-deps','-r75','nag_pde_2d_gen_order2_rectangle_fig3.eps');

% diary off

function ex1
% First nag_pde_2d_gen_order2_rectangle example.  Temperature distribution for a single, one-step
% reaction of a mixture of two chemicals.
global heat_release activ_energy diffusion damkohler;
global iout isav;

% Set values for problem parameters.
mxlev = 3;
npde = 1;
npts = 2000;
xmin = 0;    xmax = 1;
ymin = 0;    ymax = 1;
tols = 0.5;
tolt = 0.01;
nx   = nag_int(21);
ny   = nx;

reaction_rate =  5;
activ_energy  = 20;
heat_release  =  1;
diffusion     =  0.1;
damkohler = reaction_rate*exp(activ_energy)/(heat_release*activ_energy);

leniwk = npts*(14 + 5*mxlev) + 7*mxlev + 2;
lenlwk = npts + 1;
lenrwk = npts*npde*(5*mxlev + 18*npde + 9) + 2*npts;

% The above are the recommended values from the documentation, but nag_pde_2d_gen_order2_rectangle
% returns with ifail = 3 and the minumum workspace size requirements if
% these are used.  So we beef them up here with some fudge factors.
leniwk = 2*leniwk;
lenlwk = nag_int(2*lenlwk);
lenrwk = 2*lenrwk;

% Create some work arrays - see NAG documentation.
rwk = zeros(lenrwk, 1);
iwk = zeros(leniwk, 1, nag_int_name);

% Initialize some problem variables.
ind    = nag_int(0);
itrace = nag_int(0);
ts     = 0;
dt     = [0.1e-2; 0.0; 0.0];
twant  = [0.24; 0.25];
opti = zeros(4, 1, nag_int_name);
optr = ones(3, npde);

% index of next set of saved results
isav = 1;

% We run through the problem twice - once from t=0 up to t=twant(1), and
% then onwards to t=twant(2).
warning('Off');
fprintf('Example 1\n==========\n\n');
for iout = 1:2
  tout = twant(iout);
  [ts, dt, rwk, iwk, ind, ifail] = ...
  nag_pde_2d_gen_order2_rectangle( ...
         ts, tout, dt, xmin, xmax, ymin, ymax, nx, ny, tols, tolt, ...
         @ex1_pdedef, @ex1_bndary, @ex1_pdeiv, @ex1_monitr, opti, ...
         optr, rwk, iwk, itrace, ind, 'lenlwk', lenlwk);

  fprintf('\nTime =%8.4f \n', ts);
  fprintf('Total number of accepted timesteps %5d \n', iwk(1));
  fprintf('Total number of rejected timesteps %5d \n', iwk(2));
  fprintf('\n              Total (max) number of\n');
  fprintf('          Residual  Jacobian    Newton   Lin sys\n');
  fprintf('             evals     evals     iters     iters\n');
  fprintf('At level\n');
  maxlev = 3;
  for j = 1:maxlev
    if iwk(j+2) ~= 0
      evals = iwk(j+2:maxlev:j+2+5*maxlev);
      fprintf('%8d%10d%10d%6d(%2d)%6d(%2d)\n', j, evals);
    end;
  end;
end


function [res] = ex1_pdedef(npts, npde, t, x, y, u, ut, ux, uy, uxx, uxy, uyy)
  % Evaluate the system of PDEs for this problem.

  global heat_release activ_energy diffusion damkohler;

  res = zeros(npts, npde);
  for i = 1:npts
    e = exp(-activ_energy/u(i,1));
    res(i,1) = ut(i,1) - diffusion*(uxx(i,1) + uyy(i,1)) - ...
               damkohler*(1.0 + heat_release - u(i,1))*e;
  end


function [u] = ex1_pdeiv(npts, npde, t, x, y)
  % Evaluate initial conditions for PDEs for this system.

  u = ones(npts, npde);


function [res] = ex1_bndary(npts, npde, t, x, y, u, ut, ux, uy, ...
                            nbpts, lbnd, res)
  % Implement boundary conditions for the domain.

  tol = 10*nag_machine_precision();
  for i = 1:nbpts
    j = lbnd(i);
    if (abs(x(j)) <= tol)
      res(j,1) = ux(j,1);
    elseif (abs(x(j)-1) <= tol)
      res(j,1) = u(j,1) - 1;
    elseif (abs(y(j)) <= tol)
      res(j,1) = uy(j,1);
    elseif (abs(y(j)-1) <= tol)
      res(j,1) = u(j,1) - 1;
    end
end


function [ierr] = ex1_monitr(npde, t, dt, dtnew, tlast, nlev, ...
                             ngpts, xpts, ypts, lsol, sol, ierr)
  % Save the results at specified times.
  times = [0.001; 0.228; 0.240; 0.25];

  global iout isav;
  global xsav ysav ssav1 tsav;

  % Save this time step only if the current time is equal to
  % or just past the next output time
  if (t < times(isav))
    return;
  end
  fprintf('Saving set %d at time = %f\n', isav, t);

  % Specify the grid level for extracting the solution.
  level = 1;
  npts = ngpts(level);
  ipsol = lsol(level);

  % Allocate space for saves the first time through.
  nside = round(sqrt(double(npts)));
  if isav == 1
    nsav = length(times);
    xsav = zeros(nside, 1);
    ysav = zeros(nside, 1);
    ssav1 = zeros(nside, nside, nsav);
    tsav = zeros(nsav, 1);
  end

  % Save this time step.
  tsav(isav) = t;
  for i = 1:nside
    for j = 1:nside
      k = (i-1)*nside + j;
      xsav(j) = xpts(k);
      ysav(i) = ypts(k);
      ssav1(j, i, isav) = sol(ipsol+k);
    end
  end

  % Look for the next time step, unless called for very last time.
  if ~(tlast && iout == 2)
    isav = isav+1;
  end


function ex1_plot
  % Plot the results.
  global xsav ysav ssav1 tsav isav;
  % Set the colours for each surface, and the array of handles for the plots.
  colours = [[1 0 0]; [0 1 0]; [1 0 1]; [0 0 1]; [0 1 1]; [2 1 0]; [2 0 1];
             [1 0 2]; [1 2 0]; [0 1 2]; [0 2 1]; [3 2 1]; [2 3 1]; [2 1 3];
             [3 1 2]; [1 3 2]; [1 2 3]; [0.3 2 1]; [2 0.3 1]; [2 1 0.3]; 
             [0.3 1 2]; [1 0.3 2]; [1 2 0.3]; [0.3 0.2 1]; [0.2 0.3 1];
             [0.2 1 0.3]; [0.3 1 0.2]; [1 0.3 0.2]; [1 0.2 0.3]];
             
  h = zeros(isav,1);
  % Check the allocation of colours against the number to be plotted.
  if (length(colours(:,1)) < isav)
    fprintf(['Not enough colours allocated in example1_plot: ', ...
             '%d needed\n'], isav);
  end
  % Plot all the surfaces, and colour each one.
  for i = 1:isav
    h(i) = mesh(xsav, ysav, ssav1(:,:,i));
    hold on;
    set(h(i), 'EdgeColor', colours(i,:));
  end
  hold off;
  % Label the axes, and set the title.
  xlabel('x');
  ylabel('y');
  zlabel('U(x,y,t)');
  title({'Model for a Single, One-step Reaction', ...
         'of a Mixture of Two Chemicals'});
  % Add a legend, using the plot handles and the array of time values.
  hleg = legend(h, num2str(tsav(1:isav)));
  % Set the view to something nice (determined empirically).
  view(15, 30);
  
  
function ex2
  % Second nag_pde_2d_gen_order2_rectangle example.  Concentration distribution in a multispecies
  % (predator-prey) food web model.
  global alpha beta;
  global iout icount;

  % Set values for problem parameters.
  npde = nag_int(2);
  npts = nag_int(4000);
  xmin = 0;  xmax = 1;
  ymin = 0;  ymax = 1;
  tols = 0.075;
  tolt = 0.1;
  nx = nag_int(21);
  ny = nag_int(21);
  alpha = 50.0;
  beta = 300.0;

  maxlev = 4;
  leniwk = npts*(14 + 5*maxlev) + 7*maxlev + 2;
  lenlwk = npts + 1;
  lenrwk = npts*npde*(5*maxlev + 18*npde + 9) + 2*npts;

  % The above are the recommended values from the documentation, but nag_pde_2d_gen_order2_rectangle
  % returns with ifail = 3 and the minumum workspace size requirements if
  % these are used.  So we beef them up here with some fudge factors.
  leniwk = 2*leniwk;
  lenlwk = nag_int(3*lenlwk);
  lenrwk = 3*lenrwk;

  % Create some work arrays - see NAG documentation.
  rwk = zeros(lenrwk, 1);
  iwk = zeros(leniwk, 1, nag_int_name);

% Initialize some problem variables.
  ind    = nag_int(0);
  itrace = nag_int(0);
  ts     = 0;
  dt     = [0.5e-3; 1.0e-6; 0.0];
  twant  = [0.01; 0.025];
  opti    = zeros(4, 1, nag_int_name);
  opti(1) = 4;
  optr      = ones(3, npde);
  optr(1,1) = 250;
  optr(1,2) = 1.5e+6;

  % Counts how many times monitor routine is called.
  icount = 0;
  fprintf('\n\nExample 2\n=========\n\n');

  % We run through the problem twice - once from t=0 up to t=twant(1), and
  % then onwards to t=twant(2).
  for iout = 1:2
    tout = twant(iout);
    [ts, dt, rwk, iwk, ind, ifail] = ...
    nag_pde_2d_gen_order2_rectangle( ...
           ts, tout, dt, xmin, xmax, ymin, ymax, nx, ny, tols, tolt, ...
           @ex2_pdedef, @ex2_bndary, @ex2_pdeiv, @ex2_monitr, opti, optr, ...
           rwk, iwk, itrace, ind, 'lenlwk', lenlwk);

    % Output some statistics.
    fprintf('\nTime =%8.4f \n', ts);
    fprintf('Total number of accepted timesteps %5d \n', iwk(1));
    fprintf('Total number of rejected timesteps %5d \n', iwk(2));
    fprintf('\n              Total (max) number of\n');
    fprintf('          Residual  Jacobian    Newton   Lin sys\n');
    fprintf('             evals     evals     iters     iters\n');
    fprintf('At level\n');
    for j = 1:maxlev
      if iwk(j+2) ~= 0
        evals = iwk(j+2:maxlev:j+2+5*maxlev);
        fprintf('%8d%10d%10d%6d(%2d)%6d(%2d)\n', j, evals);
      end;
    end;
end


function [res] = ex2_pdedef(npts, npde, t, x, y, u, ut, ux, uy, ...
                            uxx, uxy, uyy)
  % Evaluate the system of PDEs for this problem.

  global alpha beta;

  fp = 4*pi;
  res = zeros(npts, npde);
  for i = 1:npts
    b1 = 1 + alpha*x(i)*y(i) + beta*sin(fp*x(i))*sin(fp*y(i));
    res(i,1) = ut(i,1) - uxx(i,1) - uyy(i,1) - ...
               u(i,1)*(b1 - u(i,1) - 0.5e-6*u(i,2));
    res(i,2) = -0.05*(uxx(i,2) + uyy(i,2)) - ...
               u(i,2)*(-b1 + 1.0e4*u(i,1) - u(i,2));
  end


function [u] = ex2_pdeiv(npts, npde, t, x, y)
  % Evaluate initial conditions for PDEs for this system.

  global alpha beta;

  fp = 4*pi;
  u = zeros(npts, npde);
  for i = 1:npts
    u(i,1) = 10 + (16*x(i)*(1 - x(i))*y(i)*(1 - y(i)))^2;
    u(i,2) = -1 - alpha*x(i)*y(i) - beta*sin(fp*x(i))*sin(fp*y(i)) + ...
             1.0e4*u(i,1);
  end


function [res] = ex2_bndary(npts, npde, t, x, y, u, ut, ux, uy, ...
                            nbpts, lbnd, res)
  % Implement boundary conditions for the domain.

  tol = 10*nag_machine_precision;
  for i = 1:nbpts
    j = lbnd(i);
    if (abs(x(j)) <= tol || abs(x(j)-1) <= tol)
      res(j,1) = ux(j,1);
      res(j,2) = ux(j,2);
    elseif (abs(y(j)) <= tol || abs(y(j)-1) <= tol)
      res(j,1) = uy(j,1);
      res(j,2) = uy(j,2);
    end
  end


function [ierr] = ex2_monitr(npde, t, dt, dtnew, tlast, nlev, ...
                             ngpts, xpts, ypts, lsol, sol, ierr)
  % Monitor the results at specified intervals.
           
  global iout icount;
  global xsav ysav ssav2 tsav isav;

  % Specify the maximum number of times the results get saved, and the
  % interval for saves.
%  nsav = 5;
%  nint = 10;
 size(xpts);
 size(ypts);

  % Do we want to save this time?
  icount = icount+1;
 % if mod(icount,nint) ~= 1 && ~tlast
 %   return;
 % end

 % isav = 1 + round(icount/nint);
 % if isav > nsav
 %   fprintf('Not enough save space allocated in monitr2: %d\n', ...
 %           nsav);
 %   return;
 % end
  isav = icount;

  % Specify the grid level for extracting solution.
  level = 1;
  npts = ngpts(level);
  ipsol = lsol(level);

  % Allocate space for saves the first time through.
  nside = round(sqrt(double(npts)));

  % Save this time step.  For this problem, calculating two
  % concentrations as a function of x and y (and time).
  tsav(isav) = t;
  if (icount==1)
    xsav(1:nside) = xpts(1:nside);
    ysav(1:nside) = ypts(1:nside:nside^2);
  end

  s1 = reshape(sol(ipsol+1:ipsol+nside^2),[nside,nside]);
  s2 = reshape(sol(ipsol+npts+1:ipsol+npts+nside^2),[nside,nside]);
  ssav2(:,:,isav,1) = s1';
  ssav2(:,:,isav,2) = s2';

function ex2_plot(id)
  % Plot the results.
  global xsav ysav ssav2 tsav isav;
  % Check the value of the array identifier.
  if (id ~= 1 && id ~= 2)
    fprintf('Illegal value for array identifier in plot: %d\n', id);
    return;
  end
  % Label the axes, and set the title according to the array identifier.
  v = ssav2(:,:,:,id);
  slice(xsav, ysav, tsav, v ,[0,0.4,0.6,0.8],[1],[0.01,0.02]);
  colormap hot;
  colorbar;
  xlabel('x');
  ylabel('y');
  zlabel('time');
  if (id == 1)
    title({'Multispecies Food Web Model:', ...
           'Time-dependent Predator Concentration'});
  else
    title({'Multispecies Food Web Model:', ...
           'Time-dependent Prey Concentration'});
  end
  % Add a legend, using the plot handles and the array of time values.
  % Set the view to something nice (determined empirically).
  view(24,42);
