function nag_pde_3d_ellip_fd_example

% diary nag_pde_3d_ellip_fd_example.r
fprintf('nag_pde_3d_ellip_fd example results\n\n');

% Solve Laplace equation in a rectangular box using nag_pde_3d_ellip_fd. 
% Diricchlet boundary conditions are given as eact soltion on boundary.

% Mesh size
n1 = nag_int(16);
n2 = nag_int(20);
n3 = nag_int(24);

% Allocate arrays and set up initial values.
a = zeros(n1, n2, n3);
b = a; c = a; c = a; e = a; f = a; g = a; q = a; u0 = a;
aparam = 1;
itmax = nag_int(100); itcoun = nag_int(0); ndir = nag_int(1);
ixn = nag_int(0); iyn = ixn; izn = ixn;
conres = 1e-06; conchn = 1e-06;

% Set up mesh coordinates.
delta = 12.0/double(n1*(n1-1));
for i = 1:n1    
  x(i) = delta*double(i*(i-1))/2;        
end
delta = 20.0/double(n2*(n2-1));
for i = 1:n2
  y(i) = delta*double(i*(i-1))/2;
end
delta = 30.0/double(n3*(n3-1));
for i = 1:n3
  z(i) = delta*double(i*(i-1))/2;
end

% Set up difference equation coefficients, source terms and initial
% approximation.
for k = 2:n3-1
  a(2:n1-1,2:n2-1,k) = 2/((z(k)-z(k-1))*(z(k+1)-z(k-1)));
  g(2:n1-1,2:n2-1,k) = 2/((z(k+1)-z(k))*(z(k+1)-z(k-1)));
end
for j = 2:n2-1
  b(2:n1-1,j,2:n3-1) = 2/((y(j)-y(j-1))*(y(j+1)-y(j-1)));
  f(2:n1-1,j,2:n3-1) = 2/((y(j+1)-y(j))*(y(j+1)-y(j-1)));
end
for i = 2:n1-1
  c(i,2:n2-1,2:n3-1) = 2/((x(i)-x(i-1))*(x(i+1)-x(i-1)));
  e(i,2:n2-1,2:n3-1) = 2/((x(i+1)-x(i))*(x(i+1)-x(i-1)));
end
d = - a - b - c - e - f - g;    

scale = 1/y(n2);
exact = @(x,y,z)(exp((x+1)*scale)* cos(sqrt(2)*y*scale)* exp((-z-1)*scale));
q(1,   1:n2, 1:n3) = exact(x(1), y',   z);
q(n1,  1:n2, 1:n3) = exact(x(n1),y',   z);
q(1:n1,1   , 1:n3) = exact(x',   y(1), z);
q(1:n1,n2  , 1:n3) = exact(x',   y(n2),z);
q(1:n1,1:n2, 1)    = exact(x',   y,    z(1));
q(1:n1,1:n2, n3)   = exact(x',   y,    z(n3));

[u, itcoun, itused, resids, chngs, ifail] = ...
nag_pde_3d_ellip_fd( ...
       n1, n2, n3, a, b, c, d, e, f, g, q, u0, aparam, itmax, itcoun, ...
       ndir, ixn, iyn, izn, conres, conchn);

% Output solution statistics.
fprintf('Number of iterations  %4d\n',itused);
fprintf('Maximum residual       %8.3e\n',resids(itused));

% Plot results using slices on the (x,y)-plane.
fig1 = figure;
[xs,ys,zs] = meshgrid(y,x,z);
slice(xs,ys,zs,u,[],[],double([1 3 5 7]));
axis([y(1) y(end) x(1) x(end) 0 7]);
xlabel('x'); ylabel('y'); zlabel('z');
title('Laplace''s Equation in a rectangular box');
view(16, 14);
h = colorbar;
ylabel(h, 'U(x,y,z)')
% print(fig1,'-dpng','-r75','nag_pde_3d_ellip_fd_fig1.png');
% print(fig1,'-deps','-r75','nag_pde_3d_ellip_fd_fig1.eps');

% diary off
