function nag_quad_md_sgq_multi_vec_example

% diary nag_quad_md_sgq_multi_vec_example.r
fprintf('nag_quad_md_sgq_multi_vec example results\n\n');

% Evaluate an estimate to the set of integrals of sin(n+|x|)log(|x|),
% with |x| = sum(j*x(j)), for n=1:10 on a hypercube of dimension 4.

ni = nag_int(10);
d  = nag_int(4);

% Initialize nag_quad_md_sgq_multi_vec and set options
iopts = zeros(100,1,nag_int_name);
opts  = zeros(100,1);
[iopts, opts, ifail] = nag_quad_opt_set('Initialize = nag_quad_md_sgq_multi_vec', iopts, opts);
[iopts, opts, ifail] = nag_quad_opt_set('Absolute Tolerance = 0.0', iopts, opts);
[iopts, opts, ifail] = nag_quad_opt_set('Relative Tolerance = 1.0e-3', iopts, opts);
[iopts, opts, ifail] = nag_quad_opt_set('Maximum Level = 6', iopts, opts);
[iopts, opts, ifail] = nag_quad_opt_set('Index Level = 5', iopts, opts);

maxdlv = zeros(d,1,nag_int_name);
[dinest, errest, ivalid, user, ifail] = nag_quad_md_sgq_multi_vec(ni, @f, maxdlv, iopts, opts);

fprintf('%10s%13s%13s%14s\n','integrand', 'integral','error','state');
for i = 1:ni
  fprintf('%7d%16.6f%15.2e%10d\n',i,dinest(i),errest(i),ivalid(i));
end

% diary off

function [fm, iflag, user] = ...
         f(ni, ndim, nx, xtr, nntr, icolzp, irowix, xs, qs, iflag, user)

  fm = zeros(ni,nx);

  % Decompose |x| as |x_{tr}| + |(x_{non-tr} - x_{tr})|:
  % Here is |x_{tr}|:
  s_tr = xtr*double(ndim*(ndim+1))/2;

  for i = 1:nx
    i1 = icolzp(i);
    i2 = icolzp(i+1)-1;
    
    % Here is |(x_{non-tr} - x_{tr})|:
    s_ntr = sum(double(irowix(i1:i2)).*(xs(i1:i2)-xtr));
    
    for j = 1:ni
      fm(j,i) = sin(double(j)+s_ntr+s_tr)*log(s_ntr+s_tr);
    end
  end
