function nag_rand_field_2d_generate_example

% diary nag_rand_field_2d_generate_example.r
fprintf('nag_rand_field_2d_generate example results\n\n');

% Use symmetric stable variogram
icov2 = nag_int(1); 
params = [0.1; 0.15; 1.2];

% Random Field variance
var = 0.5;
% Domain endpoints
xmin = -1;
xmax =  1;
ymin = -0.5;
ymax =  0.5;
% Number of sample points in x and y
ns = [nag_int(5), 5];
% maximum dimensions for circulant matrix
maxm = [nag_int(64), 64];
% Scaling factor, rho = 1.
icorr = nag_int(2);

% Get square roots of the eigenvalues of the embedding matrix
[lam, xx, yy, m, approx, rho, icount, eig, ifail] = ...
  nag_rand_field_2d_predef_setup( ...
         ns, xmin, xmax, ymin, ymax, maxm, var, ...
         icov2, params, 'icorr', icorr);

fprintf('\nSize of embedding matrix = %d\n\n', m(1)*m(2));

% Display approximation information if approximation used
if approx == 1
  fprintf('Approximation required\n\n');
  fprintf('rho = %10.5f\n', rho);
  fprintf('eig = %10.5f%10.5f%10.5f\n', eig(1:3));
  fprintf('icount = %d\n', icount);
else
  fprintf('Approximation not required\n\n');
end

% Initialize state array
genid = nag_int(1);
subid = nag_int(1);
seed  = [nag_int(14965)];
[state, ifail] = nag_rand_init_repeat( ...
                        genid, subid, seed);

% Compute s random field realisations
s = nag_int(5);
[state, z, ifail] = nag_rand_field_2d_generate( ...
                           ns, s, m, lam, rho, state);

% Display realisations

% Set row labels to grid points (column label is realisation number).
rlabs = cell(ns(1)*ns(2), 1);
for j=1:ns(2)
  for i=1:ns(1)
    if i == 1
      rlabs{(j-1)*ns(1)+i} = sprintf('%6.1f%6.1f', xx(i), yy(j));
    else
      rlabs{(j-1)*ns(1)+i} = sprintf('%6.1f     .', xx(i));
    end
  end
end

% Matrix printing parameters
mtitle = 'Random field realisations (x,y coordinates first):';
matrix = 'General';
diag   = 'Non-unit';
fmt    = 'f10.5';
rlabel = 'Character';
clabel = 'Integer';
clabs  = {' '};
ncols  = nag_int(80);
indent = nag_int(0);

[ifail] = nag_file_print_matrix_real_gen_comp( ...
                 matrix, diag, z, fmt, mtitle, rlabel, rlabs, clabel, ...
                 clabs, ncols, indent);

% diary off
