function nag_sparse_complex_gen_precon_bdilu_example

% diary nag_sparse_complex_gen_precon_bdilu_example.r
fprintf('nag_sparse_complex_gen_precon_bdilu example results\n\n');

% Sparse matrix A
n   = nag_int(9);
nz  = nag_int(33);
a    = zeros(20*nz, 1);
irow = zeros(20*nz, 1, nag_int_name);
icol = zeros(20*nz, 1, nag_int_name);
a(1:nz) = [ 96 - 64i; -20 + 22i; -36 + 14i; 
           -12 + 10i;  96 - 64i; -20 + 22i; -36 + 14i;
           -12 + 10i;  96 - 64i; -36 + 14i;
           -28 + 18i;  96 - 64i; -20 + 22i; -36 + 14i;
           -28 + 18i; -12 + 10i;  96 - 64i; -20 + 22i; -36 + 14i;
           -28 + 18i; -12 + 10i;  96 - 64i; -36 + 14i;
           -28 + 18i;  96 - 64i; -20 + 22i;
           -28 + 18i; -12 + 10i;  96 - 64i; -20 + 22i; 
           -28 + 18i; -12 + 10i;  96 - 64i];
irow(1:nz) = [1; 1; 1;  2; 2; 2; 2;  3; 3; 3;  4; 4; 4; 4;  5; 5; 5; 5; 5; 
              6; 6; 6; 6;  7; 7; 7;  8; 8; 8; 8;  9; 9; 9];
icol(1:nz) = [1; 2; 4;  1; 2; 3; 5;  2; 3; 6;  1; 4; 5; 7;  2; 4; 5; 6; 8; 
              3; 5; 6; 9;  4; 7; 8;  5; 7; 8; 9;  6; 8; 9];

% 3 blocks
nb    = 3;
nover = 1;
lfill = [nag_int(0); 0; 0];
dtol  = [0; 0; 0];
pstrat = {'n'; 'n'; 'n'};
milu   = {'n'; 'n'; 'n'};

% Define diagonal block indices.
% In this example use blocks of MB consecutive rows and initialise
% assuming no overlap.
mb = idivide(n+nb-1, nag_int(nb));
istb = zeros(nb+1, 1, nag_int_name);
indb = zeros(3*n, 1, nag_int_name);
ipivp = zeros(3*n, 1, nag_int_name);
ipivq = zeros(3*n, 1, nag_int_name);
istb(1:nb) = [1:mb:nb*mb];
istb(nb+1) = n+1;
indb(1:n)  = [1:n];

% Modify indb and istb to account for overlap.
[istb, indb, ifail] = nag_sparse_complex_gen_precon_bdilu_overlap(n, nz, irow, icol, nb, ...
                                    istb, indb, 3*n, nover);
if (ifail == -999)
  error('indb is too small, size of indb = %d', numel(indb));
end

% Output matrix and blocking details
fprintf('\nOriginal matrix\n');
fprintf(' n   = %d\n', n);
fprintf(' nz  = %d\n', nz);
fprintf(' nb  = %d\n', nb);
for k=1:nb
  fprintf(' Block %d: order = %d, start row = %d\n', k, istb(k+1)-istb(k), ...
              min(indb(istb(k):istb(k+1)-1)));
end

% Calculate Factorisation
[a, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm, ifail] = ...
nag_sparse_complex_gen_precon_bdilu( ...
       n, nz, a, irow, icol, istb, indb, ...
       lfill, dtol, milu, ipivp, ipivq, 'pstrat', pstrat);

% Output details of the factorization
fprintf('\nFactorization\n');
fprintf(' nnzc = %d\n\n', nnzc);
fprintf(' Elements of factorization\n\n');
fprintf('        i   j               c(i,j)              index\n');
for k=1:nb
  fprintf(' C_%d   ----------------------------------------------\n', k);
  % Elements of the k-th block
  fmt = '     %4d%4d   (%13.5e,%13.5e)%8d\n';
  for i = istr(istb(k)):istr(istb(k+1))-1
    fprintf(fmt, irow(i), icol(i), real(a(i)), imag(a(i)), i);
  end
end

fprintf('\n Details of factorized blocks\n\n');
if max(npivm) > 0
  % Including pivoting details.
  fprintf('  k   i      istr(i)  idiag(i)   indb(i)  ipivp(i)  ipivq(i)\n');
   fmt1 = ' %4d%4d%10d%10d%10d%10d%10d\n';
   fmt2 = ' %8d%10d%10d%10d%10d%10d\n';
  for k=1:nb
    i = istb(k);
    fprintf(fmt1, k, i, istr(i), idiag(i), indb(i), ipivp(i), ipivq(i));
    for i = istb(k)+1:istb(k+1)-1
      fprintf(fmt2, i, istr(i), idiag(i), indb(i), ipivp(i), ipivq(i));
    end
    fprintf(' ------------------------------------\n');
  end
else
  % No pivoting on any block.
  fprintf('  k   i      istr(i)  idiag(i)   indb(i)\n');
  for k=1:nb
    i = istb(k);
    fprintf('%3d%4d%10d%10d%10d\n', k, i, istr(i), idiag(i), indb(i));
    for i = istb(k)+1:istb(k+1)-1
      fprintf('%7d%10d%10d%10d\n', i, istr(i), idiag(i), indb(i));
    end
    fprintf(' ------------------------------------\n');
  end
end

% diary off

function [istb, indb, ifail] = nag_sparse_complex_gen_precon_bdilu_overlap(n, nz, irow, icol, nb, ...
                                             istb, indb, lindb, nover)

  ifail = 0;

  % This function takes a set of row indices indb defining the diagonal
  % blocks to be used in nag_sparse_complex_gen_precon_bdilu to define a block Jacobi or additive Schwarz
  % preconditioner, and expands them to allow for nover levels of overlap.
  % The pointer array istb is also updated accordingly, so that the returned
  % values of istb and indb can be passed to nag_sparse_complex_gen_precon_bdilu to define overlapping
  % diagonal blocks.
  iwork = zeros(3*n+1, 1, nag_int_name);

  % Find the number of non-zero elements in each row of the matrix A, and
  % the start address of each row. Store the start addresses in
  % iwork(n+1,...,2*n+1).
  for k=1:nz
    iwork(irow(k)) = iwork(irow(k)) + 1;
  end
  iwork(n+1) = 1;
  for i = 1:n
    iwork(n+i+1) = iwork(n+i) + iwork(i);
  end

  % Loop over blocks
  for k=1:nb
    % Initialize marker array.
    iwork(1:n) = 0;

    % Mark the rows already in block k in the workspace array.
    for l = istb(k):istb(k+1)-1
      iwork(indb(l)) = 1;
    end

    % Loop over levels of overlap.
    for iover=1:nover
      % Initialize counter of new row indices to be added.
      ind = 0;

      % Loop over the rows currently in the diagonal block.
      for l = istb(k):istb(k+1)-1
        row = indb(l);

        % Loop over non-zero elements in row
        for i = iwork(n+row):iwork(n+row+1)-1

          % If the column index of the non-zero element is not in the
          % existing set for this block, store it to be added later, and
          % mark it in the marker array.
          if (iwork(icol(i))==0)
            iwork(icol(i)) = 1;
            ind = ind + 1;
            iwork(2*n+1+ind) = icol(i);
          end
        end
      end

      % Shift the indices in indb and add the new entries for block k.
      % Change istb accordingly.
      nadd = ind;
      if (istb(nb+1)+nadd-1>lindb) Then
        ifail = -999;
        return;
      end

      for i = istb(nb+1) - 1:-1:istb(k+1)
        indb(i+nadd) = indb(i);
      end
      n21 = 2*n + 1;
      ik = istb(k+1) - 1;
      indb(ik+1:ik+nadd) = iwork(n21+1:n21+nadd);
      istb(k+1:nb+1) = istb(k+1:nb+1) + nadd;
    end
  end
