function nag_sparse_real_symm_precon_ichol_solve_example

% diary nag_sparse_real_symm_precon_ichol_solve_example.r
fprintf('nag_sparse_real_symm_precon_ichol_solve example results\n\n');

% Solve sparse symmetric system Ax = b using CG method with
% Incomplete Cholesky preconditioning (IC)

% Define A and b 
n  = nag_int(9);
nz = nag_int(23);
a = zeros(3*nz,1);
irow = zeros(3*nz,1,nag_int_name);
icol = irow;
a(1:nz)    = [ 4    -1     6     1     2     3     2     4 ...
               1     2     6    -4     1    -1     6    -1 ...
              -1     3     1     1    -1     1     4 ];
irow(1:nz) = [ 1     2     2     3     3     4     5     5 ...
               6     6     6     7     7     7     7     8 ...
               8     8     9     9     9     9     9 ];    
icol(1:nz) = [ 1     1     2     2     3     4     1     5 ...
               3     4     6     2     5     6     7     4 ...
               6     8     1     5     6     8     9 ];    
b          = [4.10 -2.94  1.41 ...
              2.53  4.35  1.29 ...
              5.01  0.52  4.57];

% Setup IC factorization

lfill  = nag_int(-1);
dtol   = 0;
mic    = 'N';
dscale = 0;
ipiv   = zeros(n, 1, nag_int_name);
[a, irow, icol, ipiv, istr, nnzc, npivm, ifail] = ...
    nag_sparse_real_symm_precon_ichol(...
        nz, a, irow, icol, lfill, dtol, mic, dscale, ipiv);

% Solve Ax = b, using Cholesky factorization
[x, ifail] = nag_sparse_real_symm_precon_ichol_solve(...
                   a, irow, icol, ipiv, istr, 'C', b);
fprintf('\n   Solution of linear System\n');
fprintf('%16.4f\n', x);

% Redo calculation after reverse Cuthill-McKee permutation for
% bandwidth reduction

% Reverse Cuthill-McKee
[irow,icol,a,perm_fwd,perm_inv] =  do_rcm(n,nz,irow,icol,a);

% Repeat factorization and solution
[a, irow, icol, ipiv, istr, nnzc, npivm, ifail] = ...
    nag_sparse_real_symm_precon_ichol(...
        nz, a, irow, icol, lfill, dtol, mic, dscale, ipiv);

% Permute rhs vector and solution (PAP^T)(Px) = Pb.
y = b(perm_fwd(:));
[u, ifail] = nag_sparse_real_symm_precon_ichol_solve(...
                   a, irow, icol, ipiv, istr, 'C', y);

x = u(perm_inv(:));
fprintf('\n   Solution of linear System with reverse Cuthill-McKee\n');
fprintf('%16.4f\n', x);

% diary off

function [irow_out,icol_out,a_out,perm_fwd,perm_inv] = ...
         do_rcm(n,nz,irow,icol,a)

  % Reverse Cuthill-McKee for symmetric matrix

  % Add upper triangle, sort and remove duplicate diagonal elements
  irow(nz+1:2*nz) = icol(1:nz);
  icol(nz+1:2*nz) = irow(1:nz);
  a(nz+1:2*nz)    = a(1:nz);
  nnz = nz + nz;
  [nnz,a,icol,irow,istr,ifail] = nag_sparse_real_gen_sort(n,nnz,a,icol,irow,'R','F');

  % Reverse Cuthill-McKee
  lopts(1:5) = [false false true true true];
  mask = zeros(n,1,nag_int_name);
  [perm_fwd, info, ifail] = nag_sparse_sym_rcm(istr, irow, lopts, mask,'n',n);

  % Inverse perm
  perm_inv(perm_fwd(1:n)) = [1:n];

  % Apply permutation on column/row indices
  icol(1:nnz) = perm_inv(icol(1:nnz));
  irow(1:nnz) = perm_inv(irow(1:nnz));

  % collect only lower triangle
  j = 0;
  for i=1:nnz
    if icol(i)<=irow(i)
      j = j + 1;
      a(j) = a(i);
      irow(j) = irow(i);
      icol(j) = icol(i);
    end
  end

  nnz = nag_int(j);
  % Sort collected lower triangle
  [nnz, a, irow, icol, istr, ifail] = ...
  nag_sparse_real_symm_sort(...
         n, nnz, a, irow, icol, 'S', 'K');

  % copy to output arrays
  a_out = zeros(3*nz,1);
  irow_out = zeros(3*nz,1,nag_int_name);
  icol_out = irow_out;
  a_out(1:nnz) = a(1:nnz);
  irow_out(1:nnz) = irow(1:nnz);
  icol_out(1:nnz) = icol(1:nnz);
