function nag_sparseig_complex_proc_example

% diary nag_sparseig_complex_proc_example.r
fprintf('nag_sparseig_complex_proc example results\n\n');

global rho;

n   = nag_int(100);
nev = nag_int(4);
ncv = nag_int(20);
rho = 10;
imon = 0;

irevcm = nag_int(0);
resid = complex(zeros(n,1));
v  = complex(zeros(n,ncv));
x  = complex(zeros(n,1));
mx = complex(zeros(n,1));
z  = complex(zeros(n,1));

% Initialisation Step
[icomm, comm, ifail] = nag_sparseig_complex_init( ...
                              n, nev, ncv);
[icomm, comm, ifail] = nag_sparseig_complex_option( ...
                              'Regular Inverse', icomm, comm);
[icomm, comm, ifail] = nag_sparseig_complex_option( ...
                              'Generalized', icomm, comm);

% Factorize B
h  = 1/double(n+1);
cl = complex(h*ones(n-1,1));
cd = complex(4*h*ones(n,1));
cu = cl;

[cl, cd, cu, cu2, ipiv, info] = nag_lapack_zgttrf( ...
                                       cl, cd, cu);

% Solve
while (irevcm ~= 5)
  [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    nag_sparseig_complex_iter( ...
           irevcm, resid, v, x, mx, comm, icomm);
  if (irevcm == 1 || irevcm == -1)
    % Solve By = Ax;
    ax = nag_sparseig_complex_proc_Ax(n, x);
    [x, info] = nag_lapack_zgttrs( ...
                       'N', cl, cd, cu, cu2, ipiv, ax);
  elseif (irevcm == 2)
    % y = Bx
    x = nag_sparseig_complex_proc_Ax(n, x);
  elseif (irevcm == 4 && imon==1)
    [niter, nconv, ritz, rzest] = nag_sparseig_complex_monit( ...
                                         icomm, comm);
    fprintf(['Iteration %2d, No. converged = %d, ', ...
             'norm of estimates = %10.2e\n'], ...
            niter, nconv, norm(rzest(1:nev),2));
  end
end

% Post-process to compute eigenvalues/vectors
sigma = complex(0);
[nconv, d, z, v, comm, icomm, ifail] = ...
nag_sparseig_complex_proc( ...
       sigma, resid, v, comm, icomm);

fprintf('Largest %d Eigenvalues are:\n',nconv);
fprintf('%12.4f%+12.4fi\n',[real(d(1:nconv)) imag(d(1:nconv))]');

% diary off

function [y] = nag_sparseig_complex_proc_Ax(n, x)

  global rho;

  y = complex(ones(n,1));

  h = 1/double(n+1);
  s = rho/2;
  dd = complex(2/h);
  dl = complex(-1/h - s);
  du = complex(-1/h + s);

  y(1) = dd*x(1) + du*x(2);
  for j=2:n-1
    y(j) = dl*x(j-1) + dd*x(j) +du*x(j+1);
  end
  y(n) = dl*x(n-1) + dd*x(n);


function [y] = nag_sparseig_complex_proc_Bx(n,x)

  y = complex(ones(n,1));

  h = 1/double(n+1);
  dd = complex(4*h);
  dl = complex(h);
  du = complex(h);
  y(1) = dd*x(1) +du*x(2);
  for j=2:n-1
    y(j) = dl*x(j-1) + dd*x(j) +du*x(j+1);
  end
  y(n) = dl*x(n-1) + dd*x(n);
