function nag_sparseig_real_symm_iter_example

% diary nag_sparseig_real_symm_iter_example.r
fprintf('nag_sparseig_real_symm_iter example results\n\n');

fprintf('Example 1:\n');
ex1;
fprintf('Example 2:\n\n');
ex2;

% diary off

function ex1
  n   = nag_int(100);
  nev = nag_int(4);
  ncv = nag_int(10);
  imon = 0;

  irevcm = nag_int(0);
  resid = zeros(n,1);
  v  = zeros(n,ncv);
  x  = zeros(n,1);
  mx = zeros(n,1);

  sig = 0;
  
  h   = 1/double(n+1);
  h2  = h*h;
  ad(1:n)  = 2/h2 - sig;
  adl(1:n) = -1/h2;
  adu(1:n) = adl(1:n);

  [adl, ad, adu, adu2, ipiv, info] = nag_lapack_dgttrf( ...
                                            adl, ad, adu);

  % Initialisation Step
  [icomm, comm, ifail] = nag_sparseig_real_symm_init( ...
                                n, nev, ncv);
  [icomm, comm, ifail] = nag_sparseig_real_symm_option( ...
                                'Shifted Inverse', icomm, comm);
  
  % Solve
  while (irevcm ~= 5)
    [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    nag_sparseig_real_symm_iter( ...
           irevcm, resid, v, x, mx, comm, icomm);
    if (irevcm == 1 || irevcm == -1)
      [x, info] = nag_lapack_dgttrs( ...
                         'N', adl, ad, adu, adu2, ipiv, x);
    elseif (irevcm == 4 && imon==1)
      [niter, nconv, ritz, rzest] = nag_sparseig_real_symm_monit( ...
                                           icomm, comm);
      fprintf(['Iteration %2d, No. converged = %d, ', ...
               'norm of estimates = %10.2e\n'], ...
              niter, nconv, norm(rzest(1:nev),2));
    end
  end

  % Post-process to compute eigenvalues/vectors
  [nconv, d, z, v, comm, icomm, ifail] = ...
  nag_sparseig_real_symm_proc( ...
         sig, resid, v, comm, icomm);

  fprintf('\nThe %d Eigenvalues of smallest magnitude are:\n',nconv);
  disp(d(1:nconv));

function ex2

  m = nag_int(500);
  n = nag_int(100);
  nev = nag_int(4);
  ncv = nag_int(10);

  irevcm = nag_int(0);
  resid = zeros(n,1);
  v = zeros(n,ncv);
  x = zeros(n,1);
  mx = zeros(n,1);

  sigma = 0;

  % Initialisation Step
  [icomm, comm, ifail] = nag_sparseig_real_symm_init( ...
                                n, nev, ncv);

  % Solve
  while (irevcm ~= 5)
    [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    nag_sparseig_real_symm_iter( ...
           irevcm, resid, v, x, mx, comm, icomm);
    if (irevcm == 1 || irevcm == -1)
      % x = A^TAx
      y = nag_sparseig_real_symm_iter_Ax(m,n,x);
      x = nag_sparseig_real_symm_iter_Atx(m,n,y);
    end
  end

  % Post-process
  [nconv, d, z, v, comm, icomm, ifail] = ...
  nag_sparseig_real_symm_proc( ...
         sigma, resid, v, comm, icomm);
  
  % Singular values (squared) are returned in the first column of D and the
  % corresponding right singular vectors are in the V(:,1:nconv).

  for j = 1:nconv
    d(j,1) = sqrt(d(j,1));

    % Compute the left singular vectors from u = Av/sigval/norm(Av).
    ax = nag_sparseig_real_symm_iter_Ax(m,n,v(:,j));
    u(:,j) = ax/norm(ax);

    % Compute the residual norm ||A*v - sigma*u|| for the leading terms
    resid(j) = norm(ax - d(j,1)*u(:,j));
  end

  fprintf('Leading %d singular values and direct residuals:\n',nconv);
  fprintf('%9.5f%12.2e\n',[d(1:nconv) resid(1:nconv)]');


function [y] = nag_sparseig_real_symm_iter_Ax(m,n,x)

  y = zeros(m,1);

  h = 1/double(m+1);
  k = 1/double(n+1);
  t = 0;

  for j=1:n
    t = t + k;
    s = 0;
    for l = 1:j
      s = s + h;
      y(l) = y(l) + k*s*(t-1)*x(j);
    end
    for l = j+1:m
      s = s + h;
      y(l) = y(l) + k*t*(s-1)*x(j);
    end
  end

function [y] = nag_sparseig_real_symm_iter_Atx(m,n,x)

  y = zeros(n,1);

  h = 1/double(m+1);
  k = 1/double(n+1);
  t = 0;

  for j=1:n
    t = t + k;
    s = 0;
    for l = 1:j
      s = s + h;
      y(j) = y(j) + k*s*(t-1)*x(l);
    end
    for l = j+1:m
      s = s + h;
      y(j) = y(j) + k*t*(s-1)*x(l);
    end
  end
