function nag_tsa_cp_pelt_user_example

% diary nag_tsa_cp_pelt_user_example.r
fprintf('nag_tsa_cp_pelt_user example results\n\n');

% Input series
y = [ 0.00; 0.78; 0.02; 0.17; 0.04; 1.23; 0.24; 1.70; 0.77; 0.06;
      0.67; 0.94; 1.99; 2.64; 2.26; 3.72; 3.14; 2.28; 3.78; 0.83;
      2.80; 1.66; 1.93; 2.71; 2.97; 3.04; 2.29; 3.71; 1.69; 2.76;
      1.96; 3.17; 1.04; 1.50; 1.12; 1.11; 1.00; 1.84; 1.78; 2.39;
      1.85; 0.62; 2.16; 0.78; 1.70; 0.63; 1.79; 1.21; 2.20; 1.34;
      0.04; 0.14; 2.78; 1.83; 0.98; 0.19; 0.57; 1.41; 2.05; 1.17;
      0.44; 2.32; 0.67; 0.73; 1.17; 0.34; 2.95; 1.08; 2.16; 2.27;
      0.14; 0.24; 0.27; 1.71; 0.04; 1.03; 0.12; 0.67; 1.15; 1.10;
      1.37; 0.59; 0.44; 0.63; 0.06; 0.62; 0.39; 2.63; 1.63; 0.42;
      0.73; 0.85; 0.26; 0.48; 0.26; 1.77; 1.53; 1.39; 1.68; 0.43];

% The cost function is a function of the sum of Y, so for
% efficiency we will calculate the cumulative sum
% It should be noted that this may introduce some rounding issues
% with very extreme data, we also pre-pend a value of 0
csy = [0.0; cumsum(y)];

% Shape parameter used in the cost function
a = 2.1;

% The value of K is defined by the cost function being used
% in this example a value of 0.0 is the required value
k = 0;

% The cumulative sum of the input series and shape parameter
% constitute the information that needs to be passed to the
% costfun, so pack them together into a cell array which will
% get passed through the NAG function
user = {csy; a};

% Length of the input series
n = nag_int(numel(y));

% Penalty term
beta = 3.4;

% Drop small regions
minss = nag_int(3);

[tau] = nag_tsa_cp_pelt_user( ...
               n, beta, k, @costfn, 'minss', minss, 'user', user);

% Print the results
fprintf('  -- Change Points --\n');
fprintf('  Number     Position\n');
fprintf(' =====================\n');
for i = 1:numel(tau)
  fprintf(' %4d       %6d\n', i, tau(i));
end

% Plot the results
fig1 = figure;

% Plot the original series
plot(y,'Color','red');

% Mark the change points, drop the last one as it is always
% at the end of the series
xpos = transpose(double(tau(1:end-1))*ones(1,2));
ypos = diag(ylim)*ones(2,numel(tau)-1);
line(xpos,ypos,'Color','black');

% Add labels and titles
title({'{\bf nag_tsa_cp_pelt_user Example Plot}',
      'Simulated time series and the corresponding changes in scale b',
      'assuming y ~ Ga(2.1,b)'});
xlabel('{\bf Time}');
ylabel('{\bf Value}');

% print(fig1,'-dpng','-r75','nag_tsa_cp_pelt_user_fig1.png');
% print(fig1,'-deps','-r75','nag_tsa_cp_pelt_user_fig1.eps');

% diary off

function [c,user,info] = costfn(ts, r, user, info)
  % Cost function, C. This cost function is based on the likelihood of
  % the gamma distribution
  csy = user{1};
  a = user{2};

  % Only need to test which way around ts and r are once
  if (ts<r(1))
    si = csy(r+1) - csy(ts+1);
    dn = double(r - ts);
  else
    si = csy(ts+1) - csy(r+1);
    dn = double(ts - r);
  end
  c = (2*dn*a) .* (log(si) - log(dn*a));

  % Set info nonzero to terminate execution for any reason
  info = nag_int(0);
