function nag_tsa_inhom_iema_example

% diary nag_tsa_inhom_iema_example.r
fprintf('nag_tsa_inhom_iema example results\n\n');

m     = nag_int(2);
inter = [nag_int(3); 2];
tau   = [0.5; 2; 8];
sinit = [5; 0.5; 0.5; 0.5];
nb    = [5, 10, 15];
t     = cell(3, 1);
iema  = cell(3, 1);
t{1}    = [ 7.5;  8.2; 18.1; 22.8; 25.8];
iema{1} = [ 0.6;  0.6;  0.8;  0.1;  0.2];
t{2}    = [26.8; 31.1; 38.4; 45.9; 48.2; 48.9; 57.9; 58.5; 63.9; 65.2];
iema{2} = [ 0.2;  0.5;  0.7;  0.1;  0.4;  0.7;  0.8;  0.3;  0.2;  0.5];
t{3}    = [66.6; 67.4; 69.3; 69.9; 73.0; 75.6; 77.0; 84.7; 86.8; 88.0; ...
           88.5; 91.0; 93.0; 93.7; 94.0];
iema{3} = [ 0.2;  0.3;  0.8;  0.6;  0.1;  0.7;  0.9;  0.6;  0.3;  0.1;  ...
            0.1;  0.4;  1.0;  1.0;  0.1];

fprintf('             Time       Iterated EMA\n');

fig1 = figure;
hold on
linecol = {'blue','green','red'};
xlabel('Time');
ylabel('Value');
title({'Simulated inhomogeneous time series and corresponding',
       'EMA(\tau,2;y) for 3 \tau values'});
tm = [t{1}; t{2}; t{3}];
jm = [iema{1}; iema{2}; iema{3}];
plot(tm,jm,'cs');

% Loop over different values of tau
for k = 1:numel(tau);
  % Loop over each block of data.
  for i = 1:numel(nb)
    if i == 1
      % process first block and create pn
      [ema, pn, rcomm, ifail] = ...
      nag_tsa_inhom_iema( ...
             iema{i}, t{i}, tau(k), m, sinit, inter, 'rcomm', zeros(22,1));
      jm = ema;
    else
      % Update the iterated EMA for this block of data, overwriting the input
      % data with the iterated EMA.
      [ema, pn, rcomm, ifail] = ...
      nag_tsa_inhom_iema( ...
             iema{i}, t{i}, tau(k), m, sinit, inter, 'pn', pn, 'rcomm', rcomm);
      jm = [jm; ema];
    end

    % Display the results for this block of data (tau = 2 only)
    if k==2
      for l=1:nb(i)
        fprintf('%3d    %10.1f    %10.3f\n', pn-nb(i)+l, t{i}(l), ema(l));
      end
      fprintf('\n');
    end
  end
  plot(tm,jm,linecol{k});
end
legend('Original data', '\tau=0.5', '\tau=2', '\tau=8', ...
       'Location', 'northwest');
legend('boxoff');
hold off
% print(fig1,'-dpng','-r75','nag_tsa_inhom_iema_fig1.png');
% print(fig1,'-deps','-r75','nag_tsa_inhom_iema_fig1.eps');

% diary off
