function nag_wav_2d_coeff_ins_example

% diary nag_wav_2d_coeff_ins_example.r
fprintf('nag_wav_2d_coeff_ins example results\n\n');

% 2D Data
m = nag_int(7);
n = nag_int(6);
a = zeros(m,n);
a(1:2:m,:) = 0.01;
a(2:2:m,:) = 1;

% Add Noise
genid = nag_int(3);
subid = nag_int(0);
seed(1) = nag_int(642521);
[state, ifail] = nag_rand_init_repeat(genid, subid, seed);
[state, x, ifail] = nag_rand_dist_normal(m*n, 0, 1.0e-4, state);
an = a + reshape(x,[m,n]);

fprintf('\nInput data a:\n');
disp(a);
fprintf('\nNoisy data an:\n');
disp(an);

% Wavelet setup
wavnam = 'DB6';
mode = 'Period';
wtrans = 'Multilevel';
[nwl, nf, nwct, nwcn, icomm, ifail] = ...
  nag_wav_2d_init(...
        wavnam, wtrans, mode, m, n);

% Multi-level wavelet transform on noisy data
lenc = nwct;
% Perform Discrete Wavelet transform
[c, dwtlvm, dwtlvn, icomm, ifail] = ...
  nag_wav_2d_multi_fwd(...
        an, lenc, nwl, icomm);

% Reconstruct without thresholding of detail coefficients
[b, ifail] = nag_wav_2d_multi_inv(nwl, c, m, n, icomm);
 
% Mean square error of noisy reconstruction
mse = (norm(reshape(a-b,[m*n,1]))^2)/(double(m*n));
fprintf('Without denoising Mean Square Error is %9.6f\n',mse);

% De-noise by applying hard threshold to detail coefficients
thresh = 0.01*sqrt(2*log(double(m*n)));
nt = 0;
nnt = 0;
for ilev = 1:nwl
  level = nag_int(nwl - ilev + 1);

  for detail = nag_int(1:3)
    % Extract the selected set of coefficients. 
    [d, icomm, ifail] = nag_wav_2d_coeff_ext(...
                               level, detail, c, icomm);
  
    % Threshold
    d1 = dwtlvm(ilev);
    d2 = dwtlvn(ilev);
    for i = 1:d1
      for j = 1:d2
        if abs(d(i,j))<thresh
           d(i,j) = 0;
           nt = nt + 1;
        end
        nnt = nnt + 1;
      end
    end
    % Insert de-noised coefficients back into c
    [c, icomm, ifail] = nag_wav_2d_coeff_ins(...
                               level, detail, c, d, icomm);
  end
end

fprintf('\nNumber of coefficients denoised is %3d out of %3d\n',nt,nnt);

% Reconstruct data after threholding
[b, ifail] = nag_wav_2d_multi_inv(nwl, c, m, n, icomm);

% Mean square error of de-noised reconstruction
mse = (norm(reshape(a-b,[m*n,1]))^2)/(double(m*n));
fprintf('With denoising Mean Square Error is %9.6f\n\n',mse);

disp('Reconstruction of denoised input: ');
disp(b);

% diary off
