function s21bf_example

% diary s21bf_example.r
fprintf('s21bf example results\n\n');

phi = [pi/6  pi/3   pi/2];
dm  = [1/4   1/2    3/4];
result = phi;

for j = 1:numel(phi)
  [result(j), ifail] = s21bf(phi(j), dm(j));
end

fprintf('    phi      m        E(phi|m)\n');
fprintf(' %7.2f %7.2f %12.4f\n', [phi; dm; result]);

s21bf_plot;

% diary off

function s21bf_plot
  phi = [1:0.02:1.56];
  m = [0.5:0.02:0.98,0.982:0.002:1];
  F = zeros(numel(phi),numel(m));
  for i = 1:numel(phi)
    for j = 1:numel(m)
      [F(i,j), ifail] = s21bf(phi(i), m(j));
    end
  end

  fig1 = figure;
  [Y,X] = meshgrid(m,phi);
  surf(X,Y,F);
  xlabel('\Phi');
  ylabel('m');
  zlabel('F(\Phi,n)');
  title({'Incomplete Elliptic Integral of the second kind', ...
         'Classical (Legendre) form'});

  % print(fig1,'-dpng','-r75','s21bf_fig1.png');
  % print(fig1,'-deps','-r75','s21bf_fig1.eps');
