function varargout = editBoundary(varargin)

    gui_Singleton = 1;
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @editBoundary_OpeningFcn, ...
                       'gui_OutputFcn',  @editBoundary_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if nargin && ischar(varargin{1})
        gui_State.gui_Callback = str2func(varargin{1});
    end

    if nargout
        [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
    else
        gui_mainfcn(gui_State, varargin{:});
    end
end

function editBoundary_OpeningFcn(hObject, eventdata, handles, varargin)
    handles.output = hObject;
    
    if size(varargin,2) ~= 3 || ~isstruct(varargin{1}) || ~isstruct(varargin{2})
        delete(gcf);
        return;
    end

    handles.boundary = varargin{1};
    handles.constraints = varargin{2};
    handles.activeBoundary = varargin{3};

    handles.old_boundary = handles.boundary;
    handles.old_constraints = handles.constraints;
        
    
    handles.selected_right = 1;
    handles.selected_left = 1;
    handles.constraint = min(1,size(handles.constraints,2));
    
    set(handles.editBoundary,'CloseRequestFcn',{@closeGUI,handles});

    handles.output = struct('boundary', {handles.boundary},'constraints',{handles.constraints} ); 
    
    updateLists(handles);
    
    % Update handles structure
    guidata(hObject, handles);

end

function closeGUI(src, evt, handles)
        uiresume(handles.editBoundary);
end

function varargout = editBoundary_OutputFcn(hObject, eventdata, handles) 
    uiwait(handles.editBoundary);
    handles = guidata(handles.editBoundary);
    varargout{1} = handles.output;  
%     
    delete(handles.editBoundary);
end



function text_bnd_name_Callback(hObject, eventdata, handles)
    handles.boundary(handles.activeBoundary).name = get(hObject,'String');
    guidata(hObject, handles );
    
end

function text_bnd_name_CreateFcn(hObject, eventdata, handles)

    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end

end

% --- Executes on button press in button_new_constraints.
function button_new_constraints_Callback(hObject, eventdata, handles)

    % extracts the largest current constraint tag.
    handles.tagID = size(handles.constraints,2);
    
    % set handles.constraint to be zero so that editConditions GUI knows
    % that we are adding a new condition
    handles.constraint = 0;
    
    % call editConditions GUI
    handles.constraints = editConditions(handles.constraints,handles.constraint,handles.tagID);
    
    % set handles.constraint to the new condition if created
    handles.constraint = length(handles.constraints);
    
    % update handles
    guidata(hObject, handles );
    
    % update the list of constraints
    updateLists(handles);

end

function button_edit_constraints_Callback(hObject, eventdata, handles)

    % if there are no constraints there is nothing to edit
    if isempty(handles.constraints)
        return
    end

    % extracts the largest current constraint tag.
    handles.tagID = size(handles.constraints,2);
    
    % call the editConditions GUI
    handles.constraints = editConditions(handles.constraints,handles.constraint,handles.tagID);
    
    
    if handles.constraint > length(handles.constraints) || handles.constraint < 1
        handles.constraint = 1;
    end
    
    guidata(hObject, handles );
    
    updateLists(handles);

end

% --- Executes on button press in button_delete_constraints.
function button_delete_constraints_Callback(hObject, eventdata, handles)
    
    if handles.constraint >= 1 && handles.constraint <= size(handles.constraints,2) 
       
        % unconstrain edges which currently have this tagID
        
        tagID = handles.constraints(handles.constraint).tag;
        for i = 1:size(handles.boundary,2)
            constrainedIndexes = find(handles.boundary(i).edge(3,:)==tagID);
            handles.boundary(i).edge(3,constrainedIndexes) = 0;
        end
        handles.constraints = handles.constraints([1:handles.constraint-1, handles.constraint+1:end]);
        
        % rename all tagged edges, so that tags match position of
        % constraint in structure handles.constraints
        
        for i = 1:size(handles.constraints,2)
            tagID = handles.constraints(i).tag;
            for j = 1:size(handles.boundary,2)
                constrainedIndexes = find(handles.boundary(j).edge(3,:) == tagID);
                handles.boundary(j).edge(3,constrainedIndexes) = i;
            end
            handles.constraints(i).tag = i;
        end
        
        % if constraint deleted was the last on the list, move the selected
        % down one.
        if handles.constraint > size(handles.constraints,2);
            handles.constraint = size(handles.constraints,2);
        end

        guidata(hObject, handles );
        updateLists(handles);
        
    end 
end

% --- Executes on button press in button_default_hot.
function button_default_hot_Callback(hObject, eventdata, handles)

    % extracts the largest current constraint tag.
    tagID = size(handles.constraints,2);
    tagID = tagID + 1;
    
    
    handles.constraints(tagID).name = 'Hot';
    handles.constraints(tagID).expression = '100';
    handles.constraints(tagID).type = 1;
    handles.constraints(tagID).tag = tagID;
    
    handles.constraints(tagID);
   
    handles.constraint = tagID;

    guidata(hObject, handles );
    updateLists(handles);
    
end

% --- Executes on button press in button_default_cold.
function button_default_cold_Callback(hObject, eventdata, handles)

    % extracts the largest current constraint tag.
    tagID = size(handles.constraints,2);
    tagID = tagID + 1;
    
    
    handles.constraints(tagID).name = 'Cold';
    handles.constraints(tagID).expression = '0';
    handles.constraints(tagID).type = 1;
    handles.constraints(tagID).tag = tagID;
    
    handles.constraints(tagID);
   
    handles.constraint = tagID;

    guidata(hObject, handles );
    updateLists(handles);
    
end

function menu_condition_Callback(hObject, eventdata, handles)
    handles.constraint = get(hObject,'Value');
    
    guidata(hObject, handles );
    updateLists(handles);

end

function menu_condition_CreateFcn(hObject, eventdata, handles)

    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end

end


function list_edges_free_Callback(hObject, eventdata, handles)
end

function list_edges_free_CreateFcn(hObject, eventdata, handles)

    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end
end

function button_cancel_Callback(hObject, eventdata, handles)
     handles.output = struct('boundary', {handles.old_boundary},'constraints',{handles.old_constraints} ); 
     guidata(hObject, handles);
     closeGUI( handles.editBoundary, [], handles );
end

function button_ok_Callback(hObject, eventdata, handles)
     handles.output = struct('boundary', {handles.boundary},'constraints',{handles.constraints} ); 
     guidata(hObject, handles);
     closeGUI( handles.editBoundary, [], handles );     
end


% --- Executes on button press in button_help.
function button_help_Callback(hObject, eventdata, handles)
    helpeditBoundary();
end

function list_edges_constrained_Callback(hObject, eventdata, handles)
end

function list_edges_constrained_CreateFcn(hObject, eventdata, handles)

    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end
end


function button_move_right_Callback(hObject, eventdata, handles)

    activeBoundary = handles.activeBoundary;
    freeIndexes = find(handles.boundary(activeBoundary).edge(3,:)==0);

    if size(handles.constraints,2) < 1 || length(freeIndexes) == 0
        return;
    end;
    
    
    freeEdges = handles.boundary(activeBoundary).edge(:,freeIndexes);
    

    selected_left = get(handles.list_edges_free,'Value');
      
    freeEdges(3,selected_left) = handles.constraints(handles.constraint).tag;
    handles.boundary(activeBoundary).edge(:,freeIndexes) = freeEdges;

    
    guidata(hObject, handles);
    updateLists(handles);


end


function button_move_left_Callback(hObject, eventdata, handles)
    
    activeBoundary = handles.activeBoundary;
    constrainedIndexes = find(handles.boundary(activeBoundary).edge(3,:) ~= 0);
    
    if size(handles.constraints,2) < 1 || length(constrainedIndexes) == 0
        return;
    end;
    
    
    constrainedEdges = handles.boundary(activeBoundary).edge(:,constrainedIndexes);
    

    selected_right = get(handles.list_edges_constrained,'Value');
    
    constrainedEdges(3,selected_right) = 0;
    handles.boundary(activeBoundary).edge(:,constrainedIndexes) = constrainedEdges;

    
    guidata(hObject, handles);
    updateLists(handles);
end


function updateLists( handles )
    
    activeBoundary = handles.activeBoundary;
    % set the name box to have the name of the boundary in it
    set(handles.text_bnd_name,'String',handles.boundary(activeBoundary).name);
    % the number of edges of this boundary
    nb = length(handles.boundary(activeBoundary).edge);
    % generate the initial character array for the edge lists with a size
    % large enough to fit the longest of the constraint names
    le = [genCharArray('Edge #',nb),strvcat(num2str((1:nb)'))];
    le = cellstr(le);
    
    % the selected edges on the left or the right lists
    selected_right = get(handles.list_edges_constrained,'Value');
    selected_left = get(handles.list_edges_free,'Value');
    
    
    % if we have at least one constraint then...
    if length(handles.constraints) >= 1
        % number of constraints
        t = size(handles.constraints,2);
        % generate character array to go in constraints list
        l = [ strvcat( num2str((1:t)') ),genCharArray(' - ', t), strvcat(handles.constraints.name) ];
        % set the constraints list to be the above string
        set(handles.menu_condition,'String',l);
        % set the currently selected/created constraint to be showing on
        % the list
        set(handles.menu_condition,'Value',handles.constraint);
        
        % find all constrained edges
        % use an array of zeros to find indices of edge(3,:) that are 0 or
        % ~0
        Z = zeros(1,nb);
        % create a logical array of indices where edge(3,:) is non zero
        % (i.e.) constrained
        constrainedIndexes = (handles.boundary(activeBoundary).edge(3,:) ~= Z);
        
        % need to go through all the constraints and match their tags to
        % the tags of the constrainededges, and append the string cenames for
        % each edge with their associated constraint
        if max(+constrainedIndexes)
            for i = 1:t
                % extract tag of constraint i
                tagID = handles.constraints(i).tag;
                % create logical array to find indices of edges with this tag
                ci = Z+tagID;
                % pull out edges from edge
                ce = (handles.boundary(activeBoundary).edge(3,:) == ci);
                % pull out name of constraint
                name = handles.constraints(i).name;
                % append name to list of edge names if any edges are
                % constrained
                if max(ce)
                    % name of constraint in a cell array of strings
                    name = strcat(' -',name);
                    names = cellstr(char(ones(nb,1)*name));
                    % remove unnecessary strings
                    names = names(ce);
                    % append list of edge names
                    le(ce) = cellstr(strcat(le(ce),names));
                end
            end
        end
        % set list of constrained edges to contain all constrained edges
        set(handles.list_edges_constrained,'String',le(constrainedIndexes,:));
        set(handles.list_edges_constrained,'Value',1);        
        
    else
        % set conditions list to be empty
        set(handles.menu_condition,'String',' ');
        set(handles.menu_condition,'Value',1);   
        % and constrained edges list to be empty
        set(handles.list_edges_constrained,'String',' ');       
        set(handles.list_edges_constrained,'Value',1);          
    end    
    
    handles.boundary(activeBoundary).edge;  
    % zeros for use in finding unconstrained edges.
    Z = zeros(1,nb);
    % logical array of free edges
    freeIndexes = (handles.boundary(activeBoundary).edge(3,:) == Z);
    
    % set free edges list to be these
    set(handles.list_edges_free,'String',le(freeIndexes,:));
    set(handles.list_edges_free,'Value',1);

end

% a simple function to generate an array of characters
function res = genCharArray( text, nb)
    res = char(ones(nb,1)* text );
end
