function varargout = mesh_generator(varargin)

v = ver;

for i = 1:size(v,2)
    if (strcmp(v(i).Name,'MATLAB'))
        vv = textscan(v(i).Version,'%1d%c%2d');
    end
end

if (vv{1} < 7 || ( vv{1} == 7 && vv{3} < 6))
    errordlg('This demo does not work with R2007b or earlier.');
    return
end

gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @mesh_generator_OpeningFcn, ...
    'gui_OutputFcn',  @mesh_generator_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT
end

function mesh_generator_OpeningFcn(hObject, evt, handles, varargin)

% sets the warning output of NAG tools to be silent
warning('off', 'NAG:warning')

handles.output = hObject;
% load configuration

% if config file exists, load it

configFile = strcat(prefdir, '/meshGenConfig.xml');
if exist(configFile,'file')
    handles.config = load_config(configFile);
else
    % set the default configuration and save it
    handles.config.axes = [-1.5,1.5,-1.5,1.5];
    handles.config.grid = [1,0.2,0.2];
    handles.config.solution = [0 0 1];

    % Set the default parameters for the three triangulation
    % algorithms.
    handles.config.algorithms = struct('default', {1}, ...
        'incremental', struct('bspace', {'ones(1,nvb)*0.1'}, 'smooth', {1}, 'coeff', {0.75}, 'power', {0.25}, 'boundary', {10}), ...
        'delaunay', struct('weight', {'ones(1,nbinter)*0.1'}, 'npropa', {-5}, 'boundary', {5}), ...
        'front', struct('weight', {'ones(1,nbinter)*0.1'}, 'boundary', {5}) );
    % have put in another variable that determines the number of points on the boundary. NF 08/10/08
    % (for the Advancing Front algorithm).

    save_config(handles.config,configFile);
end

% all the state variables
handles.state.activeDomain = 0;                 % id of the current selected domain
handles.state.filename = 0;                     % filename of the current file
handles.state.showEdgesNumbers = 0;             % flag to show edge numbers
handles.state.showConstraintLabels = 0;         % flag to show constraint labels on edges
handles.state.tool = 3;                         % id of current tool (1=circle, 2=rectangle, 3=polyline)
handles.state.playing = 0;                      % flag to know if solution is playing
handles.state.paused = 0;                       % flag to know if solution is paused
handles.state.recording = 0;                    % flag to know if solution is being recorded
handles.frames = [];                            % frames for the solution recording

% sets the view and saves it
view([0 90]);
handles.state.view = view();

% change the toolbar toggle buttons value according to the configuration
if handles.config.solution(1)
    set( handles.toolbar_tool_toggleMesh, 'state', 'on');
end

if handles.config.solution(2)
    set( handles.toolbar_tool_toggleGrid, 'state', 'on');
end

% set static text box to display nothing
set(handles.panel_helptext,'String','')

%     if handles.config.solution(3)
%         set( handles.toolbar_tool_toggleSmooth, 'state', 'on');
%     end

% sets the default algorithm from the configuration
handles.state.algorithm = handles.config.algorithms.default;

% structure to hold the domains' properties
handles.domains = struct ('boundaries', struct('coor', {}, 'edge', {}, 'grad', {}, 'reverse', {}, 'name', {}, 'type', {}),...
    'boundary', struct('coor', {}, 'edge', {}, 'nbinter', {0}), ...
    'mesh', struct('coor', {}, 'conn', {} ), ...
    'activeBoundary', {1}, ...
    'showMesh', {false}, ...
    'changed', {false}, ...
    'bnd', struct('coor', {}, 'grad',{}, 'lined', {}, 'nlcomp',{}, 'lcomp', {} ), 'boundaryID',{});

% structure for the constraints
handles.constraints = struct( 'name', {}, 'expression', {}, 'type', {}, 'tag', {} );

% equation configuration
handles.fem.poisson.source = '0';
handles.fem.equation = 'laplace';
handles.fem.solutions = struct('result',{});
handles.fem.time = struct('T', {0.2}, 'dt', {0.01} );
handles.fem.init = '0';

% Update the axes view.
updateAxes(handles);

% sets the callbacks
set(handles.panel_Algorithm,'SelectionChangeFcn',@panel_Algorithm_SelectionChangeFcn);
set(handles.Main,'CloseRequestFcn',{@closeGUI,handles});
set(handles.menu_Quit,'Callback',{@closeGUI,handles});

% Display some helpful text telling the user how to start.
set(handles.panel_helptext, 'String', 'To start, select a boundary type (Circle, Rectangle or Polyline) from the toolbar.');

% Update handles structure
guidata(hObject, handles);
end

function closeGUI(src, evt, handles )

% This asks the user to confirm that they want to quit the program.
selection = questdlg('Are you sure you want to quit?','Closing confirmation','Yes','No','Yes');

switch selection
    case 'Yes'
        delete(gcf)
    case 'No'
        return
end
end

function varargout = mesh_generator_OutputFcn(hObject, eventdata, handles)
% change the output of the program
varargout{1} = handles.output;
end

function menu_New_Callback(hObject, eventdata, handles)

% resets all the defaults
handles.domains = struct ('boundaries', struct('coor', {}, 'edge', {}, 'grad', {}, 'reverse', {}, 'name', {}, 'type', {}),...
    'boundary', struct('coor', {}, 'edge', {}, 'nbinter', {0}), ...
    'mesh', struct('coor', {}, 'conn', {} ), ...
    'activeBoundary', {1}, ...
    'showMesh', {false}, ...
    'changed', {false}, ...
    'bnd', struct('coor', {}, 'grad',{}, 'lined', {}, 'nlcomp',{}, 'lcomp', {} ), 'boundaryID',{});
handles.constraints = struct( 'name', {}, 'expression', {}, 'type', {}, 'tag', {} );
handles.fem.poisson.source = '0';
handles.fem.equation = 'laplace';
handles.state.activeDomain = 0;
handles.state.filename = 0;

set(handles.toolbar_tool_record,'Enable','On')
set(handles.toolbar_tool_play,'Enable','On')
set(handles.toolbar_tool_pause,'Enable','On')
set(handles.toolbar_tool_stop,'Enable','On')
set(handles.toolbar_Open,'Enable','On')
set(handles.toolbar_Save,'Enable','On')
set(handles.toolbar_New,'Enable','On')
set(handles.button_Edit_boundary,'Enable','On')
set(handles.button_Delete_boundary,'Enable','On')
set(handles.toolbar_tool_polyline,'Enable','On')
set(handles.toolbar_tool_quad,'Enable','On')
set(handles.toolbar_tool_circle,'Enable','On')
set(handles.rbutton_Front,'Enable','On')

% save the data
guidata(hObject,handles);

% update the list and redraw the view
updateLists(hObject,handles);
redraw(handles);

end

function menu_Open_Callback(hObject, eventdata, handles)

% opens a dialog to ask a filename
[filename, pathname, filterindex] = uigetfile({'*.sim','Simulation (*.sim)'; '*.*',  'All Files (*.*)'}, 'Save as');
if ~isequal(filename,0) && ~isequal(pathname,0)
    handles.state.filename = fullfile(pathname,filename);
    guidata(hObject, handles);
else
    return;
end

% load the file
[infos, config, rbutton_state] = load_Domains(handles.state.filename);

% put the data into the structures
handles.domains = infos.domains;
handles.constraints = infos.constraints;
set(handles.rbutton_Front,'Value',rbutton_state);
set(handles.button_Delete_boundary,'Enable','Off')
set(handles.button_Edit_boundary,'Enable','Off')

% load the config settings 04/11/08 NF
handles.config = config;

handles.state.activeDomain = size( handles.domains, 2 );

% saves the structures
guidata(hObject, handles);

% update the lists
updateLists(hObject, handles);
guidata(hObject, handles);

% redraws the view
redraw(handles);

end

function menu_Save_Callback(hObject, eventdata, handles)
% if the current filename is not defined, use Save_as first to get the filename
if isequal(handles.state.filename,0)
    menu_Save_as_Callback(hObject, eventdata, handles)
    return
end

rbutton_state = get(handles.rbutton_Front,'Value');
% save the domains
save_Domains( handles.domains, handles.constraints, handles.config, handles.state.filename, rbutton_state);

end

function menu_Save_as_Callback(hObject, eventdata, handles)
% ask for a filename and call the Save_callback
[filename, pathname, filterindex] = uiputfile({'*.sim','Simulation (*.sim)'; '*.*',  'All Files (*.*)'}, 'Save as');
if ~isequal(filename,0) && ~isequal(pathname,0)
    handles.state.filename = fullfile(pathname,filename);
    guidata(hObject, handles);
    menu_Save_Callback(hObject, eventdata, handles)
end
end

function menu_Quit_Callback(hObject, eventdata, handles)
% ask for confirmation
closeGUI(handles);
end

function New_boundary(hObject, handles)

% if a domain already exists, sets the drawing mode to only draw the
% boundaries of the current domain to ease the drawing of inside
% boundaries
activeDomain = handles.state.activeDomain;
if handles.state.activeDomain ~= 0
    handles.domains(activeDomain).showMesh = false;
    redraw(handles);
end

switch handles.state.tool
    case 1
        set(handles.toolbar_tool_polyline,'Enable','Off')
        set(handles.toolbar_tool_quad,'Enable','Off')
        s = sprintf('Create Circle:\n\nLeft click to place centre of circle.\nLeft click again at desired radius distance');
        set(handles.panel_helptext,'String',s)
    case 2
        set(handles.toolbar_tool_circle,'Enable','Off')
        set(handles.toolbar_tool_polyline,'Enable','Off')
        s = sprintf('Create Rectangle:\n\nLeft click to place first corner of rectangle.\nLeft click again to place opposite corner of rectangle.');
        set(handles.panel_helptext,'String',s);
    case 3
        set(handles.toolbar_tool_circle,'Enable','Off')
        set(handles.toolbar_tool_quad,'Enable','Off')
        s = sprintf('Create Polygon:\n\nMake sure you create the outer boundary in an anticlockwise orientation and an inner boundary in a clockwise orientation.\nLeft click to place vertices of polygon.\nRight click to place final vertex.');
        set(handles.panel_helptext,'String',s);
end

set(handles.toolbar_tool_record,'Enable','Off')
set(handles.toolbar_tool_play,'Enable','Off')
set(handles.toolbar_tool_pause,'Enable','Off')
set(handles.toolbar_tool_stop,'Enable','Off')
set(handles.toolbar_Open,'Enable','Off')
set(handles.toolbar_Save,'Enable','Off')
set(handles.button_Edit_boundary,'Enable','Off')
set(handles.button_Delete_boundary,'Enable','Off')

% get the new boundary depending on the currently selected tool
%
% coor: coordinates of the points
% edge: 2xn array containing the index of the point for each edge
% reverse: true if the polygon is entered counter-clockwise
% problem: 1 if there was an error creating the new boundary
problem = 0;
switch handles.state.tool
    case 1  % circle
        [coor, edge, grad, reverse, problem] = inputCircle(handles, handles.config.grid(1),handles.config.grid(2), handles.config.grid(3), handles.config.axes(1), handles.config.axes(2), handles.config.axes(3), handles.config.axes(4));
    case 2  % rectangle
        [coor, edge, grad, reverse, problem] = inputQuad(handles,handles.config.grid(1),handles.config.grid(2), handles.config.grid(3), handles.config.axes(1), handles.config.axes(2), handles.config.axes(3), handles.config.axes(4));
    case 3  % polyline
        [coor, edge, grad, reverse, problem] = inputPolygon(handles,handles.config.grid(1),handles.config.grid(2), handles.config.grid(3), handles.config.axes(1), handles.config.axes(2), handles.config.axes(3), handles.config.axes(4));
end


switch handles.state.tool
    case 1
        set(handles.toolbar_tool_polyline,'Enable','On')
        set(handles.toolbar_tool_quad,'Enable','On')
        set(handles.panel_helptext,'String','')
    case 2
        set(handles.toolbar_tool_circle,'Enable','On')
        set(handles.toolbar_tool_polyline,'Enable','On')
        set(handles.panel_helptext,'String','')
    case 3
        set(handles.toolbar_tool_circle,'Enable','On')
        set(handles.toolbar_tool_quad,'Enable','On')
        set(handles.panel_helptext,'String','')
end

set(handles.toolbar_tool_record,'Enable','On')
set(handles.toolbar_tool_play,'Enable','On')
set(handles.toolbar_tool_pause,'Enable','On')
set(handles.toolbar_tool_stop,'Enable','On')
set(handles.toolbar_Open,'Enable','On')
set(handles.toolbar_Save,'Enable','On')
set(handles.button_Edit_boundary,'Enable','On')
set(handles.button_Delete_boundary,'Enable','On')

%  if the polygon is invalid, we don't add it
if size(coor,2) < 3 || problem ~= 0

    % if a domain already exists, reset the normal view for the
    % selected domain
    if handles.state.activeDomain ~= 0
        handles.domains(handles.state.activeDomain).showMesh = true;
    end

    % redraws and exists
    redraw(handles);
    return;
end

% if no active domain exists on the list, create a new one.
if handles.state.activeDomain == 0
    handles.state.activeDomain = 1;
    handles.domains(1).name = 'Domain';
end

% id is the location where the details of the new boundary will be stored.
id = size(handles.domains(handles.state.activeDomain).boundaries,2)+1;

% boundaryID is the index of the current boundary.  Set it to zero if it
% doesn't have a value (otherwise MATLAB thinks it's an empty (0 by 0)
% double matrix, which causes problems below when we try and use it).
if isempty(handles.domains(handles.state.activeDomain).boundaryID)
    handles.domains(handles.state.activeDomain).boundaryID = 0;
end

% newID is the index of the new boundary.
newID = handles.domains(handles.state.activeDomain).boundaryID + 1;
handles.domains(handles.state.activeDomain).boundaryID = newID;

% Get a default name for the new boundary.
name = sprintf('Boundary_#%d', newID);

% If an internal polygon is a rectangle or circle, then output an error
% message, and reset the algorithm to incremental.
switch handles.state.tool
    case 1
        type = 'Circle';
        if id > 1 && handles.state.algorithm == 3
            errordlg('Cannot use Advancing Front when any internal boundary is either a rectangle or a circle.','Advancing Front Error');
            uiwait;
            handles.state.algorithm = 1;
            set(handles.rbutton_Incremental,'Value',1.0)
        end
    case 2
        type = 'Rectangle';
        if id > 1 && handles.state.algorithm == 3
            errordlg('Cannot use Advancing Front when any internal boundary is either a rectangle or a circle.','Advancing Front Error');
            uiwait;
            handles.state.algorithm = 1;
            set(handles.rbutton_Incremental,'Value',1.0)
        end
    case 3
        type = 'Polygon';
end
% add the polygon to the polygon list
handles.domains(handles.state.activeDomain).boundaries(id) = struct( 'coor', coor, 'edge', edge, 'grad', grad, 'reverse', reverse,'name', name, 'type', type );

% sets the flags for the drawing
handles.domains(handles.state.activeDomain).showMesh = true;
handles.domains(handles.state.activeDomain).changed = true;

guidata(hObject, handles);

% change the active boundary of the selected domain to the new one
handles.domains(handles.state.activeDomain).activeBoundary = id;

% update the lists
updateLists(hObject, handles);
guidata(hObject, handles);

% redraws
redraw(handles);
end

function button_Delete_boundary_Callback(hObject, eventdata, handles)

% if there's no domain, there's nothing to delete
activeDomain = handles.state.activeDomain;
if activeDomain == 0
    return;
end

% number of boundaries of the selected domain
t = size(handles.domains(activeDomain).boundaries,2);

% if the domain has no boundaries, there's nothing to delete
if t < 1
    return;
end

% removes the boundary from the selected domain
handles.domains(handles.state.activeDomain).boundaries = handles.domains(handles.state.activeDomain).boundaries([1:handles.domains(handles.state.activeDomain).activeBoundary-1,handles.domains(handles.state.activeDomain).activeBoundary+1:end]);
guidata(hObject, handles);

% update the lists
updateLists(hObject,handles);
handles = guidata(hObject);

% redraws
redraw(handles);

end

function button_Edit_boundary_Callback(hObject, eventdata, handles)

% if the polygon list is empty, there's nothing to edit
if handles.state.activeDomain == 0 || size(handles.domains,2) < 1 || size(handles.domains(handles.state.activeDomain).boundaries,2) < 1
    return;
end

% redraws the view with edge numbers for the selected boundary to
% assign constraints
handles.state.showEdgesNumbers = 1;
guidata(hObject, handles);
redraw(handles);

% Prepare to call editBoundary.
constraints = handles.constraints;
edges = handles.domains(handles.state.activeDomain).boundaries;
activeBoundary = handles.domains(handles.state.activeDomain).activeBoundary;

% call the GUI to edit the boundary
res = editBoundary(edges, constraints, activeBoundary);

% save the result
handles.constraints = res.constraints;
handles.domains(handles.state.activeDomain).boundaries = res.boundary;

% redraws the normal view and update the lists
handles.state.showEdgesNumbers = 0;
guidata(hObject, handles);
updateLists(hObject,handles);
redraw(handles);

end

function list_Boundaries_Callback(hObject, eventdata, handles)

% change the selected boundary for the selected domain if it exists
if handles.state.activeDomain ~= 0
    handles.domains(handles.state.activeDomain).activeBoundary = get(hObject,'Value');

    guidata(hObject, handles);
    redraw(handles);
end
end

function list_Domains_Callback(hObject, eventdata, handles)

% change the selected domain and updates the lists
if handles.state.activeDomain ~= 0
    handles.state.activeDomain = get(hObject,'Value');

    guidata(hObject, handles);
    updateLists(hObject,handles);
    redraw(handles);
end
end

% These two functions set the background of the domain & boundary lists to white.
function list_Boundaries_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end

function list_Domains_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end

function updateAxes(handles)

hold on

% Set up the limits of each axis.
set(handles.axes_View,'XLim', handles.config.axes(1:2) );
set(handles.axes_View,'YLim', handles.config.axes(3:4) );

% Get the axis spacings.
dx = handles.config.grid(2);
dy = handles.config.grid(3);

% Set up the grid.
x = handles.config.axes(1):dx:handles.config.axes(2);
y = handles.config.axes(3):dy:handles.config.axes(4);
set(handles.axes_View,'XTick', x );
set(handles.axes_View,'YTick', y );

grid on
end

function updateLists(hObject,handles)

% get the number of domains
nbDomains = size(handles.domains,2);

% if no domains, set the set the lists empty and return
if nbDomains == 0
    set( handles.list_Boundaries, 'String', {} );
    set( handles.list_Domains, 'String', {} );
    handles.state.activeDomain = 0;

    guidata(hObject, handles);
    return;
end

% list of domains names
nld = strvcat( handles.domains.name );

% set the domains list with the names
set(handles.list_Domains, 'string', nld );

% if selected domain is > than the domains count, fix it (occurs when
% deleting the last domain of the list)
if handles.state.activeDomain  > nbDomains
    handles.state.activeDomain = nbDomains;
end

% gets the boundaries count from the selected domain
nbBoundaries = size(handles.domains(handles.state.activeDomain).boundaries, 2 );

% get the index of the selected boundary
activeBoundary = handles.domains(handles.state.activeDomain).activeBoundary;

% build the boundaries list
if nbBoundaries == 0
    nlb = {};
else
    nlb = strvcat( handles.domains(handles.state.activeDomain).boundaries.name );
end

% sets the boundaries list
set(handles.list_Boundaries, 'string', nlb );

% fix the selected boundary if the last boundary has been deleted
if activeBoundary > nbBoundaries
    handles.domains(handles.state.activeDomain).activeBoundary = nbBoundaries;
end

% set the list pointer to the selected domain/boundary
set(handles.list_Boundaries,'Value',handles.domains(handles.state.activeDomain).activeBoundary );
set(handles.list_Domains,'Value',handles.state.activeDomain );

% save
guidata(hObject, handles);

end

function redraw(handles)

% Added in the reset view commands in this next IF statement so that
% when pressing menu_file -> new, the axes refresh and you can no
% longer see the previous domain(s).  Similarly fixes refresh of axes if you
% delete a domain without first deleting the boundaries associated with
% it. NF

% if no domains, resets view and returns
if handles.state.activeDomain == 0
    % resets the view
    cla(handles.axes_View,'reset');
    updateAxes(handles);
    return;
end

% resets the view
cla(handles.axes_View,'reset');
updateAxes(handles);
hold on;

% for each domain
for i=1:size(handles.domains,2)

    % for each boundary of the domain
    if size(handles.domains(i).boundaries,2) > 0

        % case without the mesh, only the outline (when adding a new
        % boundary to the selected domain or when editing a boundary )
        if ~handles.domains(i).showMesh || ( handles.state.showEdgesNumbers && i == handles.state.activeDomain )

            % different colors if clockwise or counter clockwise
            col = ['b','r'];

            % show only the outline (with numbers if editing)
            for j=1:size(handles.domains(i).boundaries,2)
                displayEdgesArray( handles.domains(i).boundaries(j).edge, handles.domains(i).boundaries(j).coor, col(handles.domains(i).boundaries(j).reverse+1), handles.state.showEdgesNumbers && (j == handles.domains(i).activeBoundary));
            end
            % normal case
        else
            % if the current domain was edited, we need to regenerate
            % the mesh
            if handles.domains(i).changed
                % regenerate the mesh
                update_Mesh(handles, i );

                % reload the data
                handles = guidata(handles.list_Domains);
            end

            % shortcuts
            conn = handles.domains(i).mesh.conn;
            coor = handles.domains(i).mesh.coor;

            % show the triangles
            trisurf(conn',coor(1,:)',coor(2,:)',zeros(length(coor(1,:)),1));

        end
        if handles.state.showConstraintLabels && ~handles.state.showEdgesNumbers
            col = ['b','r'];

            for j = 1:size(handles.domains(i).boundaries,2)
                displayConstraintLabels( handles.domains(i).boundaries(j).edge, handles.domains(i).boundaries(j).coor,col(handles.domains(i).boundaries(j).reverse+1), handles.state.showConstraintLabels, handles.constraints)
            end
        end
        drawnow expose
    end
end

% drawing the outline of the current boundary in blue
if size(handles.domains(handles.state.activeDomain).boundaries,2) ~= 0
    displayEdgesArray( handles.domains(handles.state.activeDomain).boundaries(handles.domains(handles.state.activeDomain).activeBoundary).edge, handles.domains(handles.state.activeDomain).boundaries(handles.domains(handles.state.activeDomain).activeBoundary).coor, 'b', handles.state.showEdgesNumbers);
end

% save data
guidata(handles.axes_View, handles);

end

function panel_Algorithm_SelectionChangeFcn(hObject, eventdata)
% callback from the algorithm selector

% load the data
handles = guidata(hObject);

% Can't change the radio button if the solution is playing.
if handles.state.playing == 1
    switch handles.state.algorithm
        case 1
            set(handles.rbutton_Incremental,'Value', 1.0);
        case 2
            set(handles.rbutton_Delaunay,'Value',1.0);
        case 3
            set(handles.rbutton_Front,'Value',1.0)
    end
    guidata(hObject, handles);
    return
end

old = handles.state.algorithm;
problem = 0;
% change the algorithm using the Tag of the clicked radio button
switch get(eventdata.NewValue,'Tag')

    case 'rbutton_Incremental'
        handles.state.algorithm = 1;

    case 'rbutton_Delaunay'
        handles.state.algorithm = 2;

    case 'rbutton_Front'
        handles.state.algorithm = 3;

        % The advancing front algorithm won't work if any internal
        % boundaries are circles or squares.  But, before checking
        % that, we need to make sure that we have an active domain,
        % since the user might have switched the algorithm before
        % creating a domain.
        activeDomain = handles.state.activeDomain;
        if activeDomain > 0

            % Check if any internal boundaries are circles or squares,
            % and if there are, raise a flag.
            t = size(handles.domains(activeDomain).boundaries,2);
            if t >= 2
                for i = 2:t
                    if strcmp(handles.domains(activeDomain).boundaries(i).type,'Square') || strcmp(handles.domains(activeDomain).boundaries(i).type,'Circle')
                        problem = 1;
                    end
                    if problem == 1; break; end
                end
            end
        end
    otherwise
        return;
end

% If we had a problem with the internal boundaries, issue an error
% message and change the algorithm back to Internal.
if problem == 1
    errordlg('Cannot use Advancing Front when any internal boundary is either a rectangle or a circle.','Advancing Front Error');
    uiwait;
    handles.state.algorithm = 1;
    set(handles.rbutton_Incremental,'Value',1.0)
    guidata(hObject, handles)
    redraw(handles);
    return
end

% if algorithm didn't change, no need to regenerate/redraw
if handles.state.algorithm == old
    return;
end

% algorithm changed, need to regenerate the domains
for i=1:size(handles.domains,2)
    handles.domains(i).showMesh = true;
end

% save and redraw
guidata(hObject, handles);
redraw(handles);
end

function button_Algorithm_configure_Callback(hObject, eventdata, handles)

% prepare the algorithm configuration dialog for the current algorithm
switch handles.state.algorithm
    case 1 % incremental

        % Reworded the help text, and capitalized initial words.  JPW

        % Prepare help text.
        s = sprintf('"Mesh triangle size" (in graph units) controls the size of the triangles.\n\n"Distance between boundary nodes" (as a percentage of the length of the longest domain edge) sets the distance between one boundary node and the next.');
        set(handles.panel_helptext,'String',s);

        % Prepare prompt for parameter input.
        dlg_title = 'Incremental configuration';
        prompt = {'Mesh triangle size (graph units)', 'Distance between boundary nodes (% of longest domain edge)'};

        % Get the current (default) values for the mesh triangle size
        % and the distance between boundary nodes.  The triangle size
        % is stored as a string, and so has to be converted first.
        bspacedef = strread(handles.config.algorithms.incremental.bspace, '%*s %f', 'delimiter', '*');
        def = {num2str(bspacedef), num2str(handles.config.algorithms.incremental.boundary)};

        % removed some options of input and allowed change of the
        % boundary mesh size NF 09/10/08

    case 2 % delaunay
        s = sprintf('"Distance between boundary nodes" sets the distance between one boundary node and the next as a percentage of the length of the longest domain edge.\n\nThe domain mesh is created from the boundary mesh, and tends to be finer (smaller triangles) near the boundary.');
        set(handles.panel_helptext,'String',s);
        dlg_title = 'Delaunay configuration';
        prompt = {'Distance between boundary nodes (% of longest domain edge)'};
        def = {num2str(handles.config.algorithms.delaunay.boundary)};

    case 3 % advancing front
        s = sprintf('"Distance between boundary nodes" sets the distance between one boundary node and the next as a percentage of the length of the longest domain edge.\n\nThe domain mesh is created from the boundary mesh, and is of the same quality as the boundary mesh.');
        set(handles.panel_helptext,'String',s);
        dlg_title = 'Advancing front configuration';
        prompt = {'Distance between boundary nodes (% of longest domain edge)'};
        def = {num2str(handles.config.algorithms.front.boundary)};

end

% TODO maybe introduce some way of getting internal points, so that the
% weight options above can be replaced.  (and in the saving of them below) NF

% changed to one line from two. NF
num_lines = 1;

% configuration dialog
answer = inputdlg(prompt,dlg_title,num_lines,def);

% clears the help panel
set(handles.panel_helptext,'String','');
if size(answer,1) == 0
    return
end

% flag to know if the configuration has changed
changed = false;

switch handles.state.algorithm
    case 1 % incremental
        i=1;
        if str2double(char(answer{i})) <= 0
            errordlg(sprintf('"%s" is invalid\n using "%s" instead', char(answer(i)),char(def(i))), 'Invalid value' );
            uiwait;     % forces the GUI to wait until the error dialog box has gone before resuming
            % fixes the refresh bug after error. NF
            answer(i) = def(i);
        end
        i=2;
        if str2double(char(answer{i})) > 100 || str2double(char(answer{i})) <=0
            errordlg(sprintf('"%s" is invalid\n using "%s" instead', char(answer(i)),char(def(i))), 'Invalid value' );
            uiwait;     % forces the GUI to wait until the error dialog box has gone before resuming
            % fixes the refresh bug after error. NF
            answer(i) = def(i);
        end
    case 2 % delaunay
        i=1;
        if str2double(char(answer{i})) > 100 || str2double(char(answer{i})) <=0
            errordlg(sprintf('"%s" is invalid\n using "%s" instead', char(answer(i)),char(def(i))), 'Invalid value' );
            uiwait;     % forces the GUI to wait until the error dialog box has gone before resuming
            % fixes the refresh bug after error. NF
            answer(i) = def(i);
        end
    case 3 % advancing front
        i=1;
        if str2double(char(answer{i})) > 100 || str2double(char(answer{i})) <=0
            errordlg(sprintf('"%s" is invalid\n using "%s" instead', char(answer(i)),char(def(i))), 'Invalid value' );
            uiwait;     % forces the GUI to wait until the error dialog box has gone before resuming
            % fixes the refresh bug after error. NF
            answer(i) = def(i);
        end
end

% test the syntax of each configuration field
for i=1:size(answer,1)
    try
        switch handles.state.algorithm
            case 1 % incremental
                tempeval = eval(char(answer(i)));
                changed = true;
            case 2 % delaunay
                tempeval = eval(char(answer(i)));
                changed = true;
            case 3 % advancing front
                tempeval = eval(char(answer(i)));
                changed = true;
        end
    catch e
        errordlg(sprintf('"%s" is invalid\n using "%s" instead', char(answer(i)),char(def(i))), 'Invalid value' );
        uiwait;     % forces the GUI to wait until the error dialog box has gone before resuming
        % fixes the refresh bug after error. NF
        answer(i) = def(i);
    end
end
% neatened up the error dialogue. NF

% need to change the try above to check that the values are within the
% suggested bounds, for each case handle.state.algorithm

% TODO better error checks
if ~changed
    return;
end

% save the new configuration
switch handles.state.algorithm
    case 1  % incremental
        bspace = sprintf('%s%s','ones(1,nvb)*', char(answer(1)));
        handles.config.algorithms.incremental.bspace = char(bspace);
        handles.config.algorithms.incremental.boundary = str2double(char(answer(2)));

    case 2  % delaunay
        handles.config.algorithms.delaunay.boundary = str2double(char(answer(1)));

    case 3  % advancing front
        handles.config.algorithms.front.boundary = str2double(char(answer(1)));
end

% adjusted above three cases according to the change at start of function
% NF 09/10/08

% save the configuration
config_changed(handles);

% sets the changed flag to true for every domain, so the mesh will be
% re computed
for i=1:size(handles.domains,2)
    handles.domains(i).changed = true;
end

% save the data and redraw
guidata(hObject, handles);
redraw(handles);

end

function config_changed( handles )
% save the config
configFile = strcat(prefdir, '/meshGenConfig.xml');
save_config(handles.config, configFile);
end

function ret = updateBoundary( handles, idDomain )

% This function sets up the parameters for the boundary edges and calls the
% NAG Toolbox routine d06ba to generate the boundary mesh for domain number
% idDomain.

edge = [];
coor = [];
lcomp = [];
nbinter = 0;

for j=1:size(handles.domains(idDomain).boundaries,2)
    coor = [coor, handles.domains(idDomain).boundaries(j).coor];
    nedges = handles.domains(idDomain).boundaries(j).edge;
    nedges(1:2,:) = nedges(1:2,:) + size(edge,2);
    edge = [edge, nedges ];

    if handles.domains(idDomain).boundaries(j).reverse
        nbinter = nbinter+size(handles.domains(idDomain).boundaries(j).edge,2);
    end

    nlcomp(j) = size(handles.domains(idDomain).boundaries(j).edge,2);
    lcomp = [lcomp,size(edge,2)-nlcomp(j):size(edge,2)-1];
end

% the next bit of code works out the length of the longest edge of the
% polygon and then sets maxdist (the maximum distance between two nodes
% on the boundary mesh) to be a percentage (given by the user) of the
% longest edge.  This gets around changing the size of the grid and
% having to be careful how fine the boundary mesh is. NF 09/10/08
distance = zeros(size(edge,2));
for i=1:size(edge,2)
    distance(i) = sqrt(sum((coor( :,edge(1,i) ) - coor( :, edge(2,i) ) ).^2));
end
longestedge = max(max(distance));

switch handles.state.algorithm
    case 1 % incremental
        percentage = handles.config.algorithms.incremental.boundary;
        maxdist = longestedge * percentage / 100;

    case 2 % delaunay
        percentage = handles.config.algorithms.delaunay.boundary;
        maxdist = longestedge * percentage / 100;

    case 3 % advancing front
        percentage = handles.config.algorithms.front.boundary;
        maxdist = longestedge * percentage / 100;
end

lcomp = lcomp+1;

totnb = 0;
%preallocate for lined
lined = zeros(4,size(edge,2));
for i=1:size(edge,2)
    distance = sqrt(sum((coor( :,edge(1,i) ) - coor( :, edge(2,i) ) ).^2));
    nb = round(distance/maxdist)+1;

    % hopefully this if statement deals with boundary lengths being
    % much smaller than the maxdist. NF
    if nb == 1
        nb = 2;
    end

    % Pack the details of this edge into the lined array.  The fourth value
    % indicates the type of line segment joining the endpoints of the edge.
    % Here it should be 0, according to the documentation, as 0 indicates
    % that it is a straight line.  However it needs to be greater than zero
    % as later on in fem_solve.m this value is used and is assumed to be
    % non-zero. 24/06/09 NF
    lined(:,i) = [ nb ; edge(1,i); edge(2,i); i];

    totnb = totnb+nb;
end

% Store the coordinate and line data.
handles.domains(idDomain).bnd = struct('coor', coor, 'lined', lined, 'nlcomp', nlcomp, 'lcomp', lcomp );

% Call NAG Toolbox routine d06ba to generate the boundary mesh.  Note
% that the third parameter to d06ba is the name of the function which 
% describes the segments of the boundary (see d06ba documentation).  Since
% we've specified (using lined(4,i) = 0 above) that all of these segments 
% are straight lines, this function is never called, and we give its name 
% as d06bad (which is the name of a dummy function supplied with 
% the NAG library).
[nvb, coor, nedge, edge, user, ret] = d06ba(coor, nag_int(lined), 'd06bad', ...
    ones(2,1), lcomp*0+1, nag_int(nlcomp), nag_int(lcomp), nag_int(30000), ...
    nag_int(totnb), nag_int(0), 'nlines', nag_int(sum(nlcomp)));

% Store the details of the boundary mesh.
handles.domains(idDomain).boundary = struct('coor',coor(:,1:double(nvb)), 'edge', edge(:,1:double(nedge)), 'nbinter', nbinter );
guidata(handles.list_Domains, handles);

end

function ret = update_Mesh(handles, idDomain )

% This function sets up the parameters for the internal mesh and calls the
% appropriate NAG Toolbox routine to generate it from the boundary mesh
% for domain number idDomain.

% Generate the boundary mesh.
ret = updateBoundary(handles, idDomain);

% if something went wrong, return
if ret ~= 0
    return;
end

% Get the mesh data, as calculated by updateBoundary.
handles = guidata(handles.list_Domains);

% Get the edge and coordinates arrays.
edge = handles.domains(idDomain).boundary.edge(1:3,:);
coor = handles.domains(idDomain).boundary.coor;

% Expand the coordinates array.
ncoor = zeros(2,size(coor,2)*100);
ncoor(:,1:size(coor,2)) = coor;
coor = ncoor;

% Number of edges.
nvb = nag_int(size(edge,2));

% Number of interior points (this is only used for Delaunay and
% Advancing Front).
nbinter = nag_int(handles.domains(idDomain).boundary.nbinter);

% Set the NAG routine warning level to 'high' before calling it.
itrace = nag_int(1);

% Now call the internal meshing routine.
switch handles.state.algorithm

    case 1  % incremental
        [nv, nelt, coorOut, conn, ret] = d06aa(nag_int(edge), coor, ...
            eval(handles.config.algorithms.incremental.bspace), ...
            logical(handles.config.algorithms.incremental.smooth), ...
            itrace, 'coef',handles.config.algorithms.incremental.coeff, ...
            'power', handles.config.algorithms.incremental.power);

    case 2  % delaunay
        [nv, nelt, coorOut, conn, ret] = d06ab(nvb, nag_int(edge), coor, ...
            eval(handles.config.algorithms.delaunay.weight), ...
            nag_int(handles.config.algorithms.delaunay.npropa), itrace);

    case 3  % advancing front
        [nv, nelt, coorOut, conn, ret] = d06ac(nvb, nag_int(edge), coor, ...
            eval(handles.config.algorithms.front.weight), itrace);
end

% If there was an error in the meshing, return without modifying the
% old mesh.
if ret ~= 0
    % Issue a helpful warning message.  (Using msgbox here has a problem
    % with the text being overlaid by a random-looking bitmap.)
    set(handles.panel_helptext, 'String', 'Error generating updated internal mesh.  Try changing the algorithm''s parameters, or switching to a different algorithm.');
    return;
else
    % Otherwise, blank out the help box (in case the error message is still 
    % displayed from a previous attempt).
    set(handles.panel_helptext, 'String', '');
end

% Store the details of the internal mesh.
handles.domains(idDomain).mesh  = struct('coor',coorOut(:,1:double(nv)),'conn',conn(:,1:double(nelt)));
guidata(handles.list_Domains, handles );

end

function button_solve_poisson_Callback(hObject, eventdata, handles)

% if no domain exists, nothing to solve
if handles.state.activeDomain < 1
    return;
end
% put text in help panel
s = sprintf('The right hand side of the Poisson equation acts as a source term, analagous to a source of heat etc.\n\nTry inputting sin(x).*cos(y), or x.^(-1), or x.*y. or just a constant.');
set(handles.panel_helptext,'String',s);
% ask for right hand value
answer = inputdlg({'Enter right hand value:'},'Poisson equation',1,{handles.fem.poisson.source});

% if user pressed cancel returns
if isempty(answer)
    return;
end

% assign variables values for the syntax check
x=0;
y=0;
t=0;

% check the syntax of the expression and return if invalid
try
    eval(answer{:});
catch
    errordlg('Invalid value')
    return;
end

% stop the solution drawing
toolbar_tool_stop_ClickedCallback(hObject, eventdata, handles);

% save the equation settings
handles.fem.poisson.source = answer{:};
handles.fem.equation = 'poisson';
guidata(hObject, handles );

% call the solver
solve(handles);

end

function button_solve_laplace_Callback(hObject, eventdata, handles)

% put text into the help panel
set(handles.panel_helptext,'String','')

% if no domain exists, nothing to solve
if handles.state.activeDomain < 1
    return;
end

set(handles.panel_helptext,'String','')

% stop the solution drawing
toolbar_tool_stop_ClickedCallback(hObject, eventdata, handles);

% save the equation settings
handles.fem.equation = 'laplace';
guidata(hObject, handles );

% call the solver
solve(handles);

end

function solve(handles)

s = sprintf('The initial value is the starting value of the solution on the domain.\n\nTry inputting 0, or sin(3*x), y.^2 or some other function/constant');
set(handles.panel_helptext,'String',s);
% ask for initial value
answer = inputdlg({'Enter initial value:'},'Initialisation',1,{handles.fem.init});

% if user pressed cancel returns
if isempty(answer)
    set(handles.panel_helptext,'String','')
    return;
end

% assign variables values for the syntax check
x=0;
y=0;
t=0;

% check the syntax of the expression and return if invalid
try
    eval(answer{:});
catch
    errordlg('Invalid value')
    return;
end

handles.fem.init = answer{:};
guidata(handles.axes_View, handles );

% call the fem solver
handles = fem_solve(handles);

% save the solution
guidata(handles.axes_View,handles);

% display first time step solution
show_solutions(handles);

% Give some helpful advice
set(handles.panel_helptext,'String','Solution complete.  To view the results, press the "View solution" button on the toolbar above.')

end

function toolbar_tool_circle_ClickedCallback(hObject, eventdata, handles)
% changes the selected tool, saves it and updates the toolbar
handles.state.tool = 1;
guidata(hObject, handles );
%     updateToolbar(handles);
New_boundary(hObject, handles)
set(handles.toolbar_tool_circle,'state','off')
end

function toolbar_tool_quad_ClickedCallback(hObject, eventdata, handles)
% changes the selected tool, saves it and updates the toolbar
handles.state.tool = 2;
guidata(hObject, handles );
%     updateToolbar(handles);
New_boundary(hObject, handles)
set(handles.toolbar_tool_quad,'state','off')
end

function toolbar_tool_polyline_ClickedCallback(hObject, eventdata, handles)
% changes the selected tool, saves it and updates the toolbar
handles.state.tool = 3;
guidata(hObject, handles );
%     updateToolbar(handles);
New_boundary(hObject, handles)
set(handles.toolbar_tool_polyline,'state','off')
end

function toolbar_grid_ClickedCallback(hObject, eventdata, handles)

% copy the current config
b = handles.config.grid;

s = sprintf('"X spacing" and "Y spacing" set the distance between the grid lines on the axes.\n\nIf "magnetic grid" is enabled, then each newly added vertex of a boundary will be snapped to the nearest grid point.');
set(handles.panel_helptext,'String',s);
% call the setup grid GUI
handles.config.grid = setupGrid(handles.config.grid );

set(handles.panel_helptext,'String','')

% saves the config
guidata(hObject, handles );

% dirty way to check if the config has changed (count the differences
% between old and new config )
if nnz(b-handles.config.grid) ~= 0
    config_changed(handles);
    updateAxes(handles);
end
end

function toolbar_axes_ClickedCallback(hObject, eventdata, handles)

% copy the current config
b = handles.config.axes;

% call the setup axes GUI
handles.config.axes = axesConfiguration( handles.config.axes );

% saves the config
guidata(hObject, handles );

% if config changed, saves it and update GUI
if nnz( b - handles.config.axes ) ~= 0
    config_changed(handles);
    updateAxes(handles);
end
end

function updateToolbar(handles)
% update the state of tools toggle buttons, according to the state
% variables

% array of toolbar buttons structures
tools = [ handles.toolbar_tool_circle,handles.toolbar_tool_quad, handles.toolbar_tool_polyline ];

% set the state of all the tools to off
set(handles.toolbar_tool_quad, 'state','off' );
set(handles.toolbar_tool_polyline, 'state','off' );
set(handles.toolbar_tool_circle, 'state','off' );

% set the state of current tool to on
set( tools(handles.state.tool), 'state','on' );

end

function toolbar_resetview_ClickedCallback(hObject, eventdata, handles)
% reset the view to top view
updateAxes(handles);
view([0 90]);
end

function toolbar_tool_toggleMesh_ClickedCallback(hObject, eventdata, handles)
% toggle the mesh display flag of the solution

handles.config.solution(1) = strcmp(get(hObject, 'state'),'on');

% save the config
guidata(hObject, handles );
config_changed(handles);

end

function toolbar_tool_toggleGrid_ClickedCallback(hObject, eventdata, handles)
% toggle the grid display flag for the solution

handles.config.solution(2) = strcmp(get(hObject, 'state'),'on');

% save the config
guidata(hObject, handles );
config_changed(handles);
end

function toolbar_tool_toggleConstraintLabels_ClickedCallback(hObject, eventdata, handles)
% toggles the constraint labels flag for the boundaries of the domain

handles.state.showConstraintLabels = strcmp(get(hObject,'State'),'on');

guidata(hObject, handles);
redraw(handles)

end

%  These two call-back functions are needed so that MATLAB doesn't complain
%  that they are not there
function toolbar_tool_toggleConstraintLabels_OnCallback(hObject, eventdata, handles)

end
function toolbar_tool_toggleConstraintLabels_OffCallback(hObject, eventdata, handles)

end

% % Keep this function as we may be able to implement it later
%
% function toolbar_tool_toggleSmooth_ClickedCallback(hObject, eventdata, handles)
% % toggle the surface fitting flag
%
%     % dirty way to transform the state field from on/off to 1/0
%    3 handles.config.solution(3) = (length(get(hObject, 'state'))  == 2);
%
%     % save the config
%     guidata(hObject, handles );
%     config_changed(handles);
% end

function toolbar_colormap_ClickedCallback(hObject, eventdata, handles)
% call matlab colormap editor
colormapeditor;
end

function toolbar_tool_constraints_ClickedCallback(hObject, eventdata, handles)

% extracts the largest current constraint tag.
tagID = size(handles.constraints,2);

% call the constraints editor and save them
handles.constraints = editConditions(handles.constraints,0,tagID);
guidata(hObject, handles);

end

function toolbar_tool_simulation_ClickedCallback(hObject, eventdata, handles)

set(handles.panel_helptext,'String','The smaller T/dt is the faster the solution will be computed (depending on how fine the mesh is), and also how quickly the simulation will play back.')
% asks for the time parameters for the simulation
answer = inputdlg({'Final Time T','Time step dt'},'Time configuration',1,{num2str(handles.fem.time.T),num2str(handles.fem.time.dt)});
set(handles.panel_helptext,'String','');
% if cancel was pressed returns
if size(answer,1) == 0
    return;
end

% get the answers
T = str2double(answer(1));
dt = str2double(answer(2));

% checks if answers are correct
if T <= 0
    errordlg('Invalid T value');
    return;

elseif dt > T || dt <= 0
    errordlg('Invalid dt value');
    return;
end

% save the parameters
handles.fem.time = struct('T', T, 'dt', dt );
guidata(hObject, handles );

end

function toolbar_tool_play_ClickedCallback(hObject, eventdata, handles)

% if recording is in progress, dont change anything
if handles.state.recording == 1
    return;
end

% if the solution is not playing already, set the playing flag and
% display the solution
if handles.state.playing == 0;
    handles.state.playing = 1;
    guidata(hObject, handles);

    res = show_solutions(handles);
    if ~isstruct(res)
        if res == 0
            redraw(handles)
        end
    end
else
    return
end

handles.state.playing = 0;
guidata(hObject, handles)

% if there was an error showing solution, then res = 0

end

function toolbar_tool_pause_ClickedCallback(hObject, eventdata, handles)

% if the solution is being recorded, dont change anything
if handles.state.recording == 1
    return;
end

% if the solution is playing
if handles.state.playing == 1

    % current paused state
    old = handles.state.paused;

    % gets the new paused state and saves it
    handles.state.paused = (length(get(hObject, 'state'))  == 2);
    guidata(hObject, handles );

    % if the solution was paused before, continue the playing, if it wasnt
    % paused the display function will pause
    if old == 1
        uiresume(handles.Main);
    end
end
end

function toolbar_tool_stop_ClickedCallback(hObject, eventdata, handles)

% doesnt stop the recording
if handles.state.recording == 1
    return;
end

% if solution is playing, stops it
if handles.state.playing == 1
    handles.state.playing = 0;
    guidata(hObject, handles);
end
end

function toolbar_tool_record_ClickedCallback(hObject, eventdata, handles)

% if solution is playing, stops the playing
if handles.state.playing == 1

    % change the play flag to stop the video
    handles.state.playing = 0;
    guidata(hObject,handles);

    % if the video was paused, unpauses it so it quits
    if handles.state.paused == 1
        uiresume(handles.Main);
        handles.paused = 0;
    end
end

% asks for the video filename
[filename, pathname, filterindex] = uiputfile({'*.mat','Matlab videos (*.mat)'; '*.*',  'All Files (*.*)'}, 'Save as');
if isequal(filename,0) || isequal(pathname,0)
    return;
end

videofile = fullfile(pathname,filename);

% sets the recording flag on
handles.state.recording = 1;
guidata(hObject,  handles);

% call the display function
handles = show_solutions(handles);

% gets the video frames
ff = handles.frames;

% saves the movies, ancient samourai trick to use the matlab save
% function with a variable filename
savecmd = sprintf('save %s ff',videofile);
eval(savecmd)

% sets the recording flag off
handles.state.recording = 0;
guidata(hObject,  handles);
msgbox('Recording finished','Solution recording');

end



% This function is here to stop Matlab complaining that there isn't a
% function handle for the menu_file button.  It is empty as I don't know
% what might need putting in there at the moment. NF
% --------------------------------------------------------------------
function menu_File_Callback(hObject, eventdata, handles)
% hObject    handle to menu_File (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

end

% This function is here to stop Matlab complaining that there isn't a
% function handle for resizing the GUI window.  It is empty as I don't know
% what might need putting in there at the moment. NF
% --- Executes when Main is resized.
function Main_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to Main (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

end
